/*
   lpc.c - hardware access via the LPC47N252 sensor chip

   FanIO driver V2.x
   Copyright(c) 2001-2007 Christian Diefer
*/

static int deviceID=0;
static int revision=0;
static unsigned short base=0;
static unsigned short mboxPort=0;
static unsigned short maxFanNo=0;

static ULONG detectLPC( unsigned short cfg_base, unsigned char reg, PULONG pIOBuffer );
static ULONG writePortVal( unsigned short cfg_base, unsigned char reg );
//extern unsigned char inpbyte( unsigned short port );
//extern unsigned char outpbyte( unsigned short port, unsigned char val );

ULONG detectFanSensorLL( PULONG pIOBuffer )
{
  ULONG x;
  int logDev = 0;
  int ldn9Active = 0;
  int i;

  pIOBuffer[1] = 0;
  pIOBuffer[2] = 0xffffffff;
  pIOBuffer[3] = 0xffffffff;
  pIOBuffer[4] = 0xffffffff;
  pIOBuffer[5] = 0;
  pIOBuffer[6] = 0xffffffff;
  pIOBuffer[7] = 0xffffffff;
  pIOBuffer[8] = 0xffffffff;

  for( i=9; i<SENSOR_BUFSIZE; i++ )
    pIOBuffer[i] = 0xffffffff;

  base = 0x2e;

	x = detectLPC( 0x2e, 0x20, &pIOBuffer[1] );
  if( !x )
  {
    base = 0x4e;
    x = detectLPC( 0x4e, 0x20, &pIOBuffer[5] );
  }

  if( !x )
    return 0;

	/* select logical device 9 (LDN9) */
	outp( base, 0x07 );
	outp( base+1, 0x09 );

	/* check if LDN 9 was successfully selected */
	outp( base, 0x07 );
  logDev = inp( base+1 );
  pIOBuffer[9] = logDev;
  
  if( logDev != 9 )
    return 0;

  /* check if LDN 9 is active by reading LDN9 register 0x30 */
  outp( base, 0x30 );
  ldn9Active = inp( base+1 );
  pIOBuffer[10] = ldn9Active;
  
  // Bit 0 of Register 0x30 must be set!
  if( !(ldn9Active & 1) )
    return 0;
  
	/* get mailbox reg interface iobase via LDN9 register 0x60/61 */
	outp( base, 0x60 );
	mboxPort  = inp( base+1 ) << 8;
	outp( base, 0x61 );
	mboxPort |= inp( base+1 );
	pIOBuffer[11] = mboxPort;

  switch( deviceID )
  {
    case 0x0e:  maxFanNo = 2;      // LPC47n252 = three PWM controls
                break;
                
    case 0x28:  maxFanNo = 3;      // LPC47n250 = four PWM controls
                break;
                
    default:    maxFanNo = 0;      // unknown = no controls
  }  
  
  return -1;
}

ULONG getMaxFanNoLL()
{
  return maxFanNo;
}

ULONG setFanSpeedLL( USHORT fan, USHORT level )
{
  int val, speed, fanPort;
  ULONG t[SENSOR_BUFSIZE];
  
  if( mboxPort==0 && !detectFanSensorLL(t) )
    return 0;
    
  if( fan > maxFanNo )
    return 0;

  switch( fan )
  {
    case 0: fanPort = 0x92;
            break;
    
    case 1: fanPort = 0x93;
            break;
    
    case 2: fanPort = 0x95;
            break;
            
    case 3: fanPort = 0x98;
            break;
    
    default:return 0;
  }

  if( level > 63 )
    return 0;

  speed = 0x80|(level<<1);
    
  outp( mboxPort, fanPort );
  outp( mboxPort+1, speed );

  outp( mboxPort, fanPort );
  val = inp( mboxPort+1 );
  
  if( val != speed )
    return 0;
    
  return -1;  
}

ULONG getFanSpeedLL( USHORT fan )
{
  int fanPort, speed;
  ULONG t[SENSOR_BUFSIZE];
  
  if( mboxPort==0 && !detectFanSensorLL(t) )
    return -1;

  if( fan > maxFanNo )
    return -1;

  switch( fan )
  {
    case 0: fanPort = 0x92;
            break;
    
    case 1: fanPort = 0x93;
            break;
    
    case 2: fanPort = 0x95;
            break;
            
    case 3: fanPort = 0x98;
            break;
    
    default:return -1;
  }

  outp( mboxPort, fanPort );
  speed = inp( mboxPort+1 );

  return (speed&0x7e) >> 1;
}


/**
 * internal driver functions
 */
static ULONG detectLPC( unsigned short ioBase, unsigned char reg, PULONG pIOBuffer )
{
  int devID, exDevID, rev; 
  pIOBuffer[0] = ioBase;
  
  // Kommando 0xaa: Exit Configuration Mode
	outp( ioBase, 0xaa );
	if( inp(ioBase) == 0xaa )
		return 0;

	exDevID = inp( ioBase+1 );
	pIOBuffer[1] = exDevID;

  // Kommando 0x55: Enter Configuration Mode
	outp( ioBase, 0x55 );
	if( !writePortVal(ioBase,0x55) )
		return 0;

  // DeviceID ber Index 'reg' holen (=0x20)
	if( !writePortVal(ioBase,reg) )
		return 0;

	devID = inp( ioBase+1 );
	pIOBuffer[2] = deviceID;
	if( devID==0 || devID==0xff )
		return 0;

	// Revision holen ber Index 'reg'+1 (=0x21)
	if( !writePortVal(ioBase,(UCHAR)(reg+1)) )
		return 0;

	rev = inp( ioBase+1 );
  pIOBuffer[3] = revision;
	if( rev>=128 || devID==exDevID )
		return 0;

	if( devID==0x0e      // LPC47n252 mit Revision==1
	||  devID==0x28      // ??? mit Revision==3
   	)
	{
	  deviceID = devID;
	  revision = rev;
	  return -1;
  }
  
  return 0;
}

// low level functions
static ULONG writePortVal( unsigned short ioBase, unsigned char reg )
{
	outp( ioBase, reg );

	if( inp(ioBase) != reg )
		return 0;

	return -1;
}
