/*
   vbios.c - access to the video bios rom image

   FanIO driver V2.x
   Copyright(c) 2001-2006 Christian Diefer
*/

static PHYSICAL_ADDRESS vbiosPA;    // physische Adresse des VBIOS (0xc00000)

//--------------------------------------------------
// map Video BIOS into virtual address spaces
//--------------------------------------------------
void mapVBIOS( PULONG pIOBuffer )
{
  vbiosPA.LowPart = 0xc0000;
  vbiosPA.HighPart = 0;
  vbiosVA = (PUCHAR)MmMapIoSpace( vbiosPA, 0x10, MmNonCached );
  
  if( vbiosVA == NULL )
  {
    pIOBuffer[0] = 1;
    return;
  }

  if( vbiosVA[0]!=0x55 && vbiosVA[1]!=0xaa )
  {
    MmUnmapIoSpace( vbiosVA, 0x10 );
    pIOBuffer[0] = 2;
    return;
  }
  
  vbiosSize = (ULONG)vbiosVA[2];
  
  MmUnmapIoSpace( vbiosVA, 0x10 );
  
  vbiosPA.LowPart = 0xc0000;
  vbiosPA.HighPart = 0;
  vbiosVA = (PUCHAR)MmMapIoSpace( vbiosPA, vbiosSize*512, MmNonCached );
  
  if( vbiosVA==NULL )
  {
    pIOBuffer[0] = 3;
    return;
  }
  
  if( vbiosVA[0]!=0x55 && vbiosVA[1]!=0xaa )
  {
    MmUnmapIoSpace( vbiosVA, vbiosSize*512 );
    pIOBuffer[0] = 4;
    return;
  }

  pIOBuffer[0] = 0;
  pIOBuffer[3] = vbiosSize;
  
#ifdef _WIN64
  pIOBuffer[1] = (ULONG)(((ULONG64)vbiosVA)&0xffffffff);
  pIOBuffer[2] = (ULONG)(((ULONG64)vbiosVA)>>32);   // Highpart
#else
  pIOBuffer[1] = (ULONG)vbiosVA;
  pIOBuffer[2] = 0;
#endif

}

//--------------------------------------------------
// unmap Video BIOS address space
//--------------------------------------------------
void unmapVBIOS()
{
  if( vbiosVA != NULL )
    MmUnmapIoSpace( vbiosVA, vbiosSize );
}

//--------------------------------------------------
// get a Video BIOS page
//--------------------------------------------------
void getVBIOSChunk( PULONG pIOBuffer )
{
  ULONG pageToRetrieve = *pIOBuffer;
  int i=0;
  PUCHAR buf = (PUCHAR)pIOBuffer;
  
  if( pageToRetrieve >= vbiosSize )
    return;
    
  if( vbiosVA == NULL )
    return;

  for( i=0; i<512; i++ )
    buf[i] = vbiosVA[(pageToRetrieve*512)+i];
}

//--------------------------------------------------
