/*
   I8kfanGUI V3.1
   Copyright(c) 2001-2007 Christian Diefer

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as 
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "stdafx.h"
#include "cpuid.h"
#include "cpuspeed.h"
#include <math.h>
#include "cpudata.h"
#include "i8kfangui.h"
#include "TOptions.h"
#include "GlobalVars.h"
#include "i8kfanctrl.h"

// brauchen wir zum Zugriff auf das 'f_htt'-Flag.
extern TCPUData cpuData;

// interne Variablen:
static int clone_flag;             // Intel-clone?
static BYTE vendor_id[16] = "------------";
static DWORD family = 0;
static DWORD maxIDVal = 0;
static DWORD cpuffext = 0;
static DWORD brandid = 0;


// Globale Variablen: werden bei der Initialisierung ins Log geschrieben
//DWORD_PTR processMask;    // initiale Process affinity mask
//DWORD_PTR systemMask;     // System affinity mask

/***************************************************************
* wincpuidsupport()
*
* Input: nix
*
* Rckgabe:
*  <>0 = CPUID wird untersttzt
*  0 = CPUID wird nicht untersttzt
***************************************************************/
#ifndef _WIN64
WORD wincpuidsupport() 
{
  WORD cpuid_support = 1;

  _asm {
      pushad
      pushfd                  // EFLAGS holen
      pop eax
      mov ecx, eax
      xor eax, 200000h        // ID-Bit in EFLAGS umdrehen
      push eax                // neues EFLAGS auf dem Stack sichern
      popfd                   // neuen Wert einladen
      pushfd                  // ber den Stack...
      pop eax                 // ... in EAX kopieren
      xor eax, ecx            // kann das Bit nicht umdrehen...
      jnz support             // => Prozessor=80486
      mov cpuid_support,0     // also kein CPUID-Support
    support:
      popad
  }

  return cpuid_support;
}
#endif

/***************************************************************
* wincpuid()
*
* Inputs: none
*
* Returns:
*  0 = 8086/88
*  2 = 80286
*  3 = 80386
*  4 = 80486
*  5 = Pentium(R) Prozessor
*  6 = PentiumPro(R) Prozessor
*  7 oder hher: Pentium II oder hher
***************************************************************/
DWORD haveCPUID = 0;
WORD retCPUID;

WORD wincpuid() 
{
  if( haveCPUID )
    return retCPUID;

  WORD cpuid;

  if( wincpuidsupport() )     // wird CPUID untersttzt?
    cpuid = check_IDProc();
  else 
  {
    clone_flag = 0;
    cpuid = 3; // Annahme: 80386(!)
  }

  if( clone_flag )
    cpuid = cpuid | CLONE_MASK;

  haveCPUID = 1;
  retCPUID = cpuid;

  return cpuid;
}


/***************************************************************
* wincpufeatures()
*
* Input: nix
*
* Rckgabe:
*   0 = CPU ohne CPUID-Support: 8086, 8088, 80286, 80386 and einige 
*       ltere 80486-CPUs
*
*   ansonsten: Featureregister (EDX aus CPUID-Opcode)
***************************************************************/
DWORD haveFeat = 0;
DWORD retFeat;

DWORD wincpufeatures() 
{
  if( haveFeat )
    return retFeat;

  int i=0;
  DWORD cpuff=0x00000000;
  BYTE vendor_id[16] = "------------";
  BYTE intel_id[16] = "GenuineIntel";
  DWORD highestEAX=0;

  if( wincpuidsupport() ) 
  {
#ifdef _WIN64
    DWORD* vi = (DWORD*)vendor_id;
    ULONG x[4] = { 0,0,0,0 };
    readCPUID( &x[0], &x[1], &x[2], &x[3] );
    highestEAX = x[0];
    vi[0] = x[1];
    vi[1] = x[3];
    vi[2] = x[2];
#else
    _asm { 
        pushad
        xor eax,eax                       // CPUID mit EAX=0 aufrufen
        CPU_ID                            // VendorID kommt zurck mit hchstem EAX-Level
        mov dword ptr highestEAX,eax
        mov dword ptr vendor_id,ebx
        mov dword ptr vendor_id[+4],edx
        mov dword ptr vendor_id[+8],ecx
        popad
    }
#endif

    // nachschauen, ob's auch wirklich ein Intel ist: 'GenuineIntel'
    for( i=0; i<12; i++ )
    {
      if( !(vendor_id[i] == intel_id[i]) )
        clone_flag = 1;    // nix da, 'AuthenticAMD' oder so...
    }

#ifdef _WIN64
    if( highestEAX >= 1 )
    {
      ULONG x[4] = { 1,0,0,0 };
      readCPUID( &x[0], &x[1], &x[2], &x[3] );
      cpuff = x[3];
      cpuffext = x[2];
      brandid = x[1];
      family = x[0];
    }
#else
    _asm {
        pushad
        cmp dword ptr highestEAX,1        // EAX-Level 1 untersttzt ?
        jl end_cpuff            // nein, kann die CPU-Daten nicht erfragen
        xor eax, eax
        inc eax
        CPU_ID                  // family/model/stepping/features
        mov cpuff, edx
        mov cpuffext, ecx
        mov brandid, ebx
        mov family, eax

      end_cpuff:
        popad
    }
#endif
  }

  haveFeat = 1;
  retFeat = cpuff;

  return cpuff;
}


// Internal Private Functions //////////////////////////////////

/***************************************************************
* check_IDProc()
*
* Rckgabe:
*  CPU-Familie (4=486, 5=Pentium, 6=PPro, 7=P2 etc.
***************************************************************/
static WORD check_IDProc() 
{
  int i=0;
  WORD cpu_type=0xffff;
  BYTE stepping=0;
  BYTE model=0;
  BYTE intel_id[16] = "GenuineIntel";
  DWORD highestEAX=0;

#ifdef _WIN64
  DWORD* vi = (DWORD*)vendor_id;
  ULONG x[4] = { 0,0,0,0 };
  readCPUID( &x[0], &x[1], &x[2], &x[3] );
  maxIDVal = x[0];
  vi[0] = x[1];
  vi[1] = x[3];
  vi[2] = x[2];
  highestEAX = x[0];
#else
  _asm {     
        pushad
        xor eax,eax
        CPU_ID
        mov  maxIDVal, eax
        mov dword ptr vendor_id, ebx
        mov dword ptr vendor_id[+4], edx
        mov dword ptr vendor_id[+8], ecx
        mov dword ptr highestEAX,eax
        popad
  }
#endif

  for( i=0; i<12; i++ )
  {
    if( !(vendor_id[i] == intel_id[i]) )
      clone_flag = 1;    
  }

#ifdef _WIN64
  if( highestEAX >= 1 )
  {
    ULONG x[4] = { 1,0,0,0 };
    readCPUID( &x[0], &x[1], &x[2], &x[3] );
    stepping = (BYTE)(x[0] & 0x0f);
    model = (BYTE)((x[0] & 0xf0)>>4);
    cpu_type = (WORD)(((x[0]&0xf00) >> 8) & 0x0f);
  }
#else
  _asm {
        pushad
        cmp dword ptr highestEAX, 1 
        jl end_IDProc
        xor eax, eax
        inc eax
        CPU_ID

        mov stepping, al
        and stepping, 0x0f
        
        and al, 0f0h
        shr al, 4
        mov model, al
        
        and eax, 0f00h      // Family isolieren
        shr eax, 8
        and eax, 0fh
        mov cpu_type, ax    // _cpu_type mit Family belegen

    end_IDProc:
        popad
  }
#endif

  return cpu_type;
} // Check_IDProc()

#define EXTRACT_BITS_RL(the_val, bits_start, bits_len) ((the_val >> (bits_start - 1)) & ((1 << bits_len) - 1)) 

//-----------------------------------------------
// Anzahl der logischen CPUs ermitteln (max. 32!)
//-----------------------------------------------
size_t getLogicalCPUCount() 
{ 
  if( !GetProcessAffinityMask(OpenProcess(PROCESS_ALL_ACCESS,false,GetCurrentProcessId()),&glb_dProcessMask,&glb_dSystemMask) )
  {
    log( L"Can't retrieve process & system affinity data" );
    return 1;
  }

  int bitarray[32]; 
  size_t count=0; 

  for( size_t i=1; i<33; i++ ) 
  { 
     bitarray[i] = (int)EXTRACT_BITS_RL( glb_dSystemMask, i, 1 ); 
     if( bitarray[i] == 0 ) 
       return count; 
     
     count++; 
  }

  return 32;
}


#define HWD_MT_BIT         0x10000000     // EDX[28]  Bit 28 ist gesetzt, wenn HT oder Multi-Core untersttzt werden

#define NUM_LOGICAL_BITS   0x00FF0000     // EBX[23:16] Bits 16-23 in EBX enthalten die Anzahl der logischen CPUs
                                          // pro physischer CPU wenn CPUID mit EAX-Level 1 ausgefhrt wird

#define NUM_CORE_BITS      0xFC000000     // EAX[31:26] Bits 26-31 in EAX enthalten die Anzahl der Cores pro
                                          // physischer CPU - 1 wenn CPUID mit EAX-Level 4 ausgefhrt wird

#define INITIAL_APIC_ID_BITS  0xFF000000  // EBX[31:24] Bits 24-31 (8 Bits) geben die 8-Bit APIC-ID fr die CPU
                                          // zurck, auf der das CPUID ausgefhrt wurde
                                          
// Statusflags:
#define SINGLE_CORE_AND_HT_ENABLED		  1
#define SINGLE_CORE_AND_HT_DISABLED			2
#define SINGLE_CORE_AND_HT_NOT_CAPABLE	4
#define MULTI_CORE_AND_HT_NOT_CAPABLE		5
#define MULTI_CORE_AND_HT_ENABLED				6
#define MULTI_CORE_AND_HT_DISABLED			7
#define USER_CONFIG_ISSUE						    8


static UINT isCPUIDSupported( void );
static bool isGenuineIntel( void );
static bool isHTFlagSet( void );
static UINT maxLogicalProcPerPhysicalProc( void );
static UINT maxCorePerPhysicalProc( void );

#ifdef _WIN64
extern "C" UINT findMaskWidth( unsigned int );
#else
static UINT findMaskWidth( unsigned int );
#endif

static UCHAR getAPICID( void );
static UCHAR getNzbSubID( unsigned char, unsigned char, unsigned char );
static UCHAR getCPUCount( unsigned int *, unsigned int *, unsigned int * );

#include <windows.h>
#include <stdio.h>
#include <assert.h>

wchar_t g_s3Levels[2048];
wchar_t detectBuf[32768];


int testMCHT( void )
{
  *detectBuf = '\0';

  if( !cpuData.supportsCPUID || !cpuData.f_htt )
  {
    log( L"Single core cpu without HT feature detected" );
    return 0;
  }

  unsigned int TotAvailLogical = 0,  // Anzahl der logischen CPUs pro Core
			         TotAvailCore = 0,     // Anzahl der verfgbaren Cores pro physischer CPU
               PhysicalNum = 0;      // Anzahl physischer CPUs

  unsigned char StatusFlag = 0; 
  int MaxLPPerCore;

  if( isCPUIDSupported() < 4 ) // CPUID untersttzt nicht EAX-Level 4
	  log( L"CPUID EAX=4 is not supported or disabled" );

  StatusFlag = getCPUCount( &TotAvailLogical, &TotAvailCore, &PhysicalNum );
  
  if( USER_CONFIG_ISSUE == StatusFlag) 
  {
    log( L"***ERROR*** Not all logical processors in the system are enabled while running this process" );
    return -1;
  }

  CString s1, s2;

  switch( StatusFlag )
  {
   case MULTI_CORE_AND_HT_NOT_CAPABLE:
     s1.LoadStringW( IDS_MC1 );
	   _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_MC2 );
     _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_MC3 );
     _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_YES1 );
     s2.LoadStringW( IDS_NO1 );
     _tcscat_s( detectBuf, PhysicalNum>1 ? s1 : s2 );
	   break;

   case SINGLE_CORE_AND_HT_NOT_CAPABLE:
     s1.LoadStringW( IDS_MC1 );
	   _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_MC4 );
     _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_MC3 );
     _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_YES1 );
     s2.LoadStringW( IDS_NO1 );
     _tcscat_s( detectBuf, PhysicalNum>1 ? s1 : s2 );
	   break;

   case SINGLE_CORE_AND_HT_DISABLED:
     s1.LoadStringW( IDS_MC5 );
	   _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_MC4 );
     _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_MC3 );
     _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_YES1 );
     s2.LoadStringW( IDS_NO1 );
     _tcscat_s( detectBuf, PhysicalNum>1 ? s1 : s2 );
	   break;

   case SINGLE_CORE_AND_HT_ENABLED:
     s1.LoadStringW( IDS_MC6 );
	   _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_MC4 );
     _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_MC3 );
     _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_YES1 );
     s2.LoadStringW( IDS_NO1 );
     _tcscat_s( detectBuf, PhysicalNum>1 ? s1 : s2 );
	   break;

   case MULTI_CORE_AND_HT_DISABLED:
     s1.LoadStringW( IDS_MC5 );
	   _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_MC2 );
     _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_MC3 );
     _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_YES1 );
     s2.LoadStringW( IDS_NO1 );
     _tcscat_s( detectBuf, PhysicalNum>1 ? s1 : s2 );
	   break;

   case MULTI_CORE_AND_HT_ENABLED:
     s1.LoadStringW( IDS_MC6 );
	   _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_MC2 );
     _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_MC3 );
     _tcscat_s( detectBuf, s1 );
     s1.LoadStringW( IDS_YES1 );
     s2.LoadStringW( IDS_NO1 );
     _tcscat_s( detectBuf, PhysicalNum>1 ? s1 : s2 );
	   break;
  }

  s1.LoadStringW( IDS_MC7 );
  swprintf_s( &detectBuf[_tcslen(detectBuf)], 32768-_tcslen(detectBuf),
              s1, PhysicalNum, TotAvailCore, TotAvailLogical);

	MaxLPPerCore = maxLogicalProcPerPhysicalProc() / maxCorePerPhysicalProc();
  s1.LoadStringW( IDS_MC8 );
	swprintf_s( &detectBuf[_tcslen(detectBuf)], 32768-_tcslen(detectBuf),
              s1, maxCorePerPhysicalProc() );
  s1.LoadStringW( IDS_MC9 );
	swprintf_s( &detectBuf[_tcslen(detectBuf)], 32768-_tcslen(detectBuf),
              s1,  MaxLPPerCore );

  log( L"CPU HT/MC features:\r\n------------------------\r\n%s------------------------\r\n", detectBuf );

  if( PhysicalNum * maxCorePerPhysicalProc() > TotAvailCore ) 
    log( L"***ERROR*** Not all cores in the system are enabled for this application" );

  log( L"Logical cpu properties:\n%s", g_s3Levels );

  log( L"Target CPU values: AffinityMask:%d, PackageNo:%d, CoreNo:%d, SMTNo:%d",
       glb_dTargetAffinity, minPkgID, minCoreID /*maxCoreID*/, minSMTID );

	return 0;
}

//------------------------------------------------
// Rckgabe 0, wenn CPUID nicht untersttzt wird,
// ansonsten der maximale EAX-Level fr CPUID
//------------------------------------------------
static UINT isCPUIDSupported( void )
{
  UINT maxInputValue;

  try
  {
    maxInputValue = 0;

#ifdef _WIN64
    ULONG x[4] = { 0,0,0,0 };
    readCPUID( &x[0], &x[1], &x[2], &x[3] );
    maxInputValue = x[0];
#else
    __asm
    {
      xor eax, eax      // CPUID mit EAX=0 ausfhren
      cpuid
      mov maxInputValue, eax
    }
#endif
  } 
  catch (...)
  {
    return 0;        // CPUID nicht verfgbar
  }

  return maxInputValue;
}

//------------------------------------------------
// Rckgabe 0, wenn kein GenuineIntel
//------------------------------------------------
static bool isGenuineIntel( void )
{
	unsigned int VendorID[3] = {0, 0, 0};
	
  try
	{
#ifdef _WIN64
    ULONG x[4] = { 0,0,0,0 };
    readCPUID( &x[0], &x[1], &x[2], &x[3] );
    VendorID[0] = x[1];
    VendorID[1] = x[3];
    VendorID[2] = x[2];
#else
    __asm		
		{
      pushad
			xor eax, eax
      cpuid					// VendorID sichern
			mov VendorID, ebx
			mov VendorID + 4, edx
			mov VendorID + 8, ecx
      popad
		}
#endif
  }		
	catch (...)
	{
		return false;		             
	}

  // ist's ein GenuineIntel?
  return ( (VendorID[0] == 'uneG') &&
			     (VendorID[1] == 'Ieni') &&
			     (VendorID[2] == 'letn')); 
}

//------------------------------------------------
// Rckgabe der maximalen Anzahl Cores pro physischer CPU
//------------------------------------------------
static UINT maxCorePerPhysicalProc( void )
{
  unsigned int Regeax = 0;

  if( !isHTFlagSet() ) 
    return (unsigned int) 1;  // Single core

#ifdef _WIN64
  ULONG x[4] = { 0,0,0,0 };
  readCPUID( &x[0], &x[1], &x[2], &x[3] );
  if( x[0] >= 4 )     // EAX >= 4 (CPUID-Level >= 4 )
  {
    x[0] = 4;   // EAX=4
    x[2] = 0;   // ECX=0
    readCPUID( &x[0], &x[1], &x[2], &x[3] );
    Regeax = x[0];
  }
#else
  __asm
  {
    pushad
    xor eax, eax
    cpuid
    cmp eax, 4			  // EAX=4 untersttzt?
    jl single_core		// nein, also nur ein Core
    mov eax, 4			
    mov ecx, 0			  // Start mit Index=0, EAX-Level 4
    cpuid
    mov Regeax, eax
    jmp multi_core

single_core:
    xor eax, eax		

multi_core:
     popad			
  }
#endif

  return (unsigned int)((Regeax & NUM_CORE_BITS) >> 26)+1;
}


//------------------------------------------------
// Prft auf gesetztes HT-Bit mit Flagregister
//------------------------------------------------
static bool isHTFlagSet( void )
{
  unsigned int Regedx = 0;

  if( isCPUIDSupported()>=1 && isGenuineIntel() )
  {
#ifdef _WIN64
    ULONG x[4] = { 1,0,0,0 };
    readCPUID( &x[0], &x[1], &x[2], &x[3] );
    Regedx = x[3];
#else
    __asm
    {
      mov eax, 1
      cpuid
      mov Regedx, edx
     }		
#endif
  }

  return (Regedx & HWD_MT_BIT) ? true : false;   
}

//
// Function returns the maximum logical processors per physical package. Note that the number of 
// AVAILABLE logical processors per physical to be used by an application might be less than this
// maximum value.
//
static UINT maxLogicalProcPerPhysicalProc( void )
{
	unsigned int Regebx = 0;

	if( !isHTFlagSet() )
    return (unsigned int) 1;
	
#ifdef _WIN64
  ULONG x[4] = { 1,0,0,0 };
  readCPUID( &x[0], &x[1], &x[2], &x[3] );
  Regebx = x[1];
#else
  __asm
	{
		mov eax, 1
		cpuid
		mov Regebx, ebx
	}
#endif

  return (unsigned int) ((Regebx & NUM_LOGICAL_BITS) >> 16);
}


static UCHAR getAPICID( void )
{
	unsigned int Regebx = 0;

#ifdef _WIN64
  ULONG x[4] = { 1,0,0,0 };
  readCPUID( &x[0], &x[1], &x[2], &x[3] );
  Regebx = x[1];
#else
  __asm
	{
		mov eax, 1
		cpuid
		mov Regebx, ebx
	}
#endif

	return (unsigned char) ((Regebx & INITIAL_APIC_ID_BITS) >> 24);
}


//
// Determine the width of the bit field that can represent the value count_item. 
//
#ifndef _WIN64
static UINT findMaskWidth( UINT CountItem )
{
	UINT MaskWidth;
  UINT count = CountItem;

	__asm
	{
		mov eax, count
		mov ecx, 0
		mov MaskWidth, ecx
		dec eax
		bsr cx, ax
		jz next
		inc cx
		mov MaskWidth, ecx
next:
	}

	return MaskWidth;
}
#endif


//
// Extract the subset of bit field from the 8-bit value FullID.  It returns the 8-bit sub ID value
//
static UCHAR getNzbSubID( UCHAR FullID, UCHAR MaxSubIDValue, UCHAR ShiftCount )
{
	UINT MaskWidth;
	UCHAR MaskBits;

	MaskWidth = findMaskWidth((unsigned int) MaxSubIDValue);
	MaskBits  = (0xff << ShiftCount) ^ 
				((unsigned char) (0xff << (ShiftCount + MaskWidth)));

	return (FullID & MaskBits);
}

// Standardwerte fr Suche nach minimalster CPU-ID
UCHAR minPkgID = 100;
UCHAR minCoreID = 100;
UCHAR minSMTID = 100;

//
//
//
static UCHAR getCPUCount( UINT *TotAvailLogical, UINT *TotAvailCore, UINT *PhysicalNum )
{
	UCHAR StatusFlag = 0;
	UINT numLPEnabled = 0;
	DWORD dwAffinityMask;
	int j = 0, MaxLPPerCore;
	UCHAR apicID, PackageIDMask;
	UCHAR tblPkgID[256], tblCoreID[256], tblSMTID[256];
	wchar_t	tmp[256];
	DWORD_PTR dwProcessAffinity, dwSystemAffinity;
	
  g_s3Levels[0] = 0;
	*TotAvailCore = 1;
	*PhysicalNum  = 1;

  GetProcessAffinityMask( GetCurrentProcess(), &dwProcessAffinity, &dwSystemAffinity );

	if( dwProcessAffinity != dwSystemAffinity )  // nicht alle CPUs aktiviert
	{
		StatusFlag = USER_CONFIG_ISSUE;		
		return StatusFlag;
	}

	// Assume that cores within a package have the SAME number of 
	// logical processors.  Also, values returned by
	// MaxLogicalProcPerPhysicalProc and MaxCorePerPhysicalProc do not have
	// to be power of 2.

	MaxLPPerCore = maxLogicalProcPerPhysicalProc() / maxCorePerPhysicalProc();
	dwAffinityMask = 1;

	while( dwAffinityMask && dwAffinityMask<=dwSystemAffinity )
	{
		if( SetThreadAffinityMask(GetCurrentThread(),dwAffinityMask) )
		{
			Sleep( 0 );  // Ensure system to switch to the right CPU
			apicID = getAPICID();

			// Store SMT ID and core ID of each logical processor
			// Shift value for SMT ID is 0
			// Shift value for core ID is the mask width for maximum logical
			// processors per core

			tblSMTID[j]  = getNzbSubID( apicID, MaxLPPerCore, 0 );
			tblCoreID[j] = getNzbSubID( apicID, maxCorePerPhysicalProc(), (UCHAR)findMaskWidth(MaxLPPerCore) );

			// Extract package ID, assume single cluster.
			// Shift value is the mask width for max Logical per package

			PackageIDMask = (unsigned char) (0xff << findMaskWidth(maxLogicalProcPerPhysicalProc()));

			tblPkgID[j] = apicID & PackageIDMask;

      // minimale ID suchen und als targetAffinity verwenden:
      if( tblPkgID[j] < minPkgID )
      {
        minPkgID = tblPkgID[j];
        minCoreID = tblCoreID[j];
        minSMTID = tblSMTID[j];
        glb_dTargetAffinity = dwAffinityMask;
      }
      else if( tblPkgID[j]==minPkgID && tblCoreID[j]<minCoreID )
      {
        minCoreID = tblCoreID[j];
        minSMTID = tblSMTID[j];
        glb_dTargetAffinity = dwAffinityMask;
      }
      else if( tblPkgID[j]==minPkgID && tblCoreID[j]==minCoreID && tblSMTID[j]<minSMTID )
      {
        minSMTID = tblSMTID[j];
        glb_dTargetAffinity = dwAffinityMask;
      }

      CString s1;
      s1.LoadStringW( IDS_MC10 );
      swprintf_s( tmp, s1,
				              dwAffinityMask, 
                      apicID, 
                      tblPkgID[j], 
                      tblCoreID[j], 
                      tblSMTID[j] );

			_tcscat_s( g_s3Levels, tmp );

			numLPEnabled ++;   // Number of available logical processors in the system.
		}

		j++;
		dwAffinityMask = 1 << j;
	}

  // restore the affinity setting to its original state
  SetThreadAffinityMask( GetCurrentThread(), dwProcessAffinity );
	Sleep( 0 );
	
  *TotAvailLogical = numLPEnabled;
	
	//
	// Count available cores (TotAvailCore) in the system
	//
	unsigned char CoreIDBucket[256];
	DWORD ProcessorMask, pCoreMask[256];
	unsigned int i, ProcessorNum;

	CoreIDBucket[0] = tblPkgID[0] | tblCoreID[0];
	ProcessorMask = 1;
	pCoreMask[0] = ProcessorMask;

	for( ProcessorNum=1; ProcessorNum<numLPEnabled; ProcessorNum++ )
	{
		ProcessorMask <<= 1;
		for( i=0; i<*TotAvailCore; i++ )
		{
			// Comparing bit-fields of logical processors residing in different packages
			// Assuming the bit-masks are the same on all processors in the system.
			if( (tblPkgID[ProcessorNum] | tblCoreID[ProcessorNum]) == CoreIDBucket[i] )
			{
				pCoreMask[i] |= ProcessorMask;
				break;
			}

		}

		if( i == *TotAvailCore )   // did not match any bucket.  Start a new one.
		{
			CoreIDBucket[i] = tblPkgID[ProcessorNum] | tblCoreID[ProcessorNum];
			pCoreMask[i] = ProcessorMask;

			(*TotAvailCore)++;	// Number of available cores in the system
		}
	}


	// Physische CPUs im System zhlen (PhysicalNum)
	unsigned char PackageIDBucket[256];
	DWORD pPackageMask[256];

	PackageIDBucket[0] = tblPkgID[0];
	ProcessorMask = 1;
	pPackageMask[0] = ProcessorMask;

	for( ProcessorNum=1; ProcessorNum<numLPEnabled; ProcessorNum++ )
	{
		ProcessorMask <<= 1;
		for( i=0; i<*PhysicalNum; i++ )
		{
			// Comparing bit-fields of logical processors residing in different packages
			// Assuming the bit-masks are the same on all processors in the system.
			if( tblPkgID[ProcessorNum] == PackageIDBucket[i] )
			{
				pPackageMask[i] |= ProcessorMask;
				break;
			}
		}

		if( i == *PhysicalNum )
		{
			PackageIDBucket[i] = tblPkgID[ProcessorNum];
			pPackageMask[i] = ProcessorMask;

			(*PhysicalNum)++;	// Gesamtanzahl physischer CPUs
		}
	}

  // Prfung auf Multicore und Hyperthreading...
  if( *TotAvailCore > *PhysicalNum ) 
  {
	  // Multi-core
	  if( MaxLPPerCore == 1 )
		  StatusFlag = MULTI_CORE_AND_HT_NOT_CAPABLE;
	  else if( numLPEnabled > *TotAvailCore )
		  StatusFlag = MULTI_CORE_AND_HT_ENABLED;
		else 
      StatusFlag = MULTI_CORE_AND_HT_DISABLED;

  }
  else 
  {
	  // Single-core
	  if( MaxLPPerCore == 1 )
		  StatusFlag = SINGLE_CORE_AND_HT_NOT_CAPABLE;
	  else if( numLPEnabled > *TotAvailCore )
		  StatusFlag = SINGLE_CORE_AND_HT_ENABLED;
		else 
      StatusFlag = SINGLE_CORE_AND_HT_DISABLED;
  }

  return StatusFlag;
}
