/*
   I8kfanGUI V3.1
   Copyright(c) 2001-2007 Christian Diefer

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as 
   published by the Free Software Foundation.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#define _UNICODE

#include "stdafx.h"
#include "CoreTemp.h"
#include "cpuid.h"
#include "i8kfanctrl.h"
#include "TOptions.h"
#include "GlobalVars.h"

extern TOptions options;

//------------------------------------------------------------------------
// Prfen, ob die CPU, auf der dieser Thread luft, DTS untersttzt.
// Falls es irgendeinmal Multi-CPU-Notebooks gibt, wird hier angenommen,
// da nur CPUs des selben Typs verbaut sind. Dann ist es egal, auf
// welcher CPU dieser Test zur Laufzeit ausgefhrt wird.
//------------------------------------------------------------------------
bool hasIntelDTSSupport( void )
{
  BYTE realVendorID[16];
  BYTE intelVendorID[16] = { 'G', 'e', 'n', 'u', 'i', 'n', 'e', 'I', 'n', 't', 'e', 'l', '\0' };
  wchar_t vendorID[16] = L"";
  DWORD	maxIDVal, id6Val;

  // untersttzt diese CPU CPUID?
  if( !wincpuidsupport() )
    return false;

  // CPUID Level 0 ausfhren, um VendorID und minimalen CPUID-Level zu erhalten
#ifdef _WIN64
  DWORD* vi = (DWORD*)realVendorID;
  ULONG x[4] = { 0,0,0,0 };
  readCPUID( &x[0], &x[1], &x[2], &x[3] );
  vi[0] = x[1];
  vi[1] = x[3];
  vi[2] = x[2];
  maxIDVal = x[0];
#else
  __asm	{
    pushad
    xor eax,eax
    cpuid
    mov maxIDVal,eax
    mov dword ptr realVendorID,ebx
    mov dword ptr realVendorID+4,edx
    mov dword ptr realVendorID+8,ecx
    popad
  }
#endif

  // korrekte VendorID?
  if( memcmp(realVendorID,intelVendorID,12) )
    return false;

  // wird mindestens Level 6 untersttzt?
  if( maxIDVal < 6 )
    return false;

  // CPUID Level 6 ausfhren
#ifdef _WIN64
    x[0]=6; x[1]=x[2]=x[3]=0;
    readCPUID( &x[0], &x[1], &x[2], &x[3] );
    id6Val = x[0];
#else
  __asm	{
    mov eax,6
    cpuid
    mov	id6Val,eax
  }
#endif

  // wird DTS untersttzt?
  if( !(id6Val & 1) )
    return false;

  // Lesen des MSR 0xEE um TJunction zu ermitteln (85C oder 100C)
  DWORD msrEELow=0, msrEEHigh=0;

  if( !readMSR(0xee,msrEELow,msrEEHigh,glb_hHndFile) )
    return false;

  if( msrEELow & 0x40000000 )		// Bit 30 in MSR 0xee gesetzt => 85C
    glb_intelDTSTJunction = 85;
  else								// sonst 100C
    glb_intelDTSTJunction = 100;

  return true;
}

//----------------------------------------------------------------------
// Alle Cores durchlaufen und die Temperaturen ermitteln. Wenn mehr
// als zwei Cores vorhanden sind, werden die beiden hchsten
// Core-Temperaturen zurckgegeben. Wenn exakt 2 Cores vorhanden sind,
// werden die Temperaturen der Cores in der richtigen Reihenfolge
// in die globalen Vars geschrieben. Der Rckgabewert der Funktion
// ist die hchste Temperatur aller Cores. Diese Temperatur wird
// dann fr die Trayicons, den Peak-Wert und die Temperaturkontrolle
// verwendet.
//----------------------------------------------------------------------
int readCoreTemps( void )
{
  int h1=0, h2=0, t1, t2;

  for( int i=0; i<glb_iLogCPUNum; i++ )
  {
    if( i != 0 )
    {
      // auf anderen Core schalten:
      SetProcessAffinityMask( GetCurrentProcess(), glb_iCPUAffinityMask[i] );
      SetThreadAffinityMask( GetCurrentThread(), glb_iCPUAffinityMask[i] );   
    }

    int temp = intelDTSReadTemp();

    if( temp > h1 )   // die zwei hchsten Temps merken
    {
      h2 = h1;
      h1 = temp;
    }

    if( i == 0 )
      t1 = temp;
    else if( i == 1 )
      t2 = temp;
  }

  // auf 1. Core zurckschalten:
  if( !SetProcessAffinityMask(GetCurrentProcess(),glb_dTargetAffinity)
  ||  !SetThreadAffinityMask(GetCurrentThread(),glb_dTargetAffinity) )
  {
    DWORD err = GetLastError();
    log( L"Failed to switch back to logical CPU#1; Errorcode:%d", err );
  }

  if( glb_iLogCPUNum == 2 )   // genau zwei Cores => die entsprechenden Temps zurckgeben
  {
    glb_iCPUTempC1 = t1;
    glb_iCPUTempC2 = t2;
  }
  else        // mehr als 2 Cores => die beiden hchsten Temps zurckgeben
  {
    glb_iCPUTempC1 = h1;
    glb_iCPUTempC2 = h2;
  }

  int t = glb_iCPUTempC1;
  if( options.fahrenHeit )
    t = options.convertTemp( glb_iCPUTempC1, FAHRENHEIT );
  glb_szCPUTempC1.Format( L"%d", t );

  t = glb_iCPUTempC2;
  if( options.fahrenHeit )
    t = options.convertTemp( glb_iCPUTempC2, FAHRENHEIT );
  glb_szCPUTempC2.Format( L"%d", t );

  return h1;    // die hchste Temperatur als globale Temperatur zurckgeben
}

//------------------------------------------------------------------
// Ermittelt die Temperatur des Cores, auf dem dieser Thread luft.
// Dazu wird ein MSR ber den fanio-Treiber gelesen.
//------------------------------------------------------------------
int intelDTSReadTemp( void )
{
  DWORD a,b;

  if( !readMSR(0x19c,a,b,glb_hHndFile) )    // MSR_IA32_THERM_STATUS auslesen
    return -1;

  if( !(a & 0x80000000) )  // 'Valid'-Flag ist nicht gesetzt
    return -1;

  return glb_intelDTSTJunction - ((a >> 16) & 0x7f);
}
