/*
   I8kfanGUI V3.1
   Copyright(c) 2001-2007 Christian Diefer
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as 
   published by the Free Software Foundation.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "stdafx.h"
#include "i8kfangui.h"
#include "CPUData.h"
#include "cpuid.h"
#include "cpuspeed.h"
#include "setupapi.h"
#include "I8kfanctrl.h"

void TCPUData::checkCPUType()
{
  init();
  getCPUType();

  *flagText = '\0';
  CString s1;
  s1.LoadStringW( IDS_CDAT1 );
  swprintf_s( flagText, s1, cpuStr );

  wchar_t vendorUC[16];
  char2uc( vendorUC, vendorID );
  s1.LoadStringW( IDS_CDAT2 );
  swprintf_s( &flagText[_tcslen(flagText)], 65535-_tcslen(flagText), s1, vendorUC );
  s1.LoadStringW( IDS_CDAT3 );
  if( coreType != NULL )
    swprintf_s( &flagText[_tcslen(flagText)], 65535-_tcslen(flagText), s1, coreType );
  s1.LoadStringW( IDS_CDAT4 );
  if( structureSize != NULL )
    swprintf_s( &flagText[_tcslen(flagText)], 65535-_tcslen(flagText), s1, structureSize );

  if( supportsCPUID && maxIDVal>=2 )
    readCacheInfo();
  else if( cacheSize != NULL )
  {
    s1.LoadStringW( IDS_CDAT5 );
    swprintf_s( &flagText[_tcslen(flagText)], 65535-_tcslen(flagText), s1, cacheSize );
  }

  if( supportsCPUID && vendor==0 && maxIDVal>5 )
  {
    DWORD val;

#ifdef _WIN64
    ULONG x[4] = { 6,0,0,0 };
    readCPUID( &x[0], &x[1], &x[2], &x[3] );
    val = x[0];
#else
    __asm {
      pushad
      mov eax,6
      CPU_ID
      mov val,eax
      popad
    }
#endif

    if( val & 1 )
      f_dts = TRUE;    
  }

  if( *detectBuf )
  {
    Message( L"" );
    _tcscat_s( flagText, 65535-_tcslen(flagText), detectBuf );
  }

  s1.LoadStringW( IDS_CDAT6 );
  swprintf_s( &flagText[_tcslen(flagText)], 65535-_tcslen(flagText), s1, familyid );
  s1.LoadStringW( IDS_CDAT7 );
  swprintf_s( &flagText[_tcslen(flagText)], 65535-_tcslen(flagText), s1, modelid );
  s1.LoadStringW( IDS_CDAT8 );
  swprintf_s( &flagText[_tcslen(flagText)], 65535-_tcslen(flagText), s1, steppingid );
  s1.LoadStringW( IDS_CDAT9 );
  swprintf_s( &flagText[_tcslen(flagText)], 65535-_tcslen(flagText), s1, typid );
  s1.LoadStringW( IDS_CDAT10 );
  swprintf_s( &flagText[_tcslen(flagText)], 65535-_tcslen(flagText), s1, brandID&255 );
  s1.LoadStringW( IDS_CDAT11 );
  swprintf_s( &flagText[_tcslen(flagText)], 65535-_tcslen(flagText), s1, brandStr );

  s1.LoadStringW( IDS_CDAT12 );
  CString yes, no;
  yes.LoadStringW( IDS_YES );
  no.LoadStringW( IDS_NO );
  swprintf_s( &flagText[_tcslen(flagText)], 65535-_tcslen(flagText), s1, (supportsCPUID ? yes : no) );
  s1.LoadStringW( IDS_CDAT13 );
  swprintf_s( &flagText[_tcslen(flagText)], 65535-_tcslen(flagText), s1, maxIDVal );
  s1.LoadStringW( IDS_CDAT14 );
  if( exlevel >= 0x80000000 )
    swprintf_s( &flagText[_tcslen(flagText)], 65535-_tcslen(flagText), s1, exlevel );

  s1.LoadStringW( IDS_CDAT15 );
  swprintf_s( &flagText[_tcslen(flagText)], 65535-_tcslen(flagText), s1 );

  if( f_fpu )   Message( L"  FPU   - Floating Point Unit" );
  if( f_vme )   Message( L"  VME   - V86 Mode Extensions" );
  if( f_de )    Message( L"  DE    - Debug Extensions" );
  if( f_pse )   Message( L"  PSE   - Page Size Extensions" );
  if( f_tsc )   Message( L"  TSC   - Timestamp Counter" );
  if( f_msr )   Message( L"  MSR   - Model Specific Registers" );
  if( f_pae )   Message( L"  PAE   - Physical Address Extensions" );
  if( f_mce )   Message( L"  MCE   - Machine Check Exception" );
  if( f_c8x )   Message( L"  CX8   - CMPXCHG8B Instruction" );
  if( f_apic )  Message( L"  APIC  - Local APIC" );
  if( f_sep )   Message( L"  SEP   - Fast System Call" );
  if( f_mtrr )  Message( L"  MTRR  - Memory Type Range Registers" );
  if( f_pge )   Message( L"  PGE   - Page Global Enable" );
  if( f_mca )   Message( L"  MCA   - Machine Check Architecture" );
  if( f_cmov )  Message( L"  CMOV  - CMOV Instructions" );
  if( f_pse36 ) Message( L"  PSE36 - 36bit Page Size Extensions" );
  if( f_psn )   Message( L"  PSN   - Processor Serial Number" );
  if( f_mmx )   Message( L"  MMX   - MMX Instructions" );

  switch( vendor )
  {
    case 0: //m_szManufacturer = "Intel";
            if( f_acpi ) Message( L"  ACPI  - ACPI Support" );
            if( f_pat )  Message( L"  PAT   - Page Attribute Table" );
            if( f_fxsr ) Message( L"  FXSR  - FXSAVE/FXRSTOR Instructions" );
            if( f_sse )  Message( L"  SSE   - SSE Instructions" );
            if( f_sse2 ) Message( L"  SSE2  - SSE2 Instructions" );
            if( f_ss )   Message( L"  SS    - Self Snoop" );
            if( f_htt )  Message( L"  HTMC  - Hyperthreading/Multicore Technology" );
            if( f_tm1 )  Message( L"  TM1   - Thermal Monitor 1" );
            if( f_pbe )  Message( L"  PBE   - Pending break enable" );
            if( f_ia64 ) Message( L"  IA64  - IA64 Instructions" );
            if( f_sse3 ) Message( L"  SSE3  - SSE3 Instructions" );
            if( f_mon )  Message( L"  MMW   - MONITOR/MWAIT" );
            if( f_cpl )  Message( L"  CPL   - CPL qualified debug store" );
            if( f_vmx )  Message( L"  VMX   - Virtual machine extensions" );
            if( f_est )  Message( L"  EST   - Enhanced Intel speed step" );
            if( f_tm2 )  Message( L"  TM2   - Thermal Monitor 2" );
            if( f_l1id ) Message( L"  L1ID  - L1 context ID" );
            if( f_c16b ) Message( L"  C16X  - CMPXCHG16B support" );
            if( f_xtpr ) Message( L"  xTPR  - xTPR update control" );
            if( f_em64t )Message( L"  EM64T - Intel 64 Bit technology" );
            if( f_dts )  Message( L"  DTS   - Digital Thermal Sensor support" );
            break;

    case 1: //m_szManufacturer = "AMD";
            if( f_fsc )      Message( L"  FSC   - Fast System Call" );
            if( f_fcmov )    Message( L"  FCMOV - FCMOV Instruction" );
            if( f_mmxext )   Message( L"  MMXE  - Extended MMX Instructions" );
            if( f_sse )      Message( L"  SSE   - SSE Instructions" );
            if( f_sse2 )     Message( L"  SSE2  - SSE2 Instructions" );
            if( f_3dnowext ) Message( L"  3DNE  - Extended 3DNow! Instructions" );
            if( f_3dnow )    Message( L"  3DNOW - 3DNow! Instructions" );
            break;

    case 2: //m_szManufacturer = "Cyrix";
            if( f_cxmmx ) Message( L"  CXMMX - Cyrix MMX Instructions" );
            break;

    case 3: //m_szManufacturer = "Transmeta";
            break;

    case 4: //m_szManufacturer = "Centaur";
            if( f_3dnow ) Message( L"  3DNOW - 3DNow! Instructions" );
            break;

    case 5: //m_szManufacturer = "Rise";
            break;

    case 6: //m_szManufacturer = "UMC";
            break;

    case 7: //m_szManufacturer = "NexGen";
            break;

    default://m_szManufacturer = "Unknown";
            break;
  }

  if( supportsCPUID )
  {
    ULONG idvals[4];
    CString s1;
    s1.LoadStringW( IDS_CDAT16 );

    for( int i=0; i<=maxIDVal; i++ )
    {
      readCPUIDLevel( i, 0, idvals );
      swprintf_s( &flagText[_tcslen(flagText)], 65535-_tcslen(flagText), 
                  s1, 
                  i, idvals[0], idvals[1], idvals[2], idvals[3] );
    }

    if( exlevel >= 0x80000000 )
    {
      swprintf_s( &flagText[_tcslen(flagText)], 65535-_tcslen(flagText), 
                  L"\r\n" );

      s1.LoadStringW( IDS_CDAT16a );

      for( unsigned i=0x80000000; i<=exlevel; i++ )
      {       
        readCPUIDLevel( i, 0, idvals );
        swprintf_s( &flagText[_tcslen(flagText)], 65535-_tcslen(flagText), 
                    s1, 
                    i, idvals[0], idvals[1], idvals[2], idvals[3] );
      }
    }
  }
}

void TCPUData::init()
{
  features = 0;
  exfeatures = 0;
  typid = 0;
  modelid = 0;
  steppingid = 0;
  *vendorID = '\0';
  vendor = -1;
  maxIDVal = 0;
  exlevel = 0;
  brandID = 0;
  exfeatures2 = 0;
  brandStr = L"";
  cacheSize = NULL;
  structureSize = NULL;
  coreType = NULL;

  f_fpu = f_vme = f_de = f_pse = f_tsc = f_msr = f_pae = f_mce = f_c8x = f_apic = f_sep = 
  f_mtrr = f_pge = f_mca = f_cmov = f_pse36 = f_psn = f_mmx = f_pat = f_fxsr = f_sse = f_sse2 = 
  f_fsc = f_fcmov = f_mmxext = f_3dnowext = f_3dnow = f_cxmmx = f_ss = f_htt = f_tm1 = f_ia64 = 
  f_acpi = f_pbe = f_sse3 = f_mon = f_cpl = f_vmx = f_est = f_tm2 = f_l1id = f_c16b = f_em64t = 
  f_xtpr = f_dts = FALSE;
}

void TCPUData::getCPUType()
{
  cpuType = wincpuid();
  supportsCPUID = wincpuidsupport();

  switch( cpuType & (CLONE_MASK-1) )
  {
    case 0: cpuStr = L"8086";
            break;

    case 1: cpuStr = L"80186";
            break;

    case 2: cpuStr = L"80286";
            break;

    case 3: cpuStr = L"80386";
            break;

    case 4: cpuStr = L"80486";
            break;

    case 5: cpuStr = L"Pentium";
            break;

    case 6: cpuStr = L"Pentium Pro";
            break;

    case 15:cpuStr = L"Pentium 4";
            break;

    default:cpuStr = L"Unknown";
            break;
  }

  familyid = cpuType & (CLONE_MASK-1);

  if( !supportsCPUID )
    return;

  getCPUFeatures();

  typid = (family & (3<<12))>>12;
  familyid = (family&(15<<8))>>8;
  modelid = (family&(15<<4))>>4;
  steppingid = family&15;

  f_fpu = features & 1;
  f_vme = features & (1<<1);
  f_de = features & (1<<2);
  f_pse = features & (1<<3);
  f_tsc = features & (1<<4);
  f_msr = features & (1<<5);
  f_pae = features & (1<<6);
  f_mce = features & (1<<7);
  f_c8x = features & (1<<8);
  f_apic = features & (1<<9);
  f_sep = features & (1<<11);
  f_mtrr = features & (1<<12);
  f_pge = features & 8192;
  f_mca = features & 16384;
  f_cmov = features & 32768;
  f_pse36 = features & (1<<17);
  f_psn = features & (1<<18);
  f_mmx = features & (1<<23);

  if( !_stricmp((char*)vendorID,"GenuineIntel") )
  {
    vendor = 0;   // Intel
    f_pat = features & (1<<16);
    f_acpi = features & (1<<22);
    f_fxsr = features & (1<<24);
    f_sse = features & (1<<25);
    f_sse2 = features & (1<<26);
    f_ss = features & (1<<27);
    f_htt = features & (1<<28);
    f_tm1 = features & (1<<29);
    f_ia64 = features & (1<<30);
    f_pbe = features & (1<<31);
    f_sse3 = exfeatures & 1;
    f_mon = exfeatures & (1<<3);
    f_cpl = exfeatures & (1<<4);
    f_vmx = exfeatures & (1<<5);
    f_est = exfeatures & (1<<7);
    f_tm2 = exfeatures & (1<<8);
    f_l1id = exfeatures & (1<<10);
    f_c16b = exfeatures & (1<<13);
    f_xtpr = exfeatures & (1<<14);
    identifyIntel();
  }
  else if( !_stricmp((char*)vendorID,"AuthenticAMD") )
  {
    vendor = 1;   // AMD
    f_fsc = features & (1<<10);
    f_fcmov = features & (1<<16);
    f_mmxext = features & (1<<22);
    f_sse = features & (1<<25);
    f_sse2 = features & (1<<26);
//    f_3dnowext = features & (1<<30);
//    f_3dnow = features & (1<<31);
    identifyAMD();
  }
  else if( !_stricmp((char*)vendorID,"CyrixInstead") )
  {
    vendor = 2;   // Cyrix
    f_cxmmx = features & (1<<24);
  }
  else if( !_stricmp((char*)vendorID,"GenuineTMx86") )
  {
    vendor = 3;   // Transmeta
  }
  else if( !_stricmp((char*)vendorID,"CentaurHauls") )
  {
    vendor = 4;   // Centaur
    f_3dnow = features & (1<<31);
  }
  else if( !_stricmp((char*)vendorID,"RISERISERISE") )
  {
    vendor = 5;   // Rise
  }
  else if( !_stricmp((char*)vendorID,"UMC UMC UMC ") )
  {
    vendor = 6;   // UMC
  }
  else if( !_stricmp((char*)vendorID,"NexGenDriven") )
  {
    vendor = 7;   // NexGen
  }

  DWORD exl;

#ifdef _WIN64
  ULONG x[4] = { 0x80000000,0,0,0 };
  readCPUID( &x[0], &x[1], &x[2], &x[3] );
  exl = x[0];
#else
  __asm {
    pushad
    mov eax,0x80000000
    CPU_ID
    mov dword ptr exl,eax
    popad
  }
#endif

  exlevel = exl;

  if( exlevel >= 0x80000001 ) //&& vendor==1 )  // nur bei AMD
  {
    DWORD amdflags;

#ifdef _WIN64
    ULONG x[4] = { 0x80000001,0,0,0 };
    readCPUID( &x[0], &x[1], &x[2], &x[3] );
    amdflags = x[3];
#else
    __asm {
      pushad
      mov eax, 0x80000001
      CPU_ID
      mov dword ptr amdflags,edx
      popad
    }
#endif

    exfeatures2 = amdflags;

    if( vendor == 0 )
    {
      f_em64t = exfeatures2 & (1<<29);
    }

    if( vendor == 1 )
    {
      f_3dnow = amdflags & (1<<31);
      f_3dnowext = amdflags & (1<<30);
    }

  }

  if( exlevel >= 0x80000004 )
  {
    DWORD pstr[16];

#ifdef _WIN64
    ULONG x[4] = { 0x80000002,0,0,0 };
    readCPUID( &x[0], &x[1], &x[2], &x[3] );
    pstr[0] = x[0];
    pstr[1] = x[1];
    pstr[2] = x[2];
    pstr[3] = x[3];
    x[0] = 0x80000003;
    readCPUID( &x[0], &x[1], &x[2], &x[3] );
    pstr[4] = x[0];
    pstr[5] = x[1];
    pstr[6] = x[2];
    pstr[7] = x[3];
    x[0] = 0x80000004;
    readCPUID( &x[0], &x[1], &x[2], &x[3] );
    pstr[8] = x[0];
    pstr[9] = x[1];
    pstr[10] = x[2];
    pstr[11] = x[3];
    pstr[12] = 0;
#else
    __asm {
      pushad
      mov eax, 0x80000002
      CPU_ID
      mov dword ptr pstr, eax
      mov dword ptr pstr+4, ebx
      mov dword ptr pstr+8, ecx
      mov dword ptr pstr+12, edx
      mov eax, 0x80000003
      CPU_ID
      mov dword ptr pstr+16, eax
      mov dword ptr pstr+20, ebx
      mov dword ptr pstr+24, ecx
      mov dword ptr pstr+28, edx
      mov eax, 0x80000004
      CPU_ID
      mov dword ptr pstr+32, eax
      mov dword ptr pstr+36, ebx
      mov dword ptr pstr+40, ecx
      mov dword ptr pstr+44, edx

      xor eax,eax
      mov dword ptr pstr+48, eax

      popad
    }
#endif

    wchar_t text[64];
    char2uc( text, (char*)pstr );
    cpuStr = (char*)pstr;
  }
}

void TCPUData::identifyIntel()
{
  boolean noFamily = false;
  wchar_t* cpuString = NULL;

  switch( familyid )
  {
    case 4: switch( modelid )
            {
              case 0: cpuString = L"80486DX-25/33";
                      break;

              case 1: cpuString = L"80486DX-50";
                      break;

              case 2: cpuString = L"80486SX";
                      break;

              case 3: cpuString = L"80487/DX2/DX2 Overdrive";
                      break;

              case 4: cpuString = L"80486SL";
                      break;

              case 5: cpuString = L"80486SX2";
                      break;

              case 6: cpuString = L"80486DX (write-back)";
                      break;

              case 7: cpuString = L"80486DX4/DX4 Overdrive";
                      break;

              case 8: cpuString = L"80486DX4";
                      break;

              case 9: cpuString = L"80486DX4 (write-back)";
                      break;

              default:cpuString = L"Unknown 80486";
                      noFamily = true;
                      break;
            }
            break;

    case 5: switch( modelid )
            {
              case 1: if( typid == 1 )
                        cpuString = L"Pentium OverDrive (60,66)";
                      else
                        cpuString = L"Pentium (60,66)";
                      break;

              case 2: if( typid == 1 )
                        cpuString = L"Pentium (75,90,100,120,133,150,166,200)";
                      else
                        cpuString = L"Pentium OverDrive (75,90,100,120,133,150,166,200)";
                      break;

              case 3: cpuString = L"Pentium Overdrive (for 80486)";
                      break;

              case 4: if( typid == 1 )
                        cpuString = L"Pentium OverDrive MMC (75,90,100,120,133)";
                      else
                        cpuString = L"Pentium MMX (166,200)";
                      break;

              case 7: cpuString = L"P54C";
                      break;

              case 8: cpuString = L"P55C";
                      structureSize = L"0.25m";
                      break;

              default:cpuString = L"Unknown Pentium";
                      noFamily = true;
                      break;
            }
            break;

    case 6: switch( modelid )
            {
              case 0: cpuString = L"Pentium Pro (A-Step)";
                      break;

              case 1: cpuString = L"Pentium Pro";
                      break;

              case 3: if( typid == 1 )
                      {
                        cpuString = L"Pentium II OverDrive";
                        structureSize = L"0.28m";
                      }
                      else
                      {
                        cpuString = L"Intel Pentium II";
                        coreType = L"Klamath";
                        structureSize = L"0.28m";
                      }
                      break;

              case 5: cpuString = L"Intel Pentium II/II Xeon/Celeron";
                      structureSize = L"0.25m";
                      break;

              case 6: cpuString = L"Intel Pentium II/Celeron w/ on-die-L2";
                      break;

              case 7: cpuString = L"Intel Pentium III/III Xeon";
                      structureSize = L"0.25m";
                      break;
 
              case 8: cpuString = L"Intel Pentium III/III Xeon/Celeron";
                      coreType = L"Coppermine";
                      structureSize = L"0.18m";
                      cacheSize = L"256 KB";
                      break;

              case 9: cpuString = L"Intel Pentium-M";
                      structureSize = L"0.13m";
                      coreType = L"Banias";
                      cacheSize = L"1 MB";
                      break;

              case 10:cpuString = L"Intel Pentium III/PIII Xeon";
                      structureSize = L"0.18m";
                      cacheSize = L"2 MB";
                      coreType = L"Coppermine";
                      break;

              case 11:cpuString = L"Intel Pentium III";
                      structureSize = L"0.13m";
                      cacheSize = L"512 KB";
                      coreType = L"Tualatin";
                      break;

              case 13:if( steppingid != 8 )
                      {
                        cpuString = L"Intel Pentium-M";
                        structureSize = L"0.09m";
                        cacheSize = L"2 MB";
                        coreType = L"Dothan";
                      }
                      else
                      {
                        cpuString = L"Intel Pentium-M";
                        structureSize = L"0.09m";
                        cacheSize = L"2 MB";
                        coreType = L"Sonoma";
                      }
                      break;

              case 14:cpuString = L"Intel Pentium-M";
                      structureSize = L"65nm";
                      cacheSize = L"2 MB";
                      coreType = L"Yonah/Core Duo";
                      break;

              case 15:cpuString = L"Intel Pentium-M";
                      structureSize = L"65nm";
                      cacheSize = L"4 MB";
                      coreType = L"Core 2";
                      break;

              default:cpuString = L"Unknown Pentium Pro/II/III/M";
                      noFamily = true;
                      break;
            }
            break;

    case 15:switch( modelid )
            {
              case 0: cpuString = L"Intel Pentium 4";
                      structureSize = L"0.18m";
                      coreType = L"Willamette";
                      break;

              case 1: cpuString = L"Intel Pentium 4";
                      structureSize = L"0.18m";
                      coreType = L"Willamette";
                      break;

              case 2: cpuString = L"Intel Pentium 4";
                      structureSize = L"0.13m";
                      cacheSize = L"512 KB";
                      coreType = L"Northwood";
                      break;

              case 3: cpuString = L"Intel Pentium 4";
                      structureSize = L"0.09m";
                      cacheSize = L"1 MB";
                      coreType = L"Prescott";
                      break;

              case 4: cpuString = L"Intel Pentium 4 Xeon";
                      structureSize = L"0.09m";
                      cacheSize = L"1 MB";
                      coreType = L"Prescott";
                      break;

              case 6: cpuString = L"Intel Pentium 4";
                      structureSize = L"65nm";
                      cacheSize = L"2 MB";
                      break;

              default:cpuString = L"Unknown Pentium 4";
                      noFamily = true;
                      break;
            }
            break;

    default:cpuString = L"Unknown Intel CPU";
            noFamily = true;
            break;
  }

  cpuStr = cpuString;

  wchar_t* bstr = L"";

  switch( brandID & 255 )
  {
    case 1: bstr = L"Intel Celeron";
            structureSize = L"0.18m";
            coreType = L"Coppermine";
            break;

    case 2: bstr = L"Intel Pentium III";
            structureSize = L"0.18m";
            coreType = L"Coppermine";
            break;

    case 3: bstr = L"Intel Celeron/Pentium III Xeon";
            structureSize = L"0.18m";
            coreType = L"Coppermine";
            break;

    case 4: bstr = L"Intel Pentium III";
            structureSize = L"0.13m";
            coreType = L"Tualatin";
            break;

    case 6: bstr = L"Intel Mobile Pentium III-M";
            structureSize = L"0.13m";
            coreType = L"Tualatin";
            break;

    case 7: bstr = L"Intel Mobile Celeron";
            structureSize = L"0.13m";
            coreType = L"Tualatin";
            break;

    case 8: if( family == 0xf24 )
            {
              bstr = L"Intel Mobile Celeron 4";
              structureSize = L"0.13m";
              coreType = L"Northwood";
            }
            else
            {
              bstr = L"Intel Pentium 4";
              structureSize = L"0.18m";
              coreType = L"Willamette";
            }
            break;

    case 9: bstr = L"Intel Pentium 4";
            structureSize = L"0.13m";
            coreType = L"Northwood";
            break;

    case 10:bstr = L"Intel Celeron 4";
            structureSize = L"0.18m";
            coreType = L"Willamette";
            break;

    case 11:if( familyid==15 && modelid==2 )
            {
              bstr = L"Intel Pentium 4 Xeon";
              structureSize = L"0.13m";
              coreType = L"Northwood";
            }
            else
            {
              bstr = L"Intel Pentium 4 Xeon";
              structureSize = L"0.18m";
              coreType = L"Willamette";
            }
            break;

    case 12:bstr = L"Intel Pentium 4 Xeon MP";
            structureSize = L"0.13m";
            coreType = L"Northwood";
            break;

    case 14:bstr = L"Intel Mobile Pentium 4-M";
            structureSize = L"0.13m";
            coreType = L"Northwood";
            cacheSize = L"512 KB";
            break;

    case 15:if( family == 0xf27 )
              bstr = L"Intel Mobile Celeron 4";
            else
              bstr = L"Intel Mobile Pentium 4-M";
            structureSize = L"0.13m";
            coreType = L"Northwood";
            break;

    case 22:if( familyid==6 && modelid==9 )
            {
              bstr = L"Intel Pentium-M";
              structureSize = L"0.13m";
              cacheSize = L"1 MB";
              coreType = L"Banias";
            }
            else
            {
              bstr = L"Intel Pentium-M";
              structureSize = L"0.09m";
              cacheSize = L"2 MB";
              coreType = L"Dothan";
            }
            break;
  }

  if( *bstr )
  {
    cpuStr = bstr;
    brandStr = bstr;
  }
  else
    brandStr = cpuString;
}

void TCPUData::identifyAMD()
{
  switch( familyid )
  {
    case 4: switch( modelid )
            {
              case 3: cpuStr = L"486DX/2";
                      break;

              case 7: cpuStr = L"486DX/2-WB";
                      break;

              case 8: cpuStr = L"486DX/4";
                      break;

              case 9: cpuStr = L"486DX/4-WB";
                      break;

              case 14:cpuStr = L"Am5x86-WT";
                      break;

              case 15:cpuStr = L"Am5x86-WB";
                      break;

              default:cpuStr = L"Unknown 486/5x86 CPU";
                      break;
            }
            break;

    case 5: switch( modelid )
            {
              case 0: cpuStr = L"K5/SSA5 (PR75...PR100)";
                      break;

              case 1: cpuStr = L"K5 (PR120/PR133)";
                      break;

              case 2: cpuStr = L"K5 (PR150/PR166)";
                      break;

              case 3: cpuStr = L"K5 (PR200)";
                      break;

              case 6: cpuStr = L"K6 (166...233MHz)";
                      break;

              case 7: cpuStr = L"K6 (266...300MHz)";
                      break;

              case 8: cpuStr = L"K6-2 w/ 3DNow!";
                      break;

              case 9: cpuStr = L"K6-3 w/ 3DNow!";
                      break;

              case 13:cpuStr = L"K6-2+ / K6-III+ w/ 3DNow!";
                      break;

              default:cpuStr = L"Unknown K6 CPU";
                      break;
            }
            break;

    case 6: switch( modelid )
            {
              case 0:
              case 1: cpuStr = L"Athlon (25um)";
                      break;

              case 2: cpuStr = L"Athlon (18um)";
                      break;

              case 3: cpuStr = L"Duron";
                      break;

              case 4: cpuStr = L"Athlon (Thunderbird)";
                      break;

              case 6: cpuStr = L"Athlon (Palomino)";
                      break;

              case 7: cpuStr = L"Duron (Morgan)";
                      break;

              default:cpuStr = L"Unknown Athlon";
                      break;
            }
            break;

    default:cpuStr = L"Unknown AMD CPU";
            break;
  }
}

void TCPUData::identifyCyrix()
{
  switch( familyid )
  {
    case 4: switch( modelid )
            {
              case 4: cpuStr = L"MediaGX";
                      break;

              default:cpuStr = L"Unknown Cyrix CPU";
                      break;
            }
            break;

    case 5: switch( modelid )
            {
              case 2: cpuStr = L"6x86/6x86L";
                      break;

              case 4: cpuStr = L"MediaGX MMX enh.";
                      break;

              default:cpuStr = L"Unknown Cyrix CPU";
                      break;
            }
            break;
     
    case 6: switch( modelid )
            {
              case 0: cpuStr = L"m II";
                      break;

              case 5: cpuStr = L"VIA Cyrix M2 core";
                      break;

              case 6: cpuStr = L"WinChip C5A";
                      break;

              case 7: cpuStr = L"WinChip C5B";
                      break;

              default:cpuStr = L"Unknown Cyrix CPU";
                      break;
            }
            break;

    default:cpuStr = L"Unknown Cyrix CPU";
            break;
  }
}

void TCPUData::getCPUFeatures()
{
  DWORD cpuff = 0;
  DWORD cpuffext = 0;
  DWORD bid = 0;
  DWORD fam = 0;
  BYTE vid[16] = "------------";
  DWORD maxl = 0;

#ifdef _WIN64
    DWORD* vi = (DWORD*)vid;
    ULONG x[4] = { 0,0,0,0 };
    readCPUID( &x[0], &x[1], &x[2], &x[3] );
    vi[0] = x[1];
    vi[1] = x[3];
    vi[2] = x[2];
    maxl = x[0];

    if( maxl > 0 )
    {
      x[0] = 1;
      readCPUID( &x[0], &x[1], &x[2], &x[3] );
      cpuff = x[3];
      cpuffext = x[2];
      bid = x[1];
      fam = x[0];
    }
#else
  __asm {      
      pushad
      xor eax,eax
      CPU_ID
      mov dword ptr vid,ebx
      mov dword ptr vid[+4],edx
      mov dword ptr vid[+8],ecx
      mov dword ptr maxl,eax

      cmp eax,1
      jl end_cpuff
      xor eax,eax
      inc eax
      CPU_ID
      mov cpuff,edx
      mov cpuffext,ecx
      mov bid,ebx
      mov fam,eax

    end_cpuff:
      popad
  }
#endif

  strcpy_s( vendorID, (char*)vid );
  family = fam;
  brandID = bid;
  features = cpuff;
  exfeatures = cpuffext;
  maxIDVal = maxl;
}

/////////////////////////////////////////////////////////////////////////////
void TCPUData::Message( LPCTSTR lpszMessage )
{
  _tcscat_s( flagText, 16483-_tcslen(flagText), lpszMessage );
  _tcscat_s( flagText, 16383-_tcslen(flagText), L"\r\n" );
}

void TCPUData::readCPUIDLevel( unsigned level, unsigned level2, ULONG* target )
{
  ULONG x[4];

#ifdef _WIN64
  x[0] = level;
  x[1] = 0;
  x[2] = level2;
  x[3] = 0;
  readCPUID( &x[0], &x[1], &x[2], &x[3] );
#else
  __asm {      
      pushad
      mov eax,level
      mov ecx,level2
      CPU_ID
      mov dword ptr x,eax
      mov dword ptr x[+4],ebx
      mov dword ptr x[+8],ecx
      mov dword ptr x[+12],edx
      popad
  }
#endif

  target[0] = x[0];
  target[1] = x[1];
  target[2] = x[2];
  target[3] = x[3];
}

static CACHEID cids[] = 
{
  { 0x06, L"  L1 instruction cache: 8 KBytes, 4-way set associative, 32 byte line size" },
  { 0x08, L"  L1 instruction cache: 16 KBytes, 4-way set associative, 32 byte line size" },
  { 0x0a, L"  L1 data cache: 8 KBytes, 2-way set associative, 32 byte line size" },
  { 0x0c, L"  L1 data cache: 16 KBytes, 4-way set associative, 32 byte line size" },
  { 0x22, L"  L3 cache: 512 KBytes, 4-way set associative, 64 byte line size, 2 lines per sector" },
  { 0x23, L"  L3 cache: 1 MBytes, 8-way set associative, 64 byte line size, 2 lines per sector" },
  { 0x25, L"  L3 cache: 2 MBytes, 8-way set associative, 64 byte line size, 2 lines per sector" },
  { 0x29, L"  L3 cache: 4 MBytes, 8-way set associative, 64 byte line size, 2 lines per sector" },
  { 0x2c, L"  L1 data cache: 32 KBytes, 8-way set associative, 64 byte line size" },
  { 0x30, L"  L1 instruction cache: 32 KBytes, 8-way set associative, 64 byte line size" },
  { 0x41, L"  L2 cache: 128 KBytes, 4-way set associative, 32 byte line size" },
  { 0x42, L"  L2 cache: 256 KBytes, 4-way set associative, 32 byte line size" },
  { 0x43, L"  L2 cache: 512 KBytes, 4-way set associative, 32 byte line size" },
  { 0x44, L"  L2 cache: 1 MByte, 4-way set associative, 32 byte line size" },
  { 0x45, L"  L2 cache: 2 MByte, 4-way set associative, 32 byte line size" },
  { 0x46, L"  L3 cache: 4 MByte, 4-way set associative, 64 byte line size" },
  { 0x47, L"  L3 cache: 8 MByte, 8-way set associative, 64 byte line size" },
  { 0x49, L"  L2 cache: 4 MByte, 16-way set associative, 64 byte line size" },
  { 0x60, L"  L1 data cache: 16 KByte, 8-way set associative, 64 byte line size" },
  { 0x66, L"  L1 data cache: 8 KByte, 4-way set associative, 64 byte line size" },
  { 0x67, L"  L1 data cache: 16 KByte, 4-way set associative, 64 byte line size" },
  { 0x68, L"  L1 data cache: 32 KByte, 4-way set associative, 64 byte line size" },
  { 0x70, L"  Trace cache: 12K-op, 8-way set associative" },
  { 0x71, L"  Trace cache: 16K-op, 8-way set associative" },
  { 0x72, L"  Trace cache: 32K-op, 8-way set associative" },
  { 0x78, L"  L2 cache: 1 MByte, 4-way set associative, 64byte line size" },
  { 0x79, L"  L2 cache: 128 KByte, 8-way set associative, 64 byte line size, 2 lines per sector" },
  { 0x7a, L"  L2 cache: 256 KByte, 8-way set associative, 64 byte line size, 2 lines per sector" },
  { 0x7b, L"  L2 cache: 512 KByte, 8-way set associative, 64 byte line size, 2 lines per sector" },
  { 0x7c, L"  L2 cache: 1 MByte, 8-way set associative, 64 byte line size, 2 lines per sector" },
  { 0x7d, L"  L2 cache: 2 MByte, 8-way set associative, 64byte line size" },
  { 0x7f, L"  L2 cache: 512 KByte, 2-way set associative, 64-byte line size" },
  { 0x82, L"  L2 cache: 256 KByte, 8-way set associative, 32 byte line size" },
  { 0x83, L"  L2 cache: 512 KByte, 8-way set associative, 32 byte line size" },
  { 0x84, L"  L2 cache: 1 MByte, 8-way set associative, 32 byte line size" },
  { 0x85, L"  L2 cache: 2 MByte, 8-way set associative, 32 byte line size" },
  { 0x86, L"  L2 cache: 512 KByte, 4-way set associative, 64 byte line size" },
  { 0x87, L"  L2 cache: 1 MByte, 8-way set associative, 64 byte line size" },
  { 0x00, NULL }
};

void TCPUData::readCacheInfo()
{
  ULONG cid[4];

  CString s1;
  s1.LoadStringW( IDS_CDAT17 );
  Message( s1 );
  readCPUIDLevel( 2, 0, cid );

  for( int i=0; i<4; i++ )
  {
    // MSB darf nicht gesetzt sein (EAX, EBX, ECX, EDX)
    if( !(cid[i] & (1<<31)) )
    {
      // LSByte von EAX (AL) ignorieren
      if( i != 0 )
        printCacheInfo( cid[i] & 0xff );

      printCacheInfo( (cid[i]>>8) & 0xff );
      printCacheInfo( (cid[i]>>16) & 0xff );
      printCacheInfo( (cid[i]>>24) & 0xff );
    }
  }
}

void TCPUData::printCacheInfo( int value )
{
  int i=0;

  if( value <= 0 )
    return;

  for( ;; )
  {
    if( cids[i].id == 0 )
      break;
    else if( cids[i].id == value )
    {
      Message( cids[i].text );
      break;
    }
    
    i++;
  }
}

