/*
   I8kfanGUI V3.1
   Copyright(c) 2001-2007 Christian Diefer

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as 
   published by the Free Software Foundation.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "stdafx.h"
#include "AtlBase.h"
#include "i8kfangui.h"
#include "i8kfanctrl.h"
#include "GlobalVars.h"
#include <winioctl.h>
#include "..\fanio\src\fanioctl.h"
#include "cpuspeed.h"

extern "C" {
#include <powrprof.h>
}

// buffers for HD-DeviceIoControls:
static ATAInParam in;
static ATAOutParam out;
static BYTE	identity[sizeof(ATAOutParam) + IDENTIFY_BUFFER_SIZE - 1];
static BYTE	attributes[sizeof(ATAOutParam) + READ_ATTRIBUTE_BUFFER_SIZE - 1];


//-----------------------------------------
// Puffer mit Video BIOS-Image fllen
//-----------------------------------------
BOOL getBIOSImage( HANDLE hndFile, int& code, int& driverCode, UCHAR** biosImage, ULONG* size )
{
  ULONG mapping[4];
  ULONG ReturnedLength;
  BOOL IoctlResult;
  BOOL error = FALSE;

  IoctlResult = DeviceIoControl(
                        hndFile,
                        IOCTL_FANIO_MAP_VBIOS,
                        NULL,
                        0,
                        mapping,
                        4*sizeof(ULONG),
                        &ReturnedLength,
                        NULL );

  if( !IoctlResult )
  {
    code = 0;
    return FALSE;
  }

  if( mapping[0] != 0 )
  {
    code = 1;
    driverCode = mapping[0];
    return FALSE;
  }

  log( L"Video BIOS successfully mapped at 0x%08x with size %d bytes (= %d pages)", mapping[1], mapping[2]*512, mapping[2] );

  PUCHAR image = new UCHAR[mapping[2]*512];

  for( ULONG i=0; i<mapping[2]; i++ )
  {
    IoctlResult = DeviceIoControl(
                          hndFile,
                          IOCTL_FANIO_GET_VBIOS_CHUNK,
                          &i,
                          sizeof(ULONG),
                          &image[i*512],
                          512,
                          &ReturnedLength,
                          NULL );
  
    if( !IoctlResult )
    {
      error = TRUE;
      break;
    }
  }

  if( !error )
    log( L"Image successfully copied to buffer 0x%08x", image );
  else
    code = 2;

  IoctlResult = DeviceIoControl(
                        hndFile,
                        IOCTL_FANIO_UNMAP_VBIOS,
                        NULL,
                        0,
                        mapping,
                        3*sizeof(ULONG),
                        &ReturnedLength,
                        NULL );

  return !error;
}

//-----------------------------------------
// Suche nach kompatiblem Sensorchip durchfhren
//-----------------------------------------
BOOL detectFanSensor( HANDLE hndFile )
{
  ULONG sensor[16];
  ULONG ReturnedLength;
  BOOL IoctlResult;

  IoctlResult = DeviceIoControl(
                        hndFile,
                        IOCTL_FANIO_CHECK_SENSOR,
                        NULL,
                        0,
                        sensor,
                        16*sizeof(ULONG),
                        &ReturnedLength,
                        NULL );

  log( L"detectSensor: foundFlag=%08x (=> %s)", sensor[0], sensor[0]?"found":"not found" );
  log( L"detectSensor: base1=%08x exdevID1=%08x deviceID1=%08x revision1=%08x",
       sensor[1], sensor[2], sensor[3], sensor[4] );
  log( L"detectSensor: base2=%08x exdevID2=%08x deviceID2=%08x revision2=%08x",
       sensor[5], sensor[6], sensor[7], sensor[8] );
  log( L"detectSensor: logdev=%08x ldn9active=%08x mboxport=%08x", sensor[9], sensor[10], sensor[11] );

  if( !IoctlResult )                         
    return FALSE;

  return sensor[0]!=0;
}

//-----------------------------------------
// Lfterdrehzahl ber HW-Sensor setzen
//-----------------------------------------
BOOL setFanSpeedSensor( int fan, int speed, HANDLE hndFile )
{
  USHORT t[2];
  ULONG ReturnedLength;
  ULONG ret;
  BOOL IoctlResult;
    
  t[0] = (USHORT)fan;
  t[1] = (USHORT)speed;

  IoctlResult = DeviceIoControl(
                      hndFile,
                      IOCTL_FANIO_SET_SENSOR_SPEED,
                      t,
                      sizeof(ULONG),      
                      &ret,
                      sizeof(ULONG),
                      &ReturnedLength,
                      NULL );                 
 
  log( L"Set fan speed sensor; fan:%d speed:%d ret:%d", fan, speed, ret );

  if( ret == 0 )                         
    return FALSE;

  return TRUE;
}

//-----------------------------------------
// Lfterstatus ber HW-Sensor ermitteln
//-----------------------------------------
BOOL getFanSpeedSensor( int fan, int& speed, HANDLE hndFile )
{
  ULONG t;
  ULONG ReturnedLength;
  BOOL IoctlResult;

  IoctlResult = DeviceIoControl(
                      hndFile,
                      IOCTL_FANIO_GET_SENSOR_SPEED,
                      &fan,
                      sizeof(int),      
                      &t,
                      sizeof(ULONG),
                      &ReturnedLength,
                      NULL );                 

  log( L"Get fan speed sensor; fan:%d speed:%d ret:0x%08x", fan, speed, t );

  if( t == 0xffffffff )                         
    return FALSE;

  speed = t;

  return TRUE;
}

//-----------------------------------------
// aktuelle CPU-Speed ermitteln
//-----------------------------------------
BOOL getCPUSpeed( wchar_t* speed, int readVar, HANDLE hndFile )
{
  if( readVar == 1 )        // Echtzeit-Berechnung
  {
    int x = cpuSpeedHT();
    if( x == 0 )
      wcscpy_s( speed, 16, L"???" );
    else
      swprintf_s( speed, 16, L"%d", x );
  }
  else if( readVar == 0 )   // vom BIOS auslesen
  {
    ULONG ReturnedLength;
    BOOL IoctlResult;

	char asciiSpeed[16];

    IoctlResult = DeviceIoControl(
                          hndFile,
                          IOCTL_FANIO_GET_CPU_SPEED,
                          NULL,
                          0,
                          asciiSpeed,
                          8,
                          &ReturnedLength,
                          NULL );

    if( !IoctlResult )                         
      return FALSE;

	char2uc( speed, asciiSpeed );
  }
  else if( readVar == 2 )   // von Windows holen (ber PowerDLL)
  {
    if( !glb_pCallPI )
      return FALSE;

    PROCESSOR_POWER_INFORMATION ppi;
    BOOL x = (glb_pCallPI(ProcessorInformation,
                          NULL,
                          0,
                          &ppi,
                          sizeof(ppi)) == ERROR_SUCCESS);
    if( x )
    {
      log( L"curspeed:%d maxspeed:%d limit:%d Maxidlestate:%d Currentidlestate:%d",
           ppi.CurrentMhz, 
           ppi.MaxMhz, 
           ppi.MhzLimit, 
           ppi.MaxIdleState, 
           ppi.CurrentIdleState );
      wsprintfW( speed, L"%d", ppi.CurrentMhz );
    }
    else 
      return FALSE;
  }
  else if( readVar == 3 )        // Echtzeit-Berechnung alt; wie bis einschlielich V2.2.0
  {
    FREQ_INFO fi;
    int ret = getRDTSCSpeed( &fi );
    if( ret != 0 )
    {
      if( ret == 2 )
      {
        log( L"Old realtime calculation algorithm failed => switching to new algorithm" );
        options.readCPUVar = 1;     // umschalten auf neue 'Realtime Calculation'
      }
    }
    else
      wsprintfW( speed, L"%d", fi.norm_freq );
  }

  return TRUE;
}

//-----------------------------------------
// Temperatur von einem internen Sensor lesen (SMBIOS)
//-----------------------------------------
BOOL readTemp( int thermNo, int& temp, HANDLE hndFile )
{
  UINT t[5];
  ULONG ReturnedLength;
  BOOL IoctlResult;

  IoctlResult = DeviceIoControl(
                      hndFile,
                      IOCTL_FANIO_GET_TEMP,
                      &thermNo,
                      sizeof(int),      
                      t,
                      5*sizeof(UINT),
                      &ReturnedLength,       
                      NULL );                

  log( L"readTemp: %08x %08x %08x %08x %08x", t[0], t[1], t[2], t[3], t[4] );

  if( t[0] == -1 )                         
    return FALSE;

  temp = t[0];

  return TRUE;
}

//-----------------------------------------
// aktuelle Drehzahl eines Lfter in RPM ermitteln (SMBIOS)
//-----------------------------------------
BOOL readFanSpeed( int fan, int& rpm, HANDLE hndFile )
{
  UINT t[5];
  ULONG ReturnedLength;
  BOOL IoctlResult;

  IoctlResult = DeviceIoControl(
                      hndFile,
                      IOCTL_FANIO_GET_FAN_RPM,
                      &fan,
                      sizeof(int),      
                      t,
                      5*sizeof(UINT),
                      &ReturnedLength,       
                      NULL );                

  log( L"readFanSpeed: %08x %08x %08x %08x %08x", t[0], t[1], t[2], t[3], t[4] );

  if( t[0] == -1 )                          
    return FALSE;

  rpm = t[0];

  return TRUE;
}

//-----------------------------------------
// aktuellen Status eines Lfters ermitteln (SMBIOS)
//-----------------------------------------
BOOL readFanState( int fan, int& speed, HANDLE hndFile )
{
  UINT t[5];
  ULONG ReturnedLength;
  BOOL IoctlResult;

  IoctlResult = DeviceIoControl(
                      hndFile,
                      IOCTL_FANIO_GET_FAN_SPEED,
                      &fan,
                      sizeof(int),      
                      t,
                      5*sizeof(UINT),
                      &ReturnedLength,        
                      NULL );                 

  log( L"readFanState: %08x %08x %08x %08x %08x", t[0], t[1], t[2], t[3], t[4] );

  if( t[0] == -1 )                    
    return FALSE;

  speed = t[0];

  return TRUE;
}

//-----------------------------------------
// FanIO-Treiberversion ermitteln
//-----------------------------------------
BOOL getDriverVersion( PULONG ver, HANDLE hndFile )
{
  ULONG t[2];
  ULONG ReturnedLength;
  BOOL IoctlResult;
    
  IoctlResult = DeviceIoControl(
                      hndFile,
                      IOCTL_FANIO_GET_VERSION,
                      NULL,
                      0,
                      t,
                      2*sizeof(ULONG),
                      &ReturnedLength,       
                      NULL );                

  if( !IoctlResult )                         
    return FALSE;

  ver[0] = t[0];
  ver[1] = t[1];

  return TRUE;
}

//-----------------------------------------
// BIOS-Info vom Treiber ermitteln
//-----------------------------------------
BOOL getSystemInfo( wchar_t* info, HANDLE hndFile )
{
  ULONG ReturnedLength;
  BOOL IoctlResult;

  static char buf[32768];

  IoctlResult = DeviceIoControl(
                        hndFile,
                        IOCTL_FANIO_GET_SYSTEM_INFO,
                        NULL,
                        0,
                        buf,
                        16384,
                        &ReturnedLength,
                        NULL );

  if( !IoctlResult )
    return FALSE;

  char2uc( info, buf );
  
  return true;
}

//-----------------------------------------
// ACPI-Info vom Treiber holen
//-----------------------------------------
BOOL getACPIInfo( PCHAR info, HANDLE hndFile )
{
  ULONG ReturnedLength;
  BOOL IoctlResult;

  IoctlResult = DeviceIoControl(
                        hndFile,
                        IOCTL_FANIO_ACPI_INFO,
                        NULL,
                        0,
                        info,
                        32768,
                        &ReturnedLength,
                        NULL );

  if( !IoctlResult )
    return FALSE;
  
  return true;
}

//-----------------------------------------
// Drehzahl eines Lfters ber Status setzen (SMBIOS)
//-----------------------------------------
BOOL setFanSpeed( int fan, int speed, HANDLE hndFile )
{
  USHORT t[2];
  ULONG ReturnedLength;
  ULONG ret[5];
  BOOL IoctlResult;
  int errCount;

  for( errCount=0; errCount<5; errCount++ )
  {
    t[0] = (USHORT)fan;
    t[1] = (USHORT)speed;

    IoctlResult = DeviceIoControl(
                        hndFile,
                        IOCTL_FANIO_SET_FAN_SPEED,
                        t,
                        2*sizeof(USHORT),      
                        ret,
                        5*sizeof(ULONG),
                        &ReturnedLength,
                       NULL );                 

    log( L"setFanSpeed (try #%d): %08x %08x %08x %08x %08x", errCount, ret[0], ret[1], ret[2], ret[3], ret[4] );

    if( ret[0] != -1 )                         
      break;
  }

  if( errCount == 5 )
    return FALSE;

  return TRUE;
}

//---------------------------------------------------------------
// Funktionen des Power-APIs importieren (fr Win2000+)
//
// Dies muss hier dynamisch geschehen, um Kompatibilitt mit NT4 zu wahren (seufz!)
//---------------------------------------------------------------
void importPowerDLL()
{
  log( L"/bImporting POWRPROF.DLL... " );
  glb_pPowerApiDLL = LoadLibraryW( L"powrprof.dll" );
  if( glb_pPowerApiDLL == NULL )
  {
    log( L"/afailed" );
    return;
  }

  log( L"/aOk" );

  log( L"/bImporting PowerDLL function... " );
  glb_pCallPI = (LONG (WINAPI *)(POWER_INFORMATION_LEVEL,PVOID,ULONG,PVOID,ULONG))GetProcAddress( glb_pPowerApiDLL, "CallNtPowerInformation" );

  log( L"/a%s", glb_pCallPI ? L"Ok" : L"failed" );
}

//---------------------------------------------------------------
// Eine Funktion der UxTheme.DLL importieren (fr WinXP+)
//
// Dies muss hier dynamisch geschehen, um Kompatibilitt mit NT4 + 2000 zu wahren (seufz!)
//---------------------------------------------------------------
void importUxThemeDLL()
{
  log( L"/bImporting UXTHEME.DLL... " );
  glb_pUxThemeDLL = LoadLibrary( L"uxtheme.dll" );
  if( glb_pUxThemeDLL == NULL )
  {
    log( L"/afailed" );
    return;
  }

  log( L"/aOk" );

  log( L"/bImporting UxThemeDLL function... " );
  glb_pCallThemeEnable = (HRESULT (WINAPI *)(HWND,DWORD))GetProcAddress( glb_pUxThemeDLL, "EnableThemeDialogTexture" );

  log( L"/a%s", glb_pCallThemeEnable ? L"Ok" : L"failed" );
}

//-----------------------------------------
// Stringkonvertierung von big endian in little endian und von ANSI in Unicode fr HD-Strings
//-----------------------------------------
static void getHDStr( wchar_t* s, PWORD ib, unsigned int inqBufSize )
{
  PWORD p  = ib;
  char* buf = new char[(inqBufSize+1)*2];
  char* p1 = buf;

  for( unsigned int i=0; i<inqBufSize; ++i, ++p )
  {
    *p1++ = (char)(*p >> 8);
    *p1++ = (char)*p;
  }

  *p1 = '\0';
  
  p1 = buf;

  while( p1[0] == ' ' )
    memmove( p1, p1+1, strlen(p1) );

  size_t l = strlen( p1 );
  
  while( (l>0) && (p1[l-1]==' ') )
    p1[--l] = '\0';

  char2uc( s, buf ); 

  delete[] buf;
}

//-----------------------------------------
// HD-Temperatur ermitteln
//
// Returncodes:
//    0: Erfolgt
//   -1: kann HD nicht ansprechen (= Laufwerk nicht gefunden)
//   -2: (unbenutzt)
//   -3: SMART_RECEIVE_DRIVE_DATA fehlgeschlagen (= LW vorhanden, aber IDENTIFY fehlgeschlagen)
//   -4: LW untersttzt kein SMART
//   -5: HD-Smartdaten konnten nicht gelesen werden
//   -6: kein Temperaturattribut in SMART-Daten gefunden
//-----------------------------------------
int readHDTemp( int driveNum, int& temp, wchar_t* infoBuf, int infoSize, BOOL enSMART )
{
  log( L"Start von readHDTemp (HD-Sub)" );
  static wchar_t device[64];
  CString yes,no;

  yes.LoadStringW( IDS_YES );
  no.LoadStringW( IDS_NO );

  if( infoBuf != NULL )
  {
    *infoBuf = '\0';
    log( L"Gathering HD info from device %d", driveNum );
  }
  else
    log( L"Reading HD temperature from device %d", driveNum );

  wsprintfW( device, L"\\\\.\\PhysicalDrive%d", driveNum );

  log( L"HD device: <%s>", device );

  HANDLE hd = CreateFile( device, 
                          GENERIC_READ | GENERIC_WRITE, 
                          FILE_SHARE_READ | FILE_SHARE_WRITE, 
                          NULL, 
                          OPEN_EXISTING, 
                          0, 
                          NULL );

  if( hd == INVALID_HANDLE_VALUE )
  {
    log( L"Can't open HD device %d", driveNum );

    if( infoBuf != NULL )
    {
      CString s1;
      s1.LoadStringW( IDS_HD1 );
      wsprintfW( infoBuf, s1, driveNum );
    }

    return -1;
  }

  log( L"HD device opened" );

  memset( &in, 0, sizeof(int) );
  memset( &out, 0, sizeof(out) );

  if( enSMART )
    enableSMART( driveNum, hd, &in, &out );

  DWORD br;

  memset( &in, 0, sizeof(in) );
  memset( identity, 0, sizeof(identity) );

  in.cBufferSize = IDENTIFY_BUFFER_SIZE;
  in.irDriveRegs.bFeaturesReg = 0;
  in.irDriveRegs.bSectorCountReg = 1;
  in.irDriveRegs.bSectorNumberReg = 1;
  in.irDriveRegs.bCylLowReg = 0;
  in.irDriveRegs.bCylHighReg = 0;
  in.irDriveRegs.bDriveHeadReg = 0xa0 | ((driveNum & 1) << 4); 
  in.irDriveRegs.bCommandReg = IDE_ID_FUNCTION;
  in.bDriveNumber = driveNum;
  in.cBufferSize = IDENTIFY_BUFFER_SIZE;

  if( DeviceIoControl(hd,
                      SMART_RCV_DRIVE_DATA,
                      &in,
                      sizeof(ATAInParam)-1,
                      identity,
                      sizeof(ATAOutParam) + IDENTIFY_BUFFER_SIZE - 1,
                      &br,
                      NULL) == 0 )
  {
    log( L"SMART_RECEIVE_DRIVE_DATA #1 failed" );
    CloseHandle( hd );
    if( infoBuf != NULL )
    {
      CString s1;
      s1.LoadStringW( IDS_HD2 );
      wcscpy_s( infoBuf, infoSize-wcslen(infoBuf)-1, s1 );
    }
    return -3;
  }

  PWORD buf = (PWORD)(((ATAOutParam*)identity)->bBuffer);
  HDDRIVEID* di = (HDDRIVEID*)buf;

  if( infoBuf != NULL )
  {
    // Stringkonvertierung von big endian in little endian:
    wchar_t model[44];
    wchar_t firmWare[12];
    wchar_t serialNo[24];

    getHDStr( model, (PWORD)di->model, 20 );
    getHDStr( firmWare, (PWORD)di->fw_rev, 4 );
    getHDStr( serialNo, (PWORD)di->serial_no, 10 );

    log( L"Drive data: <%s> <%s> <%s>", model, firmWare, serialNo );

    CString s1;
    s1.LoadStringW( IDS_HD3 );
    if( !*model )
      wcscpy_s( model, s1 );

    if( !*firmWare )
      wcscpy_s( firmWare, s1 );

    if( !*serialNo )
      wcscpy_s( serialNo, s1 );

    s1.LoadStringW( IDS_HD4 );
    wsprintf( &infoBuf[wcslen(infoBuf)], s1, model );
    s1.LoadStringW( IDS_HD5 );
    wsprintf( &infoBuf[wcslen(infoBuf)], s1, firmWare );
    s1.LoadStringW( IDS_HD6 );
    wsprintf( &infoBuf[wcslen(infoBuf)], s1, serialNo );
    s1.LoadStringW( IDS_HD7 );
    wsprintf( &infoBuf[wcslen(infoBuf)], s1, di->capability&2 ? yes : no );

    if( di->capability & 2 )
    {
      double x = di->lba_capacity;

      x = x * 512 / 1024 / 1024 / 1024;

      s1.LoadStringW( IDS_HD8 );
      swprintf_s( &infoBuf[wcslen(infoBuf)], infoSize-_tcslen(infoBuf)-1, 
		              s1, 
                  di->lba_capacity, x );
    }

    s1.LoadStringW( IDS_HD9 );
    wsprintf( &infoBuf[wcslen(infoBuf)], s1, di->buf_size/2 );

    if( di->field_valid & 4 )   // UDMA-Info verfgbar
    {
      s1.LoadStringW( IDS_HD10 );
      wcscat_s( infoBuf, infoSize, s1 );

      if( di->dma_ultra & 64 )       wcscat_s( infoBuf, infoSize, L"0-6" );
      else if( di->dma_ultra & 32 )  wcscat_s( infoBuf, infoSize, L"0-5" );
      else if( di->dma_ultra & 16 )  wcscat_s( infoBuf, infoSize, L"0-4" );
      else if( di->dma_ultra & 8 )   wcscat_s( infoBuf, infoSize, L"0-3" );
      else if( di->dma_ultra & 4 )   wcscat_s( infoBuf, infoSize, L"0-2" );
      else if( di->dma_ultra & 2 )   wcscat_s( infoBuf, infoSize, L"0,1" );
      else if( di->dma_ultra & 1 )   wcscat_s( infoBuf, infoSize, L"0" );
      else                           
      {
        s1.LoadStringW( IDS_HD11 );
        wcscat_s( infoBuf, infoSize, s1 );
      }

      s1.LoadStringW( IDS_HD12 );
      wcscat_s( infoBuf, infoSize, s1 );

      if( di->dma_ultra & 0x4000 )        wcscat_s( infoBuf, infoSize, L"6" );
      else if( di->dma_ultra & 0x2000 )   wcscat_s( infoBuf, infoSize, L"5" );
      else if( di->dma_ultra & 0x1000 )   wcscat_s( infoBuf, infoSize, L"4" );
      else if( di->dma_ultra & 0x0800 )   wcscat_s( infoBuf, infoSize, L"3" );
      else if( di->dma_ultra & 0x0400 )   wcscat_s( infoBuf, infoSize, L"2" );
      else if( di->dma_ultra & 0x0200 )   wcscat_s( infoBuf, infoSize, L"1" );
      else if( di->dma_ultra & 0x0100 )   wcscat_s( infoBuf, infoSize, L"0" );
      else
      {
        s1.LoadStringW( IDS_HD11 );
        wcscat_s( infoBuf, infoSize, s1 );
      }
      
      wcscat_s( infoBuf, infoSize, L"\r\n" );
    }

    bool ams = (di->command_set_2!=0 && di->command_set_2!=0xffff && di->command_set_2&0x200);

    s1.LoadStringW( IDS_HD13 );
    wsprintfW( &infoBuf[wcslen(infoBuf)], s1, ams ? yes : no );

    if( ams )
    {
      bool ame = (di->cfs_enable_2 & 0x200) != 0;

      s1.LoadStringW( IDS_HD14 );
      wsprintfW( &infoBuf[wcslen(infoBuf)], s1, ame ? yes : no );
      
      CString s2, s3;
      s2.LoadStringW( IDS_HD16 );
      s3.LoadStringW( IDS_HD17 );

      if( ame )
      {
        UCHAR cv = (UCHAR)di->aam_values;

        s1.LoadStringW( IDS_HD15 );
        wsprintfW( &infoBuf[wcslen(infoBuf)], s1, 
                   cv, cv>=192 ? s2 : s3 );
      }

      UCHAR rv = (UCHAR)(di->aam_values>>8);
      s1.LoadStringW( IDS_HD18 );
      wsprintfW( &infoBuf[wcslen(infoBuf)], s1, 
                rv, rv>=192 ? s2 : s3 );
    }

    s1.LoadStringW( IDS_HD19 );
    wsprintfW( &infoBuf[wcslen(infoBuf)], s1, di->command_set_1&1 ? yes : no );
  }

  log( L"SMART support: <%s>", di->command_set_1&1 ? yes : no );

  if( !(di->command_set_1&1) )      // LW nicht SMART-fhig
  {
    log( L"HD not SMART capable" );
    CloseHandle( hd );
    return -4;
  }

  memset( &in, 0, sizeof(in) );
  memset( attributes, 0, sizeof(attributes) );

  in.cBufferSize = READ_ATTRIBUTE_BUFFER_SIZE;
  in.bDriveNumber = driveNum;
  in.irDriveRegs.bSectorCountReg = 1;
  in.irDriveRegs.bSectorNumberReg = 1;
  in.irDriveRegs.bDriveHeadReg = 0xa0 | ((driveNum & 1) << 4);;
  in.irDriveRegs.bCommandReg = IDE_EXECUTE_SMART_FUNCTION;
  in.irDriveRegs.bFeaturesReg = SMART_READ_ATTRIBUTE_VALUES;
  in.irDriveRegs.bCylHighReg = SMART_CYL_HI;
  in.irDriveRegs.bCylLowReg = SMART_CYL_LOW;

  log( L"Reading SMART data..." );

  if( DeviceIoControl(hd,
                      SMART_RCV_DRIVE_DATA,
                      &in,
                      sizeof(in)-1,
                      attributes,
                      sizeof(ATAOutParam) + READ_ATTRIBUTE_BUFFER_SIZE - 1,
                      &br,
                      NULL) == 0 )
  {
    log( L"SMART_RECEIVE_DRIVE_DATA #2 failed" );
    CloseHandle( hd );
    CString s1;
    s1.LoadStringW( IDS_HD20 );
    if( infoBuf != NULL )
      wsprintfW( &infoBuf[wcslen(infoBuf)], s1 );
    return -5;
  }

  CString s1;
  PBYTE y = (PBYTE)(((ATAOutParam*)attributes)->bBuffer);
  BOOL found = FALSE;

  for( int i=0; i<30; i++ )
  {
    PBYTE x1 = &y[2+i*12];

    if( x1[0] == 194 )      // in Smartdaten nach Temperaturattribut suchen
    {
      temp = x1[5];
      log( L"Temperature: %dC", temp );
      found = TRUE;
      break;
    }
  }

  log( L"Temperature support: <%s>", found ? yes : no );

  if( infoBuf != NULL )
  {
    s1.LoadStringW( IDS_HD21 );
    wsprintfW( &infoBuf[wcslen(infoBuf)], s1, found ? yes : no );

    if( found )
    {
      if( options.fahrenHeit )
      {
        s1.LoadStringW( IDS_HD22 );
        wsprintfW( &infoBuf[wcslen(infoBuf)], s1, options.convertTemp(temp,FAHRENHEIT) );
      }
      else
      {
        s1.LoadStringW( IDS_HD23 );
        wsprintfW( &infoBuf[wcslen(infoBuf)], s1, temp );
      }
    }
  }

  CloseHandle( hd );

  log( L"Ende von readHDTemp (HD-Sub)" );
  return found ? 0 : -6;
}

//-----------------------------------------
// SMART auf einem Laufwerk aktivieren
//-----------------------------------------
BOOL enableSMART( int driveNum, HANDLE hd, ATAInParam* in, ATAOutParam* out )
{
  log( L"Start von enableSMART (HD-Sub)" );
  DWORD br;

  memset( in, 0, sizeof(in) );
  memset( out, 0, sizeof(out) );

  in->cBufferSize = 0;
  in->irDriveRegs.bFeaturesReg = SMART_ENABLE_SMART_OPERATIONS;
  in->irDriveRegs.bSectorCountReg = 1;
  in->irDriveRegs.bSectorNumberReg = 1;
  in->irDriveRegs.bCylLowReg = SMART_CYL_LOW;
  in->irDriveRegs.bCylHighReg = SMART_CYL_HI;
  in->irDriveRegs.bDriveHeadReg = 0xa0 | ((driveNum & 1) << 4); 
  in->irDriveRegs.bCommandReg = IDE_EXECUTE_SMART_FUNCTION;
  in->bDriveNumber = driveNum;

  log( L"Ende von enableSMART (HD-Sub)" );
  return DeviceIoControl(hd,
                         SMART_SEND_DRIVE_COMMAND,
                         in,
                         sizeof(ATAInParam)-1,
                         out,
                         sizeof(ATAOutParam)-1,
                         &br,
                         NULL);
}

//-----------------------------------------
// aktuelle CPU-Last ermitteln
//-----------------------------------------
// neue Variante der CPU-Lasterkennung
DWORD readCPULoadGST()
{
  static ULARGE_INTEGER liOldIdleTime = {0,0};
  static ULARGE_INTEGER liOldKernelTime = {0,0};
  static ULARGE_INTEGER liOldUserTime = {0,0};
  static bool firstRun = true;  
  static FILETIME idleTime;
  static FILETIME kernelTime;
  static FILETIME userTime;
  static double dbIdleTime;
  static double dbSystemTime;
  DWORD retVal = 0xffffffff;
  ULARGE_INTEGER idle, kernel, user;

  if( !glb_ProcGetSystemTimes(&idleTime,&kernelTime,&userTime) )
    return 0xffffffff;

  memcpy( &idle, &idleTime, sizeof(FILETIME) );
  memcpy( &kernel, &kernelTime, sizeof(FILETIME) );
  memcpy( &user, &userTime, sizeof(FILETIME) );

  if( !firstRun )
  {
    double idleDiff = Li2Double(idle) - Li2Double(liOldIdleTime);
    double sys = (Li2Double(user) - Li2Double(liOldUserTime))       // userDiff
               + (Li2Double(kernel) - Li2Double(liOldKernelTime));  // kernelDiff
    dbIdleTime = (sys - idleDiff)*100.0/sys;
    retVal = (DWORD)dbIdleTime;   
  }
  else
    firstRun = false;

  liOldIdleTime = idle;
  liOldKernelTime = kernel;
  liOldUserTime = user;

  return retVal;
}

// alte Variante, die wohl auf neueren Windows OS's nicht mehr funktionieren knnte
DWORD readCPULoad()
{
  static SYSTEM_PERFORMANCE_INFORMATION SysPerfInfo;
  static SYSTEM_TIME_INFORMATION SysTimeInfo;
  static SYSTEM_BASIC_INFORMATION SysBaseInfo;
  static double dbIdleTime;
  static double dbSystemTime;
  static LONG status;
  static LARGE_INTEGER liOldIdleTime = {0,0};
  static LARGE_INTEGER liOldSystemTime = {0,0};
  static BOOL init = FALSE;
  static PROCNTQSI NtQuerySystemInformation = NULL;
  DWORD retVal = 0xffffffff;

  if( !init )
  {
    NtQuerySystemInformation = (PROCNTQSI)GetProcAddress( GetModuleHandleW(L"ntdll"), "NtQuerySystemInformation" );

    if( !NtQuerySystemInformation )
      return 0xffffffff;

    // Anzahl CPUs im System ermitteln
    status = NtQuerySystemInformation( SystemBasicInformation, &SysBaseInfo, sizeof(SysBaseInfo), NULL );
    if( status != NO_ERROR )
      return 0xffffffff;

    init = TRUE;
  }

  // neue Systemzeit holen
  status = NtQuerySystemInformation( SystemTimeInformation, &SysTimeInfo, sizeof(SysTimeInfo), 0 );

  if( status != NO_ERROR )
    return 0xffffffff;

  // Idlezeit der CPU holen
  status = NtQuerySystemInformation( SystemPerformanceInformation, &SysPerfInfo, sizeof(SysPerfInfo), NULL );
  if( status != NO_ERROR )
    return 0xffffffff;

  // beim ersten Aufruf berspringen
  if( liOldIdleTime.QuadPart != 0 )
  {
    // CurrentValue = NewValue - OldValue
    dbIdleTime = Li2Double( SysPerfInfo.liIdleTime ) - Li2Double( liOldIdleTime );
    dbSystemTime = Li2Double( SysTimeInfo.liKeSystemTime ) - Li2Double( liOldSystemTime );

    // CurrentCpuIdle = IdleTime / SystemTime
    dbIdleTime = dbIdleTime / dbSystemTime;

    // CurrentCpuUsage% = 100 - (CurrentCpuIdle * 100) / NumberOfProcessors
    dbIdleTime = 100.0 - dbIdleTime * 100.0 / (double)SysBaseInfo.bKeNumberProcessors + 0.5;
    
    retVal = (DWORD)dbIdleTime;
  }

  // Idle- und Systemzeit sichern fr nchsten Durchlauf
  liOldIdleTime = SysPerfInfo.liIdleTime;
  liOldSystemTime = SysTimeInfo.liKeSystemTime;

  return retVal;
}

//-----------------------------------------
// PerformanceCounter wieder zurcksetzen beim Beenden
//-----------------------------------------
void stopPerfCnt()
{
  HKEY hkey;

  RegOpenKeyEx( HKEY_DYN_DATA,
                L"PerfStats\\StopStat",
                0,
                KEY_ALL_ACCESS,
                &hkey ); 

  DWORD dwDataSize;
  DWORD dwType;
  DWORD dwDummy;

  dwDataSize = sizeof( dwDummy ); 
  RegQueryValueEx( hkey,
                   L"KERNEL\\CPUUsage",
                   NULL,
                   &dwType,
                   (LPBYTE)&dwDummy,
                   &dwDataSize ); 

  RegCloseKey( hkey ); 
}

//------------------------------------
// Versionsinfo einer DLL ermitteln
//------------------------------------
DWORD getDllVersion( LPCTSTR lpszDllName )
{
  HINSTANCE hinstDll;
  DWORD dwVersion = 0;

  hinstDll = LoadLibrary( lpszDllName );

  if( hinstDll )
  {
    DLLGETVERSIONPROC pDllGetVersion;

    pDllGetVersion = (DLLGETVERSIONPROC)GetProcAddress( hinstDll, "DllGetVersion" );

    if( pDllGetVersion )
    {
      DLLVERSIONINFO dvi;
      HRESULT hr;

      ZeroMemory( &dvi, sizeof(dvi) );
      dvi.cbSize = sizeof( dvi );

      hr = (*pDllGetVersion)( &dvi );

      if( SUCCEEDED(hr) )
      {
        dwVersion = PACKVERSION( dvi.dwMajorVersion, dvi.dwMinorVersion );
      }
    }
        
    FreeLibrary( hinstDll );
  }

  return dwVersion;
}

//------------------------------------------------------------------
// convert ansi string to unicode string
//------------------------------------------------------------------
wchar_t* char2uc( wchar_t* d, char* s )
{
  PUCHAR dest = (PUCHAR)d;
  char* t = (char*)s;
  int i=-1;
  int j=-1;
  
  do {
    dest[++i] = t[++j];
    dest[++i] = '\0';
  } while( t[j] );
  
  return d;      
}

//------------------------------------------------------------------
// Lesen eines MSR
//------------------------------------------------------------------
BOOL readMSR( ULONG msr, ULONG& low, ULONG& high, HANDLE hndFile )
{
  ULONG t[2];
  ULONG ReturnedLength;
  BOOL IoctlResult;
    
  IoctlResult = DeviceIoControl(
                      hndFile,
                      IOCTL_FANIO_READ_MSR,
                      &msr,
                      sizeof(ULONG),
                      t,
                      2*sizeof(ULONG),
                      &ReturnedLength,       
                      NULL );                

  if( !IoctlResult )                         
    return FALSE;

  high = t[0];
  low = t[1];

  return TRUE;
}
