/*
   I8kfanGUI V3.1
   Copyright(c) 2001-2007 Christian Diefer
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as 
   published by the Free Software Foundation.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __I8KFANCTRL_H
#define __I8KFANCTRL_H

void log( wchar_t* msg, ... );

#include "TOptions.h"

#ifdef _WIN64
extern "C" void readCPUID( ULONG* eax, ULONG* ebx, ULONG* ecx, ULONG* edx );
#endif

#define WM_ICONNOTIFY (WM_APP+100)

typedef struct _PROCESSOR_POWER_INFORMATION {
  ULONG  Number;
  ULONG  MaxMhz;
  ULONG  CurrentMhz;
  ULONG  MhzLimit;
  ULONG  MaxIdleState;
  ULONG  CurrentIdleState;
} PROCESSOR_POWER_INFORMATION , *PPROCESSOR_POWER_INFORMATION ;

extern TOptions options;

BOOL getBIOSImage( HANDLE hndFile, int& code, int& driverCode, UCHAR** biosImage, ULONG* size );
BOOL readTemp( int thermNo, int& temp, HANDLE hndFile );
BOOL readFanSpeed( int fan, int& rpm, HANDLE hndFile );
BOOL readFanState( int fan, int& speed, HANDLE hndFile );
BOOL getSystemInfo( wchar_t* info, HANDLE hndFile );
BOOL getACPIInfo( wchar_t* info, HANDLE hndFile );
BOOL setFanSpeed( int fan, int speed, HANDLE hndFile );
int readHDTemp( int drive, int& temp, wchar_t* infoBuf, int infoSize, BOOL enableSMART );
BOOL getCPUSpeed( wchar_t* speed, int readVar, HANDLE hndFile );
BOOL detectFanSensor( HANDLE hndFile );
BOOL setFanSpeedSensor( int fan, int speed, HANDLE hndFile );
BOOL getFanSpeedSensor( int fan, int& speed, HANDLE hndFile );
BOOL getDriverVersion( PULONG ver, HANDLE hndFile );
DWORD readCPULoadGST();
DWORD readCPULoad();
void stopPerfCnt();
void importPowerDLL();
void importUxThemeDLL();
wchar_t* char2uc( wchar_t* d, char* s );
BOOL readMSR( ULONG msr, ULONG& low, ULONG& high, HANDLE hndFile );

/************************************/
/* declarations for HD SMART access */
/************************************/
#pragma pack(push,1)

#define READ_ATTRIBUTE_BUFFER_SIZE	512
#define IDENTIFY_BUFFER_SIZE		    512
#define READ_THRESHOLD_BUFFER_SIZE	512
                                              
// Valid values for the bCommandReg member of IDEREGS
//----------------------------------------------------
#define	IDE_ATAPI_ID                0xA1  // Returns ID sector for ATAPI.
#define	IDE_ID_FUNCTION             0xEC  // Returns ID sector for ATA.
#define	IDE_EXECUTE_SMART_FUNCTION  0xB0  // Performs SMART cmd.
                                          // Requires valid bFeaturesReg,
                                          // bCylLowReg, and bCylHighReg

// Feature register defines for SMART "sub commands"
//---------------------------------------------------
#define SMART_READ_ATTRIBUTE_VALUES				      0xD0	// ATA4: Renamed 
														                          // SMART READ DATA
#define	SMART_READ_ATTRIBUTE_THRESHOLDS			    0xD1	// Obsoleted in ATA4!
#define SMART_ENABLE_DISABLE_ATTRIBUTE_AUTOSAVE	0xD2
#define SMART_SAVE_ATTRIBUTE_VALUES				      0xD3
#define	SMART_EXECUTE_OFFLINE_IMMEDIATE			    0xD4	// ATA4
// Vendor specific commands:
#define	SMART_ENABLE_SMART_OPERATIONS			      0xD8
#define	SMART_DISABLE_SMART_OPERATIONS			    0xD9
#define	SMART_RETURN_SMART_STATUS				        0xDA

// bDriverError values
//---------------------
#define	SMART_NO_ERROR			    0	  // No error
#define	SMART_IDE_ERROR			    1	  // Error from IDE controller
#define	SMART_INVALID_FLAG		  2	  // Invalid command flag
#define	SMART_INVALID_COMMAND	  3	  // Invalid command byte
#define	SMART_INVALID_BUFFER	  4	  // Bad buffer (null, invalid addr..)
#define	SMART_INVALID_DRIVE		  5	  // Drive number not valid
#define	SMART_INVALID_IOCTL		  6	  // Invalid IOCTL
#define	SMART_ERROR_NO_MEM		  7	  // Could not lock user's buffer
#define	SMART_INVALID_REGISTER	8	  // Some IDE Register not valid
#define	SMART_NOT_SUPPORTED		  9	  // Invalid cmd flag set
#define	SMART_NO_IDE_DEVICE		  10	// Cmd issued to device not present
									                  // although drive number is valid

// Cylinder register values required when issuing SMART command
//--------------------------------------------------------------
#define	SMART_CYL_LOW	0x4F
#define	SMART_CYL_HI	0xC2

typedef struct
  {
    BYTE bFeaturesReg;
    BYTE bSectorCountReg;
    BYTE bSectorNumberReg;
    BYTE bCylLowReg;
    BYTE bCylHighReg;
    BYTE bDriveHeadReg;
    BYTE bCommandReg;
    BYTE bReserved;
  } IDERegisters;

typedef struct
  {
    DWORD        cBufferSize;
    IDERegisters irDriveRegs;
    BYTE         bDriveNumber;
    BYTE         bReserved[3];
    DWORD        dwReserved[4];
    BYTE         bBuffer[1];
  } ATAInParam;

typedef struct
  {
    BYTE  bDriverError;
    BYTE  bIDEStatus;
    BYTE  bReserved[2];
    DWORD dwReserved[2];
  } DriverStat;

typedef struct
  {
    DWORD      cBufferSize;
    DriverStat DriverStatus;
    BYTE       bBuffer[1];
  } ATAOutParam;


// structure of IDENTIFY sector, taken from Linux' hdreg.h ;-)
typedef struct hd_driveid {
        unsigned short  config;         /* lots of obsolete bit flags */
        unsigned short  cyls;           /* "physical" cyls */
        unsigned short  reserved2;      /* reserved (word 2) */
        unsigned short  heads;          /* "physical" heads */
        unsigned short  track_bytes;    /* unformatted bytes per track */
        unsigned short  sector_bytes;   /* unformatted bytes per sector */
        unsigned short  sectors;        /* "physical" sectors per track */
        unsigned short  vendor0;        /* vendor unique */
        unsigned short  vendor1;        /* vendor unique */
        unsigned short  vendor2;        /* vendor unique */
        unsigned char   serial_no[20];  /* 0 = not_specified */
        unsigned short  buf_type;
        unsigned short  buf_size;       /* 512 byte increments; 0 = not_specified */
        unsigned short  ecc_bytes;      /* for r/w long cmds; 0 = not_specified */
        unsigned char   fw_rev[8];      /* 0 = not_specified */
        unsigned char   model[40];      /* 0 = not_specified */
        unsigned char   max_multsect;   /* 0=not_implemented */
        unsigned char   vendor3;        /* vendor unique */
        unsigned short  dword_io;       /* 0=not_implemented; 1=implemented */
        unsigned char   vendor4;        /* vendor unique */
        unsigned char   capability;     /* bits 0:DMA 1:LBA 2:IORDYsw 3:IORDYsupport */
        unsigned short  reserved50;     /* reserved (word 50) */
        unsigned char   vendor5;        /* vendor unique */
        unsigned char   tPIO;           /* 0=slow, 1=medium, 2=fast */
        unsigned char   vendor6;        /* vendor unique */
        unsigned char   tDMA;           /* 0=slow, 1=medium, 2=fast */
        unsigned short  field_valid;    /* bits 0:cur_ok 1:eide_ok */
        unsigned short  cur_cyls;       /* logical cylinders */
        unsigned short  cur_heads;      /* logical heads */
        unsigned short  cur_sectors;    /* logical sectors per track */
        unsigned short  cur_capacity0;  /* logical total sectors on drive */
        unsigned short  cur_capacity1;  /*  (2 words, misaligned int)     */
        unsigned char   multsect;       /* current multiple sector count */
        unsigned char   multsect_valid; /* when (bit0==1) multsect is ok */
        unsigned int    lba_capacity;   /* total number of sectors */
        unsigned short  dma_1word;      /* single-word dma info */
        unsigned short  dma_mword;      /* multiple-word dma info */
        unsigned short  eide_pio_modes; /* bits 0:mode3 1:mode4 */
        unsigned short  eide_dma_min;   /* min mword dma cycle time (ns) */
        unsigned short  eide_dma_time;  /* recommended mword dma cycle time (ns) */
        unsigned short  eide_pio;       /* min cycle time (ns), no IORDY  */
        unsigned short  eide_pio_iordy; /* min cycle time (ns), with IORDY */
        unsigned short  words69_70[2];  /* reserved words 69-70 */
        unsigned short  words71_74[4];  /* reserved words 71-74 */
        unsigned short  queue_depth;    /*  */
        unsigned short  words76_79[4];  /* reserved words 76-79 */
        unsigned short  major_rev_num;  /*  */
        unsigned short  minor_rev_num;  /*  */
        unsigned short  command_set_1;  /* bits 0:Smart 1:Security 2:Removable 3:PM */
        unsigned short  command_set_2;  /* bits 14:Smart Enabled 13:0 zero */
        unsigned short  cfsse;          /* command set-feature supported extensions */
        unsigned short  cfs_enable_1;   /* command set-feature enabled */
        unsigned short  cfs_enable_2;   /* command set-feature enabled */
        unsigned short  csf_default;    /* command set-feature default */
        unsigned short  dma_ultra;      /*  */
        unsigned short  word89;         /* reserved (word 89) */
        unsigned short  word90;         /* reserved (word 90) */
        unsigned short  CurAPMvalues;   /* current APM values */
        unsigned short  word92;         /* reserved (word 92) */
        unsigned short  hw_config;      /* hardware config */
        unsigned short  aam_values;     /* acoustic management values */
        unsigned short  words95_125[31];/* reserved words 95-125 */
        unsigned short  last_lun;       /* reserved (word 126) */
        unsigned short  word127;        /* reserved (word 127) */
        unsigned short  dlf;            /* device lock function
                                         * 15:9 reserved
                                         * 8    security level 1:max 0:high
                                         * 7:6  reserved
                                         * 5    enhanced erase
                                         * 4    expire
                                         * 3    frozen
                                         * 2    locked
                                         * 1    en/disabled
                                         * 0    capability
                                         */
        unsigned short  csfo;           /* current set features options
                                         * 15:4 reserved
                                         * 3    auto reassign
                                         * 2    reverting
                                         * 1    read-look-ahead
                                         * 0    write cache
                                         */
        unsigned short  words130_155[26];/* reserved vendor words 130-155 */
        unsigned short  word156;
        unsigned short  words157_159[3];/* reserved vendor words 157-159 */
        unsigned short  words160_255[95];/* reserved words 160-255 */
} HDDRIVEID;


#pragma pack(pop)

#define SystemBasicInformation       0
#define SystemPerformanceInformation 2
#define SystemTimeInformation        3

#define Li2Double(x) ((double)((x).HighPart) * 4.294967296E9 + (double)((x).LowPart))

typedef struct
{
    DWORD   dwUnknown1;
    ULONG   uKeMaximumIncrement;
    ULONG   uPageSize;
    ULONG   uMmNumberOfPhysicalPages;
    ULONG   uMmLowestPhysicalPage;
    ULONG   uMmHighestPhysicalPage;
    ULONG   uAllocationGranularity;
    PVOID   pLowestUserAddress;
    PVOID   pMmHighestUserAddress;
    ULONG   uKeActiveProcessors;
    BYTE    bKeNumberProcessors;
    BYTE    bUnknown2;
    WORD    wUnknown3;
} SYSTEM_BASIC_INFORMATION;

typedef struct
{
    LARGE_INTEGER   liIdleTime;
    DWORD           dwSpare[76];
} SYSTEM_PERFORMANCE_INFORMATION;

typedef struct
{
    LARGE_INTEGER liKeBootTime;
    LARGE_INTEGER liKeSystemTime;
    LARGE_INTEGER liExpTimeZoneBias;
    ULONG         uCurrentTimeZoneId;
    DWORD         dwReserved;
} SYSTEM_TIME_INFORMATION;

typedef LONG (WINAPI *PROCNTQSI)(UINT,PVOID,ULONG,PULONG);

typedef struct _NOTIFYICONDATA_CD { 
    DWORD cbSize; 
    HWND hWnd; 
    UINT uID; 
    UINT uFlags; 
    UINT uCallbackMessage; 
    HICON hIcon; 
    TCHAR szTip[128];
    DWORD dwState; 
    DWORD dwStateMask; 
    TCHAR szInfo[256]; 
    union {
      UINT  uTimeout; 
      UINT  uVersion; 
    } DUMMYUNIONNAME;
    TCHAR szInfoTitle[64]; 
    DWORD dwInfoFlags; 
} NOTIFYICONDATA_CD, *PNOTIFYICONDATA_CD; 

#define PACKVERSION(major,minor) MAKELONG(minor,major)

BOOL enableSMART( int driveNum, HANDLE driver, ATAInParam* in, ATAOutParam* out );
DWORD getDllVersion( LPCTSTR lpszDllName );


#endif
