/*
   I8kfanGUI V3.0
   Copyright(c) 2001-2006 Christian Diefer

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as 
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "stdafx.h"
#include "AtlBase.h"
#include "TOptions.h"
#include "GlobalVars.h"
#include "i8kfangui.h"
#include "ModelCheck.h"
#include "TStringTokenizer.h"
#include <stdio.h>
#include <string.h>

static wchar_t logPath[264];
static wchar_t buf[1024];
static wchar_t prodNameNew[128];


bool TModelCheck::checkModel( wchar_t* prodName, wchar_t* biosVer )
{
  wcscpy_s( prodNameNew, prodName );
  _wcslwr_s( prodNameNew );

  log( L"Searching the model configuration file for '%s'...", prodNameNew );

    // Dateinamen frs Log generieren
  if( GetModuleFileName(NULL,logPath,255) == 0 )
  {
    CString s1, s2;
    s1.LoadStringW( IDS_MOD1 );
    s2.LoadStringW( IDS_FATAL );
    MessageBoxW( NULL, s1, s2, MB_OK|MB_ICONSTOP );
    return FALSE;
  }

  size_t i = wcslen( logPath ) - 1, i1=i;
  while( i>=0 && i<=i1 && logPath[i]!='.' )
    i--;

  if( i >= 0 )
  {
    CString s1;
    s1.LoadStringW( IDS_MOD3 );
    wcscpy_s( &logPath[i], 260-i, s1 );
  }
  else
  {
    CString s1, s2;
    s1.LoadStringW( IDS_MOD2 );
    s2.LoadStringW( IDS_FATAL );
    MessageBoxW( NULL, s1, s2, MB_ICONSTOP|MB_OK );
    return FALSE;
  }

  log( L"Filename: <%s>", logPath );

  FILE* f1;
  if( _wfopen_s(&f1,logPath,L"rt") )
    return false;

  int line = 0;
  bool complete = false;

  while( !complete && fgetws(buf,1023,f1)!=NULL )
  {
    wchar_t* t = buf;
    
    line++;

    delEnd( t );

    while( iswspace(*t) ) t++;
    if( !*t || *t=='#' || *t==';' )
      continue;

    TStringTokenizer st( t );
    wchar_t* s;
    int i=0;

    while( (s=st.getNextToken()) != NULL )
    {
      log( L"Found token:<%s>", s );

      if( i == 0 )
      {
        _wcslwr_s( s, wcslen(s)+1 );
        if( !wcsstr(prodNameNew,s) )
          break;
      }
      else if( i == 1 )
      {
        _wcslwr_s( s, wcslen(s)+1 );
        if( !wcsstr(prodNameNew,s) )
          break;
      }

      if( i == 2 )
        log( L"Type found in line %d !", line );
      
      int val = _wtoi( s );

      switch( i )
      {
        case 2:   options.numFans = val;
                  log( L"# fans: <%d>", val );
                  break;

        case 3:   options.fan1Index = val;
                  log( L"Fan 1 index: <%d>", val );
                  break;

        case 4:   options.fan2Index = val;
                  log( L"Fan 2 index: <%d>", val );
                  break;

        case 5:   options.fan1Alias = s;
                  log( L"Fan 1 alias: <%s>", s );
                  break;

        case 6:   options.fan2Alias = s;
                  log( L"Fan 2 alias: <%s>", s );
                  break;

        case 7:   options.fanSpeedDivider = val;
                  log( L"Fan speed divider: <%d>", val );
                  break;

        case 8:   options.cpuSensorNo = val;
                  log( L"CPU sensor no: <%d>", val );
                  break;

        case 9:   options.gpuSensorNo = val;
                  log( L"GPU sensor no: <%d>", val );
                  break;

        case 10:  options.memorySensorNo = val;
                  log( L"Memory sensor no: <%d>", val );
                  break;

        case 11:  options.chipsetSensorNo = val;
                  log( L"Chipset sensor no: <%d>", val );
                  complete = true;
                  break;
      }

      i++;
    }
  }

  fclose( f1 );
  return true;
}

wchar_t* TModelCheck::delEnd( wchar_t* s )
{
  wchar_t* t = s + wcslen( s ) - 1;

  while( t >= s )
  {
    if( iswspace(*t) )
      t--;
    else
      break;
  }

  t[1]='\0';
  return s;
}
