/*
   I8kfanGUI V3.0
   Copyright(c) 2001-2006 Christian Diefer
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as 
   published by the Free Software Foundation.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "stdafx.h"
#include "AtlBase.h"
#include "i8kfangui.h"
#include "TOptions.h"
#include "I8kfanguiOptionDlg.h"
#include "OptionsAdvanced.h"
#include "GlobalVars.h"

// COptionsAdvanced dialog

IMPLEMENT_DYNAMIC(COptionsAdvanced, CDialog)
COptionsAdvanced::COptionsAdvanced(CWnd* pParent /*=NULL*/)
	: CDialog(COptionsAdvanced::IDD, pParent)
  , m_iFan1Index(0)
  , m_iFan2Index(0)
  , m_iFanSpeedDivider(0)
  , m_iCPUIndex(0)
  , m_iGPUIndex(0)
  , m_iMemoryIndex(0)
  , m_iOtherIndex(0)
  , m_szFan1Alias(_T(""))
  , m_szFan2Alias(_T(""))
  , m_iNumFans(0)
  , m_iCPUCompensation(0)
  , m_iGPUCompensation(0)
  , m_iMemoryCompensation(0)
  , m_iChipsetCompensation(0)
  , m_iFan1SensorIndex(0)
  , m_iFan2SensorIndex(0)
{ }

COptionsAdvanced::~COptionsAdvanced()
{}

void COptionsAdvanced::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Text(pDX, IDC_FAN1INDEX, m_iFan1Index);
  DDV_MinMaxInt(pDX, m_iFan1Index, 0, 5);
  DDX_Text(pDX, IDC_FAN2INDEX, m_iFan2Index);
  DDV_MinMaxInt(pDX, m_iFan2Index, 0, 5);
  DDX_Text(pDX, IDC_FANDIVIDER, m_iFanSpeedDivider);
  DDV_MinMaxInt(pDX, m_iFanSpeedDivider, 1, 5000);
  DDX_Text(pDX, IDC_CPUINDEX, m_iCPUIndex);
  DDV_MinMaxInt(pDX, m_iCPUIndex, 0, 9);
  DDX_Text(pDX, IDC_GPUINDEX, m_iGPUIndex);
  DDV_MinMaxInt(pDX, m_iGPUIndex, 0, 9);
  DDX_Text(pDX, IDC_MEMORYINDEX, m_iMemoryIndex);
  DDV_MinMaxInt(pDX, m_iMemoryIndex, 0, 9);
  DDX_Text(pDX, IDC_OTHERINDEX, m_iOtherIndex);
  DDV_MinMaxInt(pDX, m_iOtherIndex, 0, 9);
  DDX_Text(pDX, IDC_EDIT1, m_szFan1Alias);
  DDX_Text(pDX, IDC_EDIT2, m_szFan2Alias);
  DDX_Control(pDX, IDC_COMBO1, m_cNumFans);
  DDX_CBIndex(pDX, IDC_COMBO1, m_iNumFans);
  DDX_Text(pDX, IDC_EDIT3, m_iCPUCompensation);
  DDX_Text(pDX, IDC_EDIT4, m_iGPUCompensation);
  DDX_Text(pDX, IDC_EDIT5, m_iMemoryCompensation);
  DDX_Text(pDX, IDC_EDIT6, m_iChipsetCompensation);
  DDX_Text(pDX, IDC_EDIT7, m_iFan1SensorIndex);
  DDX_Text(pDX, IDC_EDIT8, m_iFan2SensorIndex);
}


BEGIN_MESSAGE_MAP(COptionsAdvanced, CDialog)
END_MESSAGE_MAP()


// COptionsAdvanced message handlers
// erst GUI-Elemente anlegen, dann Dialog anzeigen
BOOL COptionsAdvanced::OnInitDialog()
{
  CDialog::OnInitDialog();

  m_cNumFans.ResetContent();
  m_cNumFans.AddString( L"0" );
  m_cNumFans.AddString( L"1" );
  m_cNumFans.AddString( L"2" );

  m_iNumFans = options.numFans;
  m_iFan1Index = options.fan1Index;
  m_iFan2Index = options.fan2Index;
  m_iFanSpeedDivider = options.fanSpeedDivider;
  m_iCPUIndex = options.cpuSensorNo;
  m_iGPUIndex = options.gpuSensorNo;
  m_iMemoryIndex = options.memorySensorNo;
  m_iOtherIndex = options.chipsetSensorNo;
  m_szFan1Alias = options.fan1Alias;
  m_szFan2Alias = options.fan2Alias;
  m_iFan1SensorIndex = options.fan1SensorIndex;
  m_iFan2SensorIndex = options.fan2SensorIndex;

  m_iCPUCompensation = options.cpuCompensation;
  m_iGPUCompensation = options.gpuCompensation;
  m_iMemoryCompensation = options.memoryCompensation;
  m_iChipsetCompensation = options.chipsetCompensation;

  ((CWnd*)GetDlgItem(IDC_EDIT7))->EnableWindow( options.compatibleSensor );
  ((CWnd*)GetDlgItem(IDC_EDIT8))->EnableWindow( options.compatibleSensor );

  return true;
}

void COptionsAdvanced::getVars()
{
  options.numFans = m_iNumFans;
  options.fan1Index = m_iFan1Index;
  options.fan2Index = m_iFan2Index;

  if( m_iFanSpeedDivider > 0 )
    options.fanSpeedDivider = m_iFanSpeedDivider;

  options.cpuSensorNo = m_iCPUIndex;
  options.gpuSensorNo = m_iGPUIndex;
  options.memorySensorNo = m_iMemoryIndex;
  options.chipsetSensorNo = m_iOtherIndex;
  options.fan1Alias = m_szFan1Alias;
  options.fan2Alias = m_szFan2Alias;

  if( m_iFan1SensorIndex>=0 && m_iFan1SensorIndex<=3 )
    options.fan1SensorIndex = m_iFan1SensorIndex;

  if( m_iFan2SensorIndex>=0 && m_iFan2SensorIndex<=3 )
    options.fan2SensorIndex = m_iFan2SensorIndex;

  if( m_iCPUCompensation>=-25 && m_iCPUCompensation<=25 )
    options.cpuCompensation = m_iCPUCompensation;

  if( m_iGPUCompensation>=-25 && m_iGPUCompensation<=25 )
    options.gpuCompensation = m_iGPUCompensation;

  if( m_iMemoryCompensation>=-25 && m_iMemoryCompensation<=25 )
    options.memoryCompensation = m_iMemoryCompensation;

  if( m_iChipsetCompensation>=-25 && m_iChipsetCompensation<=25 )
    options.chipsetCompensation = m_iChipsetCompensation;
}

void COptionsAdvanced::fieldConstraints()
{
  ((CWnd*)GetDlgItem(IDC_EDIT7))->EnableWindow( options.compatibleSensor );
  ((CWnd*)GetDlgItem(IDC_EDIT8))->EnableWindow( options.compatibleSensor );
}
