/*
   I8kfanGUI V3.0
   Copyright(c) 2001-2006 Christian Diefer
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as 
   published by the Free Software Foundation.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "stdafx.h"
#include "AtlBase.h"
#include "i8kfangui.h"
#include "TOptions.h"
#include "I8kfanguiOptionDlg.h"
#include "OptionsCPUMon.h"
#include "I8kfanModify.h"
#include "I8kfanctrl.h"
#include "GlobalVars.h"
#include "CPUMonTab.h"
#include "GPUMonTab.h"
#include "MemoryMonTab.h"
#include "ChipMonTab.h"

// globale Instanzen der einzelnen Tabs:
CCPUMonTabDlg cpuTab;
CGPUMonTabDlg gpuTab;
CMemoryMonTabDlg memTab;
CChipMonTabDlg chipsetTab;

// COptionsCPUMon dialog

IMPLEMENT_DYNAMIC(COptionsCPUMon, CDialog)
COptionsCPUMon::COptionsCPUMon(CWnd* pParent /*=NULL*/)
	: CDialog(COptionsCPUMon::IDD, pParent)
  , m_iPreDef(3)
  , m_bHWSensor(FALSE)
  , m_bFanUsageBalancing(FALSE)
{ 
  *fileName = '\0';
  fa = false;
}

COptionsCPUMon::~COptionsCPUMon()
{ }

void COptionsCPUMon::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_CBIndex(pDX, IDC_PREDEFLIST, m_iPreDef);
  DDX_Check(pDX, IDC_HWFANS, m_bHWSensor);
  DDX_Check(pDX, IDC_USAGEBALANCE, m_bFanUsageBalancing);
  DDX_Control(pDX, IDC_TAB1, tabCtrl);
}

BEGIN_MESSAGE_MAP(COptionsCPUMon, CDialog)
  ON_BN_CLICKED(IDC_LOADBUTTON, OnBnClickedLoadbutton)
  ON_BN_CLICKED(IDC_MODIFYBUTTON, OnBnClickedModifybutton)
  ON_BN_CLICKED(IDC_SAVEBUTTON, OnBnClickedSavebutton)
  ON_CBN_SELCHANGE(IDC_PREDEFLIST, OnCbnSelchangePredeflist)
  ON_BN_CLICKED(IDC_HWFANS, OnBnClickedHwfans)
  ON_NOTIFY(TCN_SELCHANGE, IDC_TAB1, OnTcnSelchangeTab1)
END_MESSAGE_MAP()

//----------------------------------------------------
// COptionsCPUMon message handlers
//----------------------------------------------------
BOOL COptionsCPUMon::OnInitDialog()
{
	CDialog::OnInitDialog();

  fa = false;

  // Combobox mit Profilnamen im Tempmonitor fllen:
  CComboBox* cb = ((CComboBox*)GetDlgItem(IDC_PREDEFLIST));
  cb->ResetContent();

  for( int i=0; i<4; i++ )
    cb->AddString( options.ms[i].alias );
  
  m_iPreDef = options.preDefSet;
  m_bHWSensor = options.tempCtrlHWFans;

  CString s1;
  wchar_t text[64];
  TC_ITEM TabCtrlItem;
  TabCtrlItem.mask = TCIF_TEXT;
  s1.LoadStringW( IDS_CPUOPT1 );
  wcscpy_s( text, s1 );
  TabCtrlItem.pszText = text;
  tabCtrl.InsertItem( 0, &TabCtrlItem );
  TabCtrlItem.mask = TCIF_TEXT;
  s1.LoadStringW( IDS_CPUOPT2 );
  wcscpy_s( text, s1 );
  TabCtrlItem.pszText = text;
  tabCtrl.InsertItem( 1, &TabCtrlItem );
  s1.LoadStringW( IDS_CPUOPT3 );
  wcscpy_s( text, s1 );
  TabCtrlItem.pszText = text;
  tabCtrl.InsertItem( 2, &TabCtrlItem );
  s1.LoadStringW( IDS_CPUOPT4 );
  wcscpy_s( text, s1 );
  TabCtrlItem.pszText = text;
  tabCtrl.InsertItem( 3, &TabCtrlItem );

  tabs[0] = &cpuTab;
  tabs[1] = &gpuTab;
  tabs[2] = &memTab;
  tabs[3] = &chipsetTab;

  cpuTab.setParent( this );
  gpuTab.setParent( this );
  memTab.setParent( this );
  chipsetTab.setParent( this );

  // die Tabs anlegen:
  cpuTab.Create( IDD_CPUMON_TAB, this );
  gpuTab.Create( IDD_CPUMON_TAB, this );
  memTab.Create( IDD_CPUMON_TAB, this );
  chipsetTab.Create( IDD_CPUMON_TAB, this );

  // 'CPU'-Tab anzeigen, alle anderen ausblenden:
  cpuTab.ShowWindow( SW_SHOW );
  gpuTab.ShowWindow( SW_HIDE );
  memTab.ShowWindow( SW_HIDE );
  chipsetTab.ShowWindow( SW_HIDE );

  // Fensterposition und -gre berechnen:
  for( int i=0; i<4; i++ )
  {
    RECT rect;

    tabs[i]->GetClientRect( &rect );
    TabCtrl_AdjustRect( tabCtrl, 0, &rect );
    tabCtrl.ClientToScreen( &rect );
    ScreenToClient( &rect );

    tabs[i]->SetWindowPos( &wndTop,
                           rect.left,
                           rect.top,
                           rect.right - rect.left,
                           rect.bottom - rect.top + 22,
                           (i ? SWP_HIDEWINDOW : SWP_SHOWWINDOW) );

    // nur fr XP und hher (zerstrt Kompatibilitt mit Windows NT4 und 2000):
    if( glb_bWinXP && glb_pCallThemeEnable!=NULL )
      glb_pCallThemeEnable( tabs[i]->m_hWnd, ETDT_ENABLETAB );
  }

  setMonValues();

  cpuTab.UpdateData( FALSE );
  gpuTab.UpdateData( FALSE );
  memTab.UpdateData( FALSE );
  chipsetTab.UpdateData( FALSE );

  return TRUE;
}

//----------------------------------------------------
//----------------------------------------------------
void COptionsCPUMon::OnTcnSelchangeTab1( NMHDR *pNMHDR, LRESULT *pResult )
{
  int i = tabCtrl.GetCurSel();

  for( int x=0; x<4; x++ )
    tabs[x]->ShowWindow( x==i ? SW_SHOW : SW_HIDE );

  *pResult = 0;
}

//----------------------------------------------------
//----------------------------------------------------
//----------------------------------------------------

void COptionsCPUMon::OnBnClickedLoadbutton()
{
  UpdateData( FALSE );
}

void COptionsCPUMon::OnBnClickedModifybutton()
{
  CI8kfanModify dlgModify;
  dlgModify.DoModal();

  CComboBox* cb = ((CComboBox*)GetDlgItem(IDC_PREDEFLIST));
  cb->ResetContent();

  for( int i=0; i<4; i++ )
    cb->AddString( options.ms[i].alias );
  
  UpdateData( FALSE );
  OnCbnSelchangePredeflist();
}

void COptionsCPUMon::OnBnClickedSavebutton()
{
}

void COptionsCPUMon::OnCbnSelchangePredeflist()
{
  UpdateData();
  UpdateTabData();
  cpuTab.getVars();
  gpuTab.getVars();
  memTab.getVars();
  chipsetTab.getVars();
  setMonValues();
}

void COptionsCPUMon::OnBnClickedHwfans()
{
  UpdateData();

  if( m_bHWSensor && options.sensorCheck )
  {
    CString s1, s2;
    s1.LoadStringW( IDS_AUTO2 );
    s2.LoadStringW( IDS_WARN );
    if( MessageBoxW(s1,s2,MB_ICONEXCLAMATION|MB_YESNO) == IDYES )
    {
      if( !detectFanSensor(glb_hHndFile) )
      {
        s1.LoadStringW( IDS_AUTO3 );
        s2.LoadStringW( IDS_ERROR );
        MessageBoxW( s1, s2, MB_ICONSTOP|MB_OK );
        m_bHWSensor = FALSE;
        UpdateData( FALSE );
        return;
      }
      else
      {
        s1.LoadStringW( IDS_AUTO4 );
        s2.LoadStringW( IDS_INFO );
        MessageBoxW( s1, s2, MB_ICONINFORMATION|MB_OK );
        options.sensorCheck = false;
        options.compatibleSensor = true;
        parent->hwSensorFound();
      }
    }
    else
    {
      m_bHWSensor = FALSE;
      UpdateData( FALSE );
      return;
    }
  }

  sensorSwitch();
}

void COptionsCPUMon::setMonValues()
{
  cpuTab.setValues( &options.ms[m_iPreDef] );
  gpuTab.setValues( &options.ms[m_iPreDef] );
  memTab.setValues( &options.ms[m_iPreDef] );
  chipsetTab.setValues( &options.ms[m_iPreDef] );
}

void COptionsCPUMon::sensorSwitch()
{
  cpuTab.sensorSwitch( m_bHWSensor );
  gpuTab.sensorSwitch( m_bHWSensor );
  memTab.sensorSwitch( m_bHWSensor );
  chipsetTab.sensorSwitch( m_bHWSensor );
}

void COptionsCPUMon::changeUnits( BOOL fa )
{
  cpuTab.changeUnits( fa );
  gpuTab.changeUnits( fa );
  memTab.changeUnits( fa );
  chipsetTab.changeUnits( fa );
}

void COptionsCPUMon::getVars()
{
  options.preDefSet = m_iPreDef;
  options.tempCtrlHWFans = m_bHWSensor;
  options.fanUsageBalancing = m_bFanUsageBalancing;

  cpuTab.getVars();
  gpuTab.getVars();
  memTab.getVars();
  chipsetTab.getVars();
}

void COptionsCPUMon::UpdateTabData()
{
  cpuTab.UpdateData();
  gpuTab.UpdateData();
  memTab.UpdateData();
  chipsetTab.UpdateData();
}
