/***************************************************************
* C file:  Speed.c
*
*   This program has been developed by Intel Corporation.  
*		You have Intel's permission to incorporate this code 
*   into your product, royalty free.  
*
*  Copyright (c) 1995, Intel Corporation.  All rights reserved.
*
*  Runtime optimizations & modifications for I8kfanGUI 
*    by C. Diefer
***************************************************************/
 
#include "stdafx.h"
#include <stdio.h>
#include <math.h>
#include <limits.h>

#include "speed.h"
#include "cpuid.h"

#define ROUND_THRESHOLD 6

struct FREQ_INFO GetRDTSCCpuSpeed()
{
  UINT64 stamp0=0, stamp1=0;         // Time Stamp Variable for beginning and end of test
  struct FREQ_INFO cpu_speed;
  LARGE_INTEGER t0,t1;              // Variables for High-Resolution Performance Counter reads
  ulong freq  =0;                   // Most current frequ. calculation
  ulong freq2 =0;                   // 2nd most current frequ. calc.
  ulong freq3 =0;                   // 3rd most current frequ. calc.
  ulong total;                      // Sum of previous three frequency calculations
  int tries=0;                      // Number of times a calculation has been made on this call to cpuspeed
  ulong total_cycles=0, cycles=0;   // Clock cycles elapsed during test
  ulong total_ticks=0, ticks;       // Microseconds elapsed during test
  LARGE_INTEGER count_freq;         // High Resolution Performance Counter frequency

#ifdef WIN32
  int iPriority;
  HANDLE hThread = GetCurrentThread();
#endif


  memset( &cpu_speed, 0x00, sizeof(cpu_speed) );

  if( !QueryPerformanceFrequency(&count_freq) )
    return cpu_speed;

  // On processors supporting the Read Time Stamp opcode, compare elapsed time on the 
  //   High-Resolution Counter with elapsed cycles on the Time Stamp Register.
    
  // This do loop runs up to 20 times or until the average of the previous three calculated frequencies is 
  //   within 1 MHz of each of the individual calculated frequencies. This resampling increases the 
  //   accuracy of the results since outside factors could affect this calculation
  do 
  {          
    tries++;        // Increment number of times sampled on this call to cpuspeed
            
    freq3 = freq2;  // Shift frequencies back to make room for new frequency measurement
    freq2 = freq;

    QueryPerformanceCounter( &t0 );   // Get high-resolution performance counter time
   
   t1 = t0;
 //   t1.LowPart = t0.LowPart;          // Set Initial time
 //   t1.HighPart = t0.HighPart;

#ifdef WIN32
    iPriority = GetThreadPriority( hThread );
    if( iPriority != THREAD_PRIORITY_ERROR_RETURN )
      SetThreadPriority( hThread, THREAD_PRIORITY_TIME_CRITICAL );
#endif // WIN32

    while( (t1-t0) < 50 ) 
    {     
      // Loop until 50 ticks have passed since last read of hi-res counter. 
      //   This accounts for overhead later.
      QueryPerformanceCounter( &t1 );

      _asm {
         push ebx
         push edx
         xor eax,eax
         cpuid
         rdtsc
		     mov dword ptr [stamp0],eax
		     mov dword ptr [stamp0+4],edx
		     pop edx
         pop ebx
//      RDTSC;
//      _asm {
//          MOV stamp0, EAX
      }
    }   
            
    t0 = t1;  
//    t0.LowPart = t1.LowPart;        // Reset Initial Time
//    t0.HighPart = t1.HighPart;

    while( (t1-t0) < 1000 ) 
    {
      // Loop until 1000 ticks have passed since last read of hi-res counter. 
      //   This allows for elapsed time for sampling.
      QueryPerformanceCounter(&t1);
  
      _asm {
         push ebx
         push edx
         xor eax,eax
         cpuid
         rdtsc
		     mov dword ptr [stamp1],eax
		     mov dword ptr [stamp1+4],edx
		     pop edx
         pop ebx
//      RDTSC;                      // Read Time Stamp
//      __asm {
           MOV stamp1, EAX
      }
    }

#ifdef WIN32
    // Reset priority
    if( iPriority != THREAD_PRIORITY_ERROR_RETURN )
      SetThreadPriority( hThread, iPriority );
#endif

    // Number of internal clock cycles is difference between two time stamp readings.
    cycles = stamp1 - stamp0;   

    // Number of external ticks is difference between two hi-res counter reads.
    ticks = t1 - t0;


    // Note that some seemingly arbitrary mulitplies and divides are done below. This is to maintain a 
    //   high level of precision without truncating the most significant data. According to what value 
    //   ITERATIIONS is set to, these multiplies and divides might need to be shifted for optimal precision.

    // Convert ticks to hundred thousandths of a tick
    ticks = ticks * 100000;

    // Hundred Thousandths of a Ticks / ( 10 ticks/second ) = microseconds (us)
    ticks = ticks / ( count_freq.LowPart/10 );      

    total_ticks += ticks;
    total_cycles += cycles;

    if( ticks%count_freq.LowPart > count_freq.LowPart/2 )
      ticks++;            // Round up if necessary

    freq = cycles/ticks;    // Cycles / us  = MHz

    if( cycles%ticks > ticks/2 )
      freq++;             // Round up if necessary

    total = ( freq + freq2 + freq3 );     // Total last three frequency calculations
  } while( (tries < 3 ) ||        
           (tries < 20)&&
           ((abs(3 * (long)freq - (long)total) > 3*TOLERANCE )||
            (abs(3 * (long)freq2 - (long)total) > 3*TOLERANCE )||
            (abs(3 * (long)freq3 - (long)total) > 3*TOLERANCE )));
    // ... Compare last three calculations to average of last three calculations.                           

  // Try one more significant digit.
  freq3 = ( total_cycles * 10 ) / total_ticks;
  freq2 = ( total_cycles * 100 ) / total_ticks;

  if( freq2 - (freq3 * 10) >= ROUND_THRESHOLD )
    freq3++;

  cpu_speed.raw_freq = total_cycles / total_ticks;
  cpu_speed.norm_freq = cpu_speed.raw_freq;

  freq = cpu_speed.raw_freq * 10;
  if( (freq3 - freq) >= ROUND_THRESHOLD )
    cpu_speed.norm_freq++;

  cpu_speed.ex_ticks = total_ticks;
  cpu_speed.in_cycles = total_cycles;

  return cpu_speed;
}
