/*
   I8kfanGUI V3.0
   Copyright(c) 2001-2006 Christian Diefer
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as 
   published by the Free Software Foundation.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef TOPTIONS_H
#define TOPTIONS_H

#define MAXDRIVES 4

#define CELSIUS    0
#define FAHRENHEIT 1

#define CPUMON 0
#define GPUMON 1
#define MEMMON 2
#define CHSMON 3

// Operationsmodi:
#define OBSERVERMODE  0
#define MANUALCONTROL 1
#define TEMPMONITOR   2
#define FANFORCING    3

static wchar_t regdummy[256];

class TMonSettings { 
public:
  wchar_t alias[128];

  // fr jeden Sensor ein Flag, ob Sensor aktiv ist:
  // [0] = CPU-Monitor aktiv
  // [1] = GPU-Monitor aktiv
  // [2] = Memory-Monitor aktiv
  // [3] = Chipset-Monitor aktiv
  BOOL monEnabled[4];

  // fr jeden Sensor ein Array mit 8 Elementen:
  // [0] = Turn on 1
  // [1] = Turn off 1
  // [2] = Turn on 2
  // [3] = Turn off 2
  // [4] = Turn on 3
  // [5] = Turn off 3
  // [6] = Turn on 4
  // [7] = Turn off 4
  int cpuMonTemp[8];
  int gpuMonTemp[8];
  int memoryMonTemp[8];
  int chipsetMonTemp[8];

  // hier wird fr jeden Sensor vermerkt, welcher Lfter jeweils
  // fr den Sensor aktiviert ist:
  BOOL cpuFan[2];     // [0]=TRUE: Fan1 fr CPU, [1]=TRUE: Fan2 fr CPU
  BOOL gpuFan[2];     // [0]=TRUE: Fan1 fr GPU, [1]=TRUE: Fan2 fr GPU
  BOOL memoryFan[2];  // [0]=TRUE: Fan1 fr Memory, [1]=TRUE: Fan2 fr Memory
  BOOL chipsetFan[2]; // [0]=TRUE: Fan1 fr Chipset, [1]=TRUE: Fan2 fr Chipset

  // Dies sind die Arrays fr die Lfterstati. Fr jeden Sensor
  // kann es maximal 2 Lfter mit jeweils 4 Stufen geben. Zustzlich
  // gibt es noch die Mglichkeit, da fr jeden Lfter noch Sensordrehzahlen
  // angegeben werden, falls ein HW-Sensor gefunden wurde. Deshalb hat jedes
  // Array vier Dimensionen:
  // [0-3] = Stati fr Lfter 1 (SMBIOS)
  // [4-7] = Stati fr Lfter 2 (SMBIOS)
  // [8-11] = Sensordrehzahlen fr Lfter 1 (HW-Sensor)
  // [12-15] = Sensordrehzahlen fr Lfter 2 (HW-Sensor)
  int cpuFanSpeeds[16];
  int gpuFanSpeeds[16];
  int memoryFanSpeeds[16];
  int chipsetFanSpeeds[16];
};

class TOptions {
public:
  TOptions();
  bool setRegKeys();
  bool getRegKeys();
  int convertTemp( int temp, int target );
  void fillList( wchar_t* t, CListCtrl* m_cList );

  int opmode;
  BOOL autoStart;
  DWORD autoInnerSlider;
  DWORD autoOuterSlider;
  BOOL terminate;
  BOOL acOnly;
  BOOL readCPUTemp;
  BOOL readFanSpeed1;
  BOOL readFanSpeed2;
  BOOL readFanStatus1;
  BOOL readFanStatus2;
  BOOL tempIcon;
  BOOL showDiagram;
  BOOL debugMode;
  DWORD intervalCPU;
  DWORD intervalHD;
  DWORD preDefSet;
  TMonSettings ms[4];
  BOOL readGPUTemp;
  BOOL readMemoryTemp;
  BOOL readChipsetTemp;
  BOOL readHDTemp;
  BOOL readHDTemp2;
  BOOL minimizeOnClose;
  BOOL speedIcon;
  BOOL readCPUSpeed;
  BOOL readCPULoad;
  BOOL sensorSupport;
  BOOL sensorCheck;
  BOOL fahrenHeit;
  BOOL maintainSpeeds;
  int readCPUVar;
  int hdNo;
  int hdNo2;
  int hds[MAXDRIVES];
  wchar_t biosVer[32];
  int trayIcon1;
  int trayIcon2;
  BOOL drawCPUTemp;
  BOOL drawGPUTemp;
  BOOL drawMemoryTemp;
  BOOL drawChipsetTemp;
  BOOL drawCPULoad;
  BOOL drawCPUSpeed;
  DWORD speedScaling;
  BOOL debugControls;
  BOOL compatibleSensor;
  int autoSpeedDirect1;
  int autoSpeedDirect2;
  BOOL autoFanSetting;
  BOOL autoStartHWFans;
  BOOL tempCtrlHWFans;
  BOOL fanUsageBalancing;
  BOOL i8100Fix;
  BOOL modelChecked;
  int numFans;
  int fan1Index;
  int fan2Index;
  int fanSpeedDivider;
  int cpuSensorNo;
  int gpuSensorNo;
  int memorySensorNo;
  int chipsetSensorNo;
  CString fan1Alias;
  CString fan2Alias;
  DWORD transparentBkg;
  DWORD trayColor1;
  DWORD trayColor2;
  BOOL statusIcon2;
  DWORD transparentBkgB;
  DWORD trayColor1b;
  DWORD trayColor2b;
  int trayIcon1b;
  int trayIcon2b;
  DWORD fanTransparentBkg;
  DWORD fanTrayColor1;
  DWORD fanTrayColor2;
  DWORD fanTraySpeedDivider;
  BOOL fanIconStateOnly;
  int cpuCompensation;
  int gpuCompensation;
  int memoryCompensation;
  int chipsetCompensation;
  int fan1SensorIndex;
  int fan2SensorIndex;
  int fanForceInterval;
  BOOL fan1Force;
  BOOL fan2Force;
  int colorTheme;

private:
  bool writeMonSet( int no );
  bool readMonSet( int no );
  bool writeIntList( CRegKey& key, int* ptr, int d, wchar_t* keyName );
  bool readIntList( CRegKey& key, int* ptr, int d, wchar_t* keyName );
};

#endif
