/*
   I8kfanGUI V3.0
   Copyright(c) 2001-2006 Christian Diefer

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as 
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "stdafx.h"
#include "AtlBase.h"
#include "TOptions.h"
#include "GlobalVars.h"
#include "i8kfangui.h"
#include "TStringTokenizer.h"

//---------------------------------------------------------------------------
TStringTokenizer::TStringTokenizer( wchar_t* s, bool c )
{
  line = s;
  checkStrings = c;
  delimiters = NULL;
  setDelimiters( L" \t\r\n" );
  pointer = line;
  ret = NULL;
}

//---------------------------------------------------------------------------
TStringTokenizer::~TStringTokenizer()
{
  if( delimiters )
    delete[] delimiters;

  if( ret )
    delete[] ret;	
}

//---------------------------------------------------------------------------
void TStringTokenizer::setDelimiters( wchar_t* s )
{
  if( delimiters )
    delete[] delimiters;

  delimiters = new wchar_t[wcslen(s)+4];
  wcscpy_s( delimiters, wcslen(s)+4, s ); 	
}

//---------------------------------------------------------------------------
bool TStringTokenizer::isDelimiter( wchar_t c )
{
  if( !delimiters )
    return false;
   
  for( int i=0; i<(int)wcslen(delimiters); i++ )
    if( c == delimiters[i] )
      return true;
  
  return false;	
}

//---------------------------------------------------------------------------
void TStringTokenizer::setString( wchar_t* s )
{
  line = s;
  pointer = line;

  if( ret ) 
  {
    delete[] ret;
    ret = NULL;
  }
}

//---------------------------------------------------------------------------
wchar_t* TStringTokenizer::getNextToken( wchar_t** u, wchar_t** v )
{
  bool inStr = false;
  wchar_t strChar;
  
  if( ret ) 
  {
    delete[] ret;
    ret = NULL;
  }

  wchar_t* s = pointer;
  while( isDelimiter(*s) ) s++;
  
  if( *s )	// String noch nicht zuende
  {
    wchar_t* t = s;
  
    if( u != NULL )	// Beginn im Originalstring zurckmelden
      *u = t;
      
    while( *t )		// Token durchgehen
    {
      if( checkStrings && (*t=='\'' || *t=='"') )
      {
      	if( !inStr )			// Stringanfang
      	{
      	  inStr = true;
      	  strChar = *t;	
      	}
      	else if( inStr && *t==strChar )	// Stringende
          inStr = false;          
        
        t++;
      }
      else if( !inStr && isDelimiter(*t) ) 
        break;
      else
        t++;
    }
    
    if( v != NULL )
      *v = t;
    
    ret = new wchar_t[t-s+8];    
    wcsncpy_s( ret, t-s+8, s, t-s );
    ret[t-s] = '\0';
    pointer = t;
    return ret;	
  } 
  
  return NULL;	
}

//---------------------------------------------------------------------------
