/*
   I8kfanGUI V3.0
   Copyright(c) 2001-2006 Christian Diefer

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as 
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "stdafx.h"
#include "AtlBase.h"
#include "i8kfangui.h"
#include "TempControl.h"
#include "TOptions.h"
#include "i8kfanctrl.h"
#include "GlobalVars.h"
#include "CoreTemp.h"

//---------------------------------------------------------------
// Folgende Daten einlesen:
// - CPU-Last
// - CPU-Speed
// - CPU-Temperatur
// - GPU-Temperatur
// - Speichertemperatur
// - Chipsettemperatur
//---------------------------------------------------------------
void CTempControl::readTempData( BOOL init )
{
  static int oldUsage = 0;
  bool cpuSpeedError = false;
  bool cpuTempError = false;
  bool gpuTempError = false;
  bool memTempError = false;
  bool chipTempError = false;


  // CPU-Last einlesen
  if( options.readCPULoad )
  {
    DWORD cpuUsage = ((glb_ProcGetSystemTimes!=NULL) ? readCPULoadGST() : readCPULoad());

    if( cpuUsage <= 100 )
    {
      glb_szCPULoad.Format( _T("%d"), cpuUsage );
      oldUsage = cpuUsage;
    }
    else 
    {
      cpuUsage = oldUsage;

      if( init )
        glb_szCPULoad.Format( _T("%d"), cpuUsage );
    }

    glb_iCPULoad = cpuUsage;
    glb_iaLoadArr[glb_iDiagramIndex] = (UCHAR)cpuUsage;
  }
  else
  {
    glb_iCPULoad = 0;
    glb_szCPULoad = "---";
    glb_iaLoadArr[glb_iDiagramIndex] = 0;
  }

  // CPU-Speed einlesen
  if( options.readCPUSpeed && !init )
  {
    wchar_t speed[16];

    if( !getCPUSpeed(speed,options.readCPUVar,glb_hHndFile) )
    {
      log( L"CPU speed reading error" );
      options.readCPUSpeed = false;

      glb_szCPUSpeed = L"---";
      glb_iaSpeedArr[glb_iDiagramIndex] = 0;

      glb_iCPUSpeed = 0;
      cpuSpeedError = true;
    }
    else
    {
      glb_szCPUSpeed = speed;
      glb_iCPUSpeed = _wtoi( speed );

      int t = glb_iCPUSpeed / 100;
      if( _wtoi(speed)%100 >= 90 )
        t++;

      glb_iaSpeedArr[glb_iDiagramIndex] = t;
    }
  }
  else
  {
    glb_iCPUSpeed = 0;
    glb_szCPUSpeed = L"---";
    glb_iaSpeedArr[glb_iDiagramIndex] = 0;
  }

  // CPU-Temperatur einlesen
  if( options.readCPUTemp )
  {
    int cpuTemp;

    if( glb_intelDTSTempSupport )
    {
      // muss fllen: glb_iCPUTempC1, glb_iCPUTempC2, glb_szCPUTempC1, glb_szCPUTempC2, cpuTemp
      int highestTemp = readCoreTemps();
      cpuTemp = highestTemp;
    }
    else
    {
      int errCount;

      for( errCount=0; errCount<5; errCount++ ) 
      {
        if( readTemp(options.cpuSensorNo,cpuTemp,glb_hHndFile) ) 
          break;
        log( L"CPU temp reading error #%d - pausing", errCount+1 );
        Sleep( 50 );
      }
 
      if( errCount == 5 )
      {  
        log( L"Permanent CPU temp reading error" );
        options.readCPUTemp = false;      // nicht nochmal versuchen
      
        if( options.opmode == TEMPMONITOR )          // Tempmonitor ausschalten
          options.opmode = OBSERVERMODE;
      
        glb_iCPUTemp = 0;
        cpuTemp = 0;

        glb_szCPUTemp = L"---";
        glb_iaTempArr[glb_iDiagramIndex] = 0;

        cpuTempError = true;
      }
      else
        cpuTemp += options.cpuCompensation;
    }
    
    if( cpuTemp >= 128 )     // alten Wert weiterverwenden, wenn auerhalb Bereich
      cpuTemp = glb_iCPUTemp;

    glb_iCPUTemp = cpuTemp;
    int t = cpuTemp;
    if( options.fahrenHeit )
      t = options.convertTemp( glb_iCPUTemp, FAHRENHEIT );
    glb_szCPUTemp.Format( L"%d", t );

    glb_iaTempArr[glb_iDiagramIndex] = cpuTemp;
  }
  else
  {
    glb_iCPUTemp = 0;
    glb_szCPUTemp = L"---";
    glb_iaTempArr[glb_iDiagramIndex] = 0;
  }

  // GPU-Temperatur einlesen
  if( options.readGPUTemp )
  {
    int gpuTemp;
    int errCount;

    for( errCount=0; errCount<5; errCount++ ) 
    {
      if( readTemp(options.gpuSensorNo,gpuTemp,glb_hHndFile) ) 
        break;
      log( L"GPU temp reading error #%d - pausing", errCount+1 );
      Sleep( 50 );
    }

    if( errCount == 5 )
    {
      log( L"Permanent GPU temp reading error" );
      options.readGPUTemp = false;      // nicht nochmal versuchen
      glb_iGPUTemp = 0;
      gpuTemp = 0;

      glb_szGPUTemp = L"---";
      glb_iaGPUArr[glb_iDiagramIndex] = 0;
      
      gpuTempError = true;
    }
    else
      gpuTemp += options.gpuCompensation;

    if( gpuTemp >= 128 )     // alten Wert weiterverwenden, wenn auerhalb Bereich
      gpuTemp = glb_iGPUTemp;

    glb_iGPUTemp = gpuTemp;
    int t = gpuTemp;
    if( options.fahrenHeit )
      t = options.convertTemp( glb_iGPUTemp, FAHRENHEIT );
    glb_szGPUTemp.Format( L"%d", t );

    glb_iaGPUArr[glb_iDiagramIndex] = gpuTemp;
  }
  else
  {
    glb_iGPUTemp = 0;
    glb_szGPUTemp = L"---";
    glb_iaGPUArr[glb_iDiagramIndex] = 0;
  }

  // Speichertemperatur einlesen
  if( options.readMemoryTemp )
  {
    int memoryTemp;
    int errCount;

    for( errCount=0; errCount<5; errCount++ ) 
    {
      if( readTemp(options.memorySensorNo,memoryTemp,glb_hHndFile) )
        break;
      log( L"Memory temp reading error #%d - pausing", errCount+1 );
      Sleep( 50 );
    }

    if( errCount == 5 )
    {
      log( L"Permanent memory temp reading error" );
      options.readMemoryTemp = false;      // nicht nochmal versuchen
      glb_iMemoryTemp = 0;
      memoryTemp = 0;

      glb_szMemoryTemp = L"---";
      glb_iaMemArr[glb_iDiagramIndex] = 0;

      memTempError = true;
    }
    else
      memoryTemp += options.memoryCompensation;

    if( memoryTemp >= 128 )     // alten Wert weiterverwenden, wenn auerhalb Bereich
      memoryTemp = glb_iMemoryTemp;

    glb_iMemoryTemp = memoryTemp;
    int t = memoryTemp;
    if( options.fahrenHeit )
      t = options.convertTemp( glb_iMemoryTemp, FAHRENHEIT );
    glb_szMemoryTemp.Format( L"%d", t );

    glb_iaMemArr[glb_iDiagramIndex] = memoryTemp;
  }
  else
  {
    glb_iMemoryTemp = 0;
    glb_szMemoryTemp = L"---";
    glb_iaMemArr[glb_iDiagramIndex] = 0;
  }

  // Chipset-Temperatur einlesen
  if( options.readChipsetTemp )
  {
    int chipsetTemp;
    int errCount;

    for( errCount=0; errCount<5; errCount++ ) 
    {
      if( readTemp(options.chipsetSensorNo,chipsetTemp,glb_hHndFile) )
        break;
      log( L"Chipset temp reading error #%d - pausing", errCount+1 );
      Sleep( 50 );
    }

    if( errCount == 5 )
    {
      log( L"Permanent chipset temp reading error" );
      options.readChipsetTemp = false;      // nicht nochmal versuchen
      glb_iChipsetTemp = 0;
      chipsetTemp = 0;

      glb_szChipsetTemp = L"---";
      glb_iaChipArr[glb_iDiagramIndex] = 0;

      chipTempError = true;
    }
    else
      chipsetTemp += options.chipsetCompensation;

    if( chipsetTemp >= 128 )     // alten Wert weiterverwenden, wenn auerhalb Bereich
      chipsetTemp = glb_iChipsetTemp;

    glb_iChipsetTemp = chipsetTemp;
    int t = chipsetTemp;
    if( options.fahrenHeit )
      t = options.convertTemp( glb_iChipsetTemp, FAHRENHEIT );
    glb_szChipsetTemp.Format( L"%d", t );

    glb_iaChipArr[glb_iDiagramIndex] = chipsetTemp;
  }
  else
  {
    glb_iChipsetTemp = 0;
    glb_szChipsetTemp = L"---";
    glb_iaChipArr[glb_iDiagramIndex] = 0;
  }

  if( cpuSpeedError )
  {
    CString s1, s2;
    s1.LoadStringW( IDS_TEMP1 );
    s2.LoadStringW( IDS_ERROR );
    MessageBoxW( NULL, s1, s2, MB_ICONEXCLAMATION|MB_OK );
  }

  if( cpuTempError )
  {
    CString s1, s2;
    s1.LoadStringW( IDS_TEMP2 );
    s2.LoadStringW( IDS_ERROR );
    MessageBoxW( NULL, s1, s2, MB_ICONEXCLAMATION|MB_OK );
  }

  if( gpuTempError )
  {
    CString s1, s2;
    s1.LoadStringW( IDS_TEMP3 );
    s2.LoadStringW( IDS_ERROR );
    MessageBoxW( NULL, s1, s2, MB_ICONEXCLAMATION|MB_OK );
  }

  if( memTempError )
  {
    CString s1, s2;
    s1.LoadStringW( IDS_TEMP4 );
    s2.LoadStringW( IDS_ERROR );
    MessageBoxW( NULL, s1, s2, MB_ICONEXCLAMATION|MB_OK );
  }

  if( chipTempError )
  {
    CString s1, s2;
    s1.LoadStringW( IDS_TEMP5 );
    s2.LoadStringW( IDS_ERROR );
    MessageBoxW( NULL, s1, s2, MB_ICONEXCLAMATION|MB_OK );
  }
}

//---------------------------------------------------------------
// Auslesen der folgenden Lfterdaten:
// - Lfter #1 Drehzahl in RPM ber SMBIOS
// - Lfter #1 Status ber SMBIOS
// - Lfter #1 Status ber HW-Sensor
// - Lfter #2 Drehzahl in RPM ber SMBIOS
// - Lfter #2 Status ber SMBIOS
// - Lfter #2 Status ber HW-Sensor
// - Umdrehung der Lfterdrehzahlen fr I8100 BIOS<=A03 Buf
//---------------------------------------------------------------
int CTempControl::readFanStates()
{
  if( options.readFanSpeed1 )
  {
    int rpm;
    int errCount;

    for( errCount=0; errCount<5; errCount++ ) 
    {
      if( readFanSpeed(options.fan1Index,rpm,glb_hHndFile) )
        break;
      log( L"Fan#1 rpm reading error #%d - pausing", errCount+1 );
      Sleep( 50 );
    }

    if( errCount == 5 )
    {
      log( L"Permanent error reading fan 1 rpm" );
      glb_szFanSpeed1 = "---";
      options.readFanSpeed1 = false;
      glb_iaFan1Speed[glb_iDiagramIndex] = 0;
      CString s1, s2;
      s1.LoadStringW( IDS_TEMP6 );
      s2.LoadStringW( IDS_ERROR );
      MessageBoxW( NULL, s1, s2, MB_ICONEXCLAMATION|MB_OK );
    }
    else
    {
      rpm /= options.fanSpeedDivider;
      glb_iFanSpeed1 = rpm;
      glb_iaFan1Speed[glb_iDiagramIndex] = rpm;
      glb_szFanSpeed1.Format( L"%d",glb_iFanSpeed1 );
    }
  }
  else 
  {
    glb_szFanSpeed1 = L"---";
    glb_iaFan1Speed[glb_iDiagramIndex] = 0;
  }

  if( options.readFanStatus1 )
  {
    int si;
    int errCount;

    for( errCount=0; errCount<5; errCount++ ) 
    {
      if( readFanState(options.fan1Index,si,glb_hHndFile) )
        break;
      log( L"Fan#1 state reading error #%d - pausing", errCount+1 );
      Sleep( 50 );
    }

    if( errCount == 5 )
    {
      log( L"Permanent error reading fan 1 state" );
      options.readFanStatus1 = false;
      si = 3;
      CString s1, s2;
      s1.LoadStringW( IDS_TEMP7 );
      s2.LoadStringW( IDS_ERROR );
      MessageBoxW( NULL, s1, s2, MB_ICONEXCLAMATION|MB_OK );
    }
    else
      stat1 = si;

    CString s1;

    switch( si )
    {
      case 0: s1.LoadStringW( IDS_TEMP10 );
              glb_szSpeedString1 = s1;
              glb_iFanStatus1 = 0;
              if( !options.readFanSpeed1 ) glb_iaFan1Speed[glb_iDiagramIndex] = 0;
              break;

      case 1: s1.LoadStringW( IDS_TEMP11 ); 
              glb_szSpeedString1 = s1;
              if( !options.readFanSpeed1 ) glb_iaFan1Speed[glb_iDiagramIndex] = 1;
              glb_iFanStatus1 = 1;
              break;

      case 2: s1.LoadStringW( IDS_TEMP12 ); 
              glb_szSpeedString1 = s1;
              if( !options.readFanSpeed1 ) glb_iaFan1Speed[glb_iDiagramIndex] = 2;
              glb_iFanStatus1 = 2;
              break;

      default:s1.LoadStringW( IDS_TEMP13 ); 
              glb_szSpeedString1.Format( s1, si );
              glb_iFanStatus1 = -1;
              if( !options.readFanSpeed1 ) glb_iaFan1Speed[glb_iDiagramIndex] = 0;
    }
  }
  else
  {
    glb_szSpeedString1 = L"---";
    glb_iFanStatus1 = -1;
    if( !options.readFanSpeed1 ) glb_iaFan1Speed[glb_iDiagramIndex] = 0;
  }

  if( options.readFanSpeed2 )
  {
    int rpm;
    int errCount;

    for( errCount=0; errCount<5; errCount++ ) 
    {
      if( readFanSpeed(options.fan2Index,rpm,glb_hHndFile) )
        break;
      log( L"Fan#2 rpm reading error #%d - pausing", errCount+1 );
      Sleep( 50 );
    }

    if( errCount == 5 )
    {
      log( L"Permanent error reading fan 2 rpm" );
      glb_szFanSpeed2 = L"---";
      options.readFanSpeed2 = false;
      glb_iaFan2Speed[glb_iDiagramIndex] = 0;
      CString s1, s2;
      s1.LoadStringW( IDS_TEMP8 );
      s2.LoadStringW( IDS_ERROR );
      MessageBoxW( NULL, s1, s2, MB_ICONEXCLAMATION|MB_OK );
    }
    else
    {
      rpm /= options.fanSpeedDivider;
      glb_iFanSpeed2 = rpm;
      glb_iaFan2Speed[glb_iDiagramIndex] = rpm;
      glb_szFanSpeed2.Format( L"%d",glb_iFanSpeed2 );
    }
  }
  else
  {
    glb_szFanSpeed2 = L"---";
    glb_iaFan2Speed[glb_iDiagramIndex] = 0;
  }

  if( options.readFanStatus2 )
  {
    int si;
    int errCount;

    for( errCount=0; errCount<5; errCount++ ) 
    {
      if( readFanState(options.fan2Index,si,glb_hHndFile) )
        break;
      log( L"Fan#2 state reading error #%d - pausing", errCount+1 );
      Sleep( 50 );
    }

    if( errCount == 5 )
    {
      log( L"Permanent error reading fan 2 state" );
      options.readFanStatus2 = false;
      si = 3;
      CString s1, s2;
      s1.LoadStringW( IDS_TEMP9 );
      s2.LoadStringW( IDS_ERROR );
      MessageBoxW( NULL, s1, s2, MB_ICONEXCLAMATION|MB_OK );
    }
    else
      stat2 = si;

    CString s1;

    switch( si )
    {
      case 0: s1.LoadStringW( IDS_TEMP10 );
              glb_szSpeedString2 = s1;
              glb_iFanStatus2 = 0;
              if( !options.readFanSpeed2 ) glb_iaFan2Speed[glb_iDiagramIndex] = 0;
              break;

      case 1: s1.LoadStringW( IDS_TEMP11 );
              glb_szSpeedString2 = s1;
              glb_iFanStatus2 = 1;
              if( !options.readFanSpeed2 ) glb_iaFan2Speed[glb_iDiagramIndex] = 1;
              break;

      case 2: s1.LoadStringW( IDS_TEMP12 );
              glb_szSpeedString2 = s1;
              glb_iFanStatus2 = 2;
              if( !options.readFanSpeed2 ) glb_iaFan2Speed[glb_iDiagramIndex] = 2;
              break;

      default:s1.LoadStringW( IDS_TEMP13 );
              glb_szSpeedString2.Format( s1, si );
              glb_iFanStatus2 = -1;
              if( !options.readFanSpeed2 ) glb_iaFan2Speed[glb_iDiagramIndex] = 0;
    }
  }
  else
  {
    glb_szSpeedString2 = L"---";
    glb_iFanStatus2 = -1;
    if( !options.readFanSpeed2 ) glb_iaFan2Speed[glb_iDiagramIndex] = 0;
  }

  glb_iSensorSpeed1 = 0;
  glb_iSensorSpeed2 = 0;

  if( options.compatibleSensor )
  {
    if( options.readFanSpeed1 )
    {
      int speed1;
      if( getFanSpeedSensor(options.fan1SensorIndex,speed1,glb_hHndFile) )
        glb_iSensorSpeed1 = speed1;
    }

    if( options.readFanSpeed2 )
    {
      int speed2;
      if( getFanSpeedSensor(options.fan2SensorIndex,speed2,glb_hHndFile) )
        glb_iSensorSpeed2 = speed2;
    }

/*
    if( glb_iFanSpeed0<glb_iFanSpeed1 && glb_iSensorSpeed1<glb_iSensorSpeed2 )
    {
      int x = glb_iSensorSpeed1;
      glb_iSensorSpeed1 = glb_iSensorSpeed2;
      glb_iSensorSpeed2 = x;
    }
*/
  }

  // Workaround fr I8100 BIOS <= A03 Bug: Drehzahl und Stati der Lfter sind vertauscht
  if( (glb_iFanSpeed1>glb_iFanSpeed2 && stat1<stat2)
  ||  (glb_iFanSpeed1<glb_iFanSpeed2 && stat1>stat2) )
  {
    int t = glb_iFanSpeed1;
    glb_iFanSpeed1 = glb_iFanSpeed2;
    glb_iFanSpeed2 = t;
    glb_szFanSpeed1.Format( L"%d", glb_iFanSpeed1 );
    glb_szFanSpeed2.Format( L"%d", glb_iFanSpeed2 );
  }

  return 0;
}

//---------------------------------------------------------------
// Lfter auf Drehzahl setzen wenn sie runtergeschaltet werden
//---------------------------------------------------------------
void CTempControl::maintainSpeeds()
{
  if( glb_bSensorFanForcing )
  {
    int si;
    if( glb_iFanForcingValue1>0 && options.readFanStatus1 )
    {
      if( getFanSpeedSensor(options.fan1SensorIndex,si,glb_hHndFile) && si<glb_iFanForcingValue1 )
        setFanSpeedSensor( options.fan1SensorIndex, glb_iFanForcingValue1, glb_hHndFile );
    }

    if( glb_iFanForcingValue2>0 && options.readFanStatus2 )
    {
      if( getFanSpeedSensor(options.fan2SensorIndex,si,glb_hHndFile) && si<glb_iFanForcingValue2 )
        setFanSpeedSensor( options.fan2SensorIndex, glb_iFanForcingValue2, glb_hHndFile );
    }
  }
  else
  {
    int si;

    if( glb_iFanForcingValue1>0 && options.readFanStatus1 )
    {
      if( readFanState(options.fan1Index,si,glb_hHndFile) && si<glb_iFanForcingValue1 )
        setFanSpeed( options.fan1Index, glb_iFanForcingValue1, glb_hHndFile );
    }

    if( glb_iFanForcingValue2>0 && options.readFanStatus2 )
    {
      if( readFanState(options.fan2Index,si,glb_hHndFile) && si<glb_iFanForcingValue2 )
        setFanSpeed( options.fan2Index, glb_iFanForcingValue2, glb_hHndFile );
    }
  }
}

//---------------------------------------------------------------
// Tempmonitor
//---------------------------------------------------------------
void CTempControl::control()
{
  // sind gar keine Lfter aktiviert, also abbrechen:
  if( options.numFans < 1 )
    return;

  TMonSettings* ms = &options.ms[options.preDefSet];
  BOOL active[4] = { FALSE, FALSE, FALSE, FALSE };
  int levels[4];

  // CPU-Tempkontrolle aktiviert? (CPU-Temperatur lesbar, CPU-Monitor aktiv und mindestens ein Lfter aktiviert)
  if( options.readCPUTemp && ms->monEnabled[CPUMON] && (ms->cpuFan[0] || ms->cpuFan[1]) )
  {
    active[0] = TRUE;
    getTempLevel( glb_iCPUTemp, glb_iCPULevel, ms->cpuMonTemp );
  }

  // GPU control
  if( options.readGPUTemp && ms->monEnabled[GPUMON] && (ms->gpuFan[0] || ms->gpuFan[1]) )
  {
    active[1] = TRUE;
    getTempLevel( glb_iGPUTemp, glb_iGPULevel, ms->gpuMonTemp );
  }

  // Memory control
  if( options.readMemoryTemp && ms->monEnabled[MEMMON] && (ms->memoryFan[0] || ms->memoryFan[1]) )
  {
    active[2] = TRUE;
    getTempLevel( glb_iMemoryTemp, glb_iMemoryLevel, ms->memoryMonTemp );
  }

  // Chipset control
  if( options.readChipsetTemp && ms->monEnabled[CHSMON] && (ms->chipsetFan[0] || ms->chipsetFan[1]) )
  {
    active[3] = TRUE;
    getTempLevel( glb_iChipsetTemp, glb_iChipsetLevel, ms->chipsetMonTemp );
  }

  log( L"Templevels: %d %d %d %d", active[0] ? glb_iCPULevel : -1, 
                                   active[1] ? glb_iGPULevel : -1,
                                   active[2] ? glb_iMemoryLevel : -1,
                                   active[3] ? glb_iChipsetLevel : -1 );

  levels[0] = glb_iCPULevel;
  levels[1] = glb_iGPULevel;
  levels[2] = glb_iMemoryLevel;
  levels[3] = glb_iChipsetLevel;

  // alle Levels eingesammelt, nun mal schauen, wie wir die Lfter einschalten mssen...
  setFanLevel( 0, ms, levels, active, options.readFanStatus1, glb_iFanStatus1, glb_iSensorSpeed1 );
  
  // zweiter Lfter aktiviert?
  if( options.numFans > 1 )
    setFanLevel( 1, ms, levels, active, options.readFanStatus2, glb_iFanStatus2, glb_iSensorSpeed2 );
}


//---------------------------------------------------------------
// Setzt den Lfterwert basierend auf den Levels
//---------------------------------------------------------------
void CTempControl::setFanLevel( int fanNo, TMonSettings* ms, int* levels, BOOL* active, 
                                BOOL statusReading, int curStatus, int curSensorSpeed )
{
  int newLevel = -1;
  int speedIndex = fanNo*4 + (options.tempCtrlHWFans ? 8 : 0);
  
  for( int i=0; i<4; i++ )
  {
    if( i==0 && !ms->cpuFan[fanNo] )
      continue;
    else if( i==1 && !ms->gpuFan[fanNo] )
      continue;
    else if( i==2 && !ms->memoryFan[fanNo] )
      continue;
    else if( i==3 && !ms->chipsetFan[fanNo] )
      continue;

    if( !active[i] )
      continue;

    int level;

    if( levels[i] == 0 )
      level = 0;
    else 
    {
      if( i == 0 )
        level = ms->cpuFanSpeeds[speedIndex+levels[i]-1];
      else if( i == 1 )
        level = ms->gpuFanSpeeds[speedIndex+levels[i]-1];
      else if( i == 2 )
        level = ms->memoryFanSpeeds[speedIndex+levels[i]-1];
      else
        level = ms->chipsetFanSpeeds[speedIndex+levels[i]-1];

      // Fanlevel korrigieren ('No change' herausfiltern)
      if( !options.tempCtrlHWFans )
      {
        if( level == 0 )    // von Index 0 (ist ja eigentlich 'aus') ...
          level = -1;       // in Index -1 konvertieren
        else
          level--;          // von 1->0, 2->1 und 3->2 (Regelstufen)
      }
    }

    if( level > newLevel )
      newLevel = level;
  }

  if( newLevel != -1 )
  {
    glb_bActiveFanForcing = true;

    log( L"TempControl: New level for fan %d: %d", fanNo, newLevel );

    if( options.tempCtrlHWFans )      // Drehzahl ber Sensor setzen
    {
      glb_bSensorFanForcing = true;

      if( fanNo == 0 )
        glb_iFanForcingValue1 = newLevel;
      else
        glb_iFanForcingValue2 = newLevel;

/*
      if( curSensorSpeed != newLevel )
        setFanSpeedSensor( fanNo==0 ? options.fan1SensorIndex : options.fan2SensorIndex,
                           newLevel, 
                           glb_hHndFile );
*/
    }
    else      // ... und hier bers SMBIOS
    {
      glb_bSensorFanForcing = false;

      if( fanNo == 0 )
        glb_iFanForcingValue1 = newLevel;
      else
        glb_iFanForcingValue2 = newLevel;

/*
      if( statusReading )
      {
        if( curStatus != newLevel )
          setSpeed( fanNo, newLevel );
      }
      else
        setSpeed( fanNo, newLevel );
*/
    }
  }
}

//---------------------------------------------------------------
// Bestimmt den aktuellen Level fr einen Sensor
//---------------------------------------------------------------
void CTempControl::getTempLevel( int temp, int& level, int* monTemp )
{
  if( temp <= monTemp[1] )
    level = 0;
  else if( temp>=monTemp[1]+1 && temp<=monTemp[0]-1 )
  {
    if( level > 0 )
      level = 1;
  }
  else if( temp>=monTemp[0] && temp<=monTemp[2]-1 )
  {
    if( level <= 1 )
      level = 1;
    else if( level >= 2 )
    {
      if( temp == monTemp[0] )
        level = 1;
      else
        level = 2;
    }
  }
  else if( temp>=monTemp[2] && temp<=monTemp[4]-1 )
  {
    if( level <= 2 )
      level = 2;
    else if( level >= 3 )
    {
      if( temp == monTemp[2] )
        level = 2;
      else
        level = 3;
    }
  }
  else if( temp>=monTemp[4] && temp<=monTemp[6]-1 )
  {
    if( level <= 3 )
      level = 3;
    else if( level == 4 )
    {
      if( temp == monTemp[4] )
        level = 3;
    }
  }
  else if( temp >= monTemp[6] )
    level = 4;
}

//---------------------------------------------------------------
// Lfterdrehzahl fr einen Lfter setzen
//---------------------------------------------------------------
void CTempControl::setSpeed( int fanNo, int speed )
{
  setFanSpeed( fanNo==0 ? options.fan1Index : options.fan2Index,  
               speed, 
               glb_hHndFile );

  // Workaround fr I8100 BIOS A08: sofort Drehzahl nochmal setzen, damit kurzzeitiges
  // "Aufheulen" der Lfter beim Einschalten vermieden wird. Obwohl dies nur frs
  // I8100 BIOS A08 notwendig ist, sollte dies keine negativen Effekte bei anderen
  // Modellen oder BIOS-Versionen hervorrufen. Keine Ahnung, ob dies ein Bug oder
  // ein Feature vom BIOS A08 ist ;-)
  if( options.i8100Fix )
    setFanSpeed( fanNo==0 ? options.fan1Index : options.fan2Index,  
                 speed, 
                 glb_hHndFile );
}
