/*
   I8kfanGUI V3.0
   Copyright(c) 2001-2006 Christian Diefer
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as 
   published by the Free Software Foundation.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "stdafx.h"
#include "AtlBase.h"
#include "TempControl.h"
#include "TOptions.h"
#include "i8kfangui.h"
#include "i8kfanguiDlg.h"
#include "GlobalVars.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static wchar_t logPath[264];
static char* uc2char( char* dest, void* s );

/////////////////////////////////////////////////////////////////////////////
// CI8kfanguiApp

BEGIN_MESSAGE_MAP(CI8kfanguiApp, CWinApp)
	//{{AFX_MSG_MAP(CI8kfanguiApp)
	//}}AFX_MSG
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CI8kfanguiApp Konstruktor

CI8kfanguiApp::CI8kfanguiApp()
{ activeMutex = FALSE; }

/////////////////////////////////////////////////////////////////////////////

CI8kfanguiApp theApp;
CI8kfanguiDlg* dlg;

/////////////////////////////////////////////////////////////////////////////
// CI8kfanguiApp Initialisierung

BOOL CI8kfanguiApp::InitInstance()
{
  InitCommonControls();

  CWinApp::InitInstance();

  // sicherstellen, da nur eine Instanz luft
  BOOL ret = FALSE;
  wchar_t* mutexName = L"I8kfangui_mutex";

  hMutex = OpenMutexW( MUTEX_ALL_ACCESS, FALSE, mutexName );
  if( hMutex == NULL )
  {
    hMutex = CreateMutex( 0, TRUE, mutexName );
    DWORD err = GetLastError();
    ret = !(hMutex==NULL || err==ERROR_ALREADY_EXISTS);
    if( hMutex != NULL )
      activeMutex = TRUE;
  }
  else
    CloseHandle( hMutex );

  CString s1, s2;

  if( ret == FALSE )
  {
    s1.LoadStringW( IDS_INIT1 );
    s2.LoadStringW( IDS_INIT2 );
    MessageBox( NULL, s1, s2, MB_ICONSTOP|MB_OK );
    return FALSE;
  }

  // Dateinamen frs Log generieren
  if( GetModuleFileName(NULL,logPath,255) == 0 )
  {
    CloseHandle( hMutex );
    s1.LoadStringW( IDS_INIT3 );
    s2.LoadStringW( IDS_INIT4 );
    MessageBox( NULL, s1, s2, MB_OK|MB_ICONSTOP );
    return FALSE;
  }

  size_t i = _tcslen( logPath ) - 1, i1=i;
  while( i>=0 && i<=i1 && logPath[i]!='.' )
    i--;

  if( i >= 0 )
    wcscpy_s( &logPath[i], 5, L".log" );
  else
  {
    CloseHandle( hMutex );
    s1.LoadStringW( IDS_INIT5 );
    s2.LoadStringW( IDS_INIT4 );
    MessageBox( NULL, s1, s2, MB_ICONSTOP|MB_OK );
    return FALSE;
  }

  // Hauptfenster anlegen, aufbauen und anzeigen
  dlg = new CI8kfanguiDlg();
  m_pMainWnd = dlg;

  if( m_pMainWnd )
  {
    log( L"Mutex: 0x%08x", hMutex );
    log( L"Init complete" );
    log( L"--------------------------------" );
    dlg->setCmdLine( m_lpCmdLine );
    return TRUE;
  }
  else
    return FALSE;
}

/////////////////////////////////////////////////////////////////////////////
int CI8kfanguiApp::ExitInstance() 
{
  if( activeMutex )  
  {
    log( _T("/bReleasing mutex (0x%08x)... "), hMutex );
    if( CloseHandle(hMutex) )
      log( _T("/aOk") );
    else
		log( _T("/aFailed") );
  }

  return CWinApp::ExitInstance();
}

/////////////////////////////////////////////////////////////////////////////
// Delete the old log file
void clearLog()
{
  DeleteFile( logPath );
}

/////////////////////////////////////////////////////////////////////////////
// Debugtext ins Logfile ausgeben
void log( wchar_t* msg, ... )
{
  if( !options.debugMode )
    return;

  BOOL noDate=FALSE, noCR=FALSE;
  static wchar_t debugBuf[32768];
  static char writeBuf[32768];
  va_list ap;
  va_start( ap, msg );

  time_t zeit = time( NULL );
  struct tm ts;
  int err = localtime_s( &ts, &zeit );

  *debugBuf = '\0';

  if( *msg == '/' )
  {
    switch( msg[1] )
    {
      case 'a': noDate=TRUE;
                msg+=2;  
                break;

      case 'b': noCR=TRUE;
                msg+=2;  
                break;

      case 'c': noDate=TRUE;
                noCR=TRUE;
                msg+=2;  
                break;
    }
  }

  if( !noDate )
    wsprintf( debugBuf,
              L"%02d:%02d:%02d ",
              ts.tm_hour,
              ts.tm_min,
              ts.tm_sec );

  vswprintf_s( &debugBuf[_tcslen(debugBuf)], 32768-_tcslen(debugBuf)-1, msg, ap );
  va_end( ap );

  if( !noCR )
    _tcscat_s( debugBuf, L"\r\n" );

  CString s1, s2;
  s2.LoadStringW( IDS_SERROR );

  HANDLE hdl = CreateFile( logPath, FILE_APPEND_DATA, FILE_SHARE_READ, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL );
  if( hdl == INVALID_HANDLE_VALUE )
  {
    options.debugMode = FALSE;
    s1.LoadStringW( IDS_INIT6 );
    wsprintfW( debugBuf, s1, GetLastError() );
    MessageBoxW( NULL, debugBuf, s2, MB_ICONSTOP|MB_OK );
    return;
  }

  if( SetFilePointer(hdl,0,NULL,FILE_END) == INVALID_SET_FILE_POINTER )
  {
    CloseHandle( hdl );
    options.debugMode = FALSE;
    s1.LoadStringW( IDS_INIT7 );
    wsprintf( debugBuf, s1, GetLastError() );
    MessageBox( NULL, debugBuf, s2, MB_ICONSTOP|MB_OK );
    return;
  }

  uc2char( writeBuf, debugBuf );

  DWORD written;
  if( WriteFile(hdl,writeBuf,(DWORD)strlen(writeBuf),&written,NULL) == 0 )
  {
    CloseHandle( hdl );
    options.debugMode = FALSE;
    s1.LoadStringW( IDS_INIT8 );
    wsprintf( debugBuf, s1, GetLastError() );
    MessageBox( NULL, debugBuf, s2, MB_ICONSTOP|MB_OK );
    return;
  }

  CloseHandle( hdl );
}

//---------------------------------------------------------------
// String in einem Puffer suchen und in Logfile ausgeben (fr
// Zugriff auf BIOS-Info)
//---------------------------------------------------------------
void logInfoString( wchar_t* buf, wchar_t* s, wchar_t* dest )
{
  wchar_t* u = wcsstr( buf, s );
  if( !u )
    return;

  wchar_t *i = u;
  wchar_t c;
  
  while( *i && *i!='\n' && *i!='\r' )
    i++;

  i--;

  for( ;; )
  {
    if( *i && isspace(*i) ) 
      i--;
    else
      break;
  }

  i++;

  if( *i )
    c = *i;

  *i = '\0';
  log( L"BIOS Info: <%s>", u );

  if( dest != NULL )
  {
    wchar_t* m = u+_tcslen(s);
    while( isspace(*m) ) m++;
    _tcscpy_s( dest, _tcslen(m)+1, m );
  }

  *i = c;
}

//------------------------------------------------------------------
// convert unicode string to ansi string
//------------------------------------------------------------------
static char* uc2char( char* dest, void* s )
{
  char* t = (char*)s;
  int i=-1;
  int j=-1;
  
  do {
    dest[++i] = t[++j];
    t++;
  } while( dest[i] );
  
  return dest;      
}

