/*
   I8kfanGUI V3.1
   Copyright(c) 2001-2007 Christian Diefer

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as 
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "stdafx.h"
#include "AtlBase.h"
#include "TOptions.h"
#include "i8kfangui.h"
#include "TempControl.h"
#include "i8kfanguiDlg.h"
#include "i8kfanguiOptionDlg.h"
#include "i8kfanctrl.h"
#include "hyperlink.h"
#include "SystemInfo.h"
#include <wtsapi32.h>
#include <stdarg.h>
#include <winuser.h>
#include "globalvars.h"
#include "TrayIconManager.h"
#include "ModelCheck.h"
#include "cpuid.h"
#include "PictureHyperlink.h"
#include "CoreTemp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//#define TIMERINFO

#define RESETLEVELS (m_iCPULevel = m_iGPULevel = m_iMemoryLevel = m_iChipsetLevel = glb_iCPULevel = glb_iGPULevel = glb_iMemoryLevel = glb_iChipsetLevel = 0)

// NachrichtenIDs fr Kommandos des Trayicon-Kontextmens:
#define IDM_OBSERVERMODE   300
#define IDM_TEMPCONTROL    301
#define IDM_TEMPPROFILE1   302
#define IDM_TEMPPROFILE2   303
#define IDM_TEMPPROFILE3   304
#define IDM_TEMPPROFILE4   305
#define IDM_SHOWMAIN       306
#define IDM_CELSIUSUNIT    307
#define IDM_FAHRENHEITUNIT 308
#define IDM_SENSORUPDATE   309
#define IDM_SHOWDIAGRAM    310
#define IDM_SPEEDICON      311
#define IDM_SHOWTRAYICON01 312
#define IDM_SHOWTRAYICON02 313
#define IDM_SHOWTRAYICON11 314
#define IDM_SHOWTRAYICON12 315
#define IDM_SHOWTRAYICON21 316
#define IDM_SHOWTRAYICON22 317
#define IDM_SHOWTRAYICON31 318
#define IDM_SHOWTRAYICON32 319
#define IDM_SHOWTRAYICON41 320
#define IDM_SHOWTRAYICON42 321
#define IDM_SHOWTRAYICON51 322
#define IDM_SHOWTRAYICON52 323
#define IDM_SHOWTRAYICON61 324
#define IDM_SHOWTRAYICON62 325
#define IDM_SHOWTRAYICON71 326
#define IDM_SHOWTRAYICON72 327
#define IDM_FORCESHUTDOWN  328
#define IDM_FANFORCING     329
#define IDM_MANUALFANS     330
#define IDM_SHOWTRAYICON81 331
#define IDM_SHOWTRAYICON82 332

#define TOOLTIP_ID 1

static UINT timerID = 1;
static wchar_t biosInfo[16392];   // Puffer fr Systeminfo vom BIOS
static wchar_t text[1024];
static TTrayIconManager trayMan;
static TModelCheck mcheck;

TCPUData cpuData;

// die Intervalle fr den FanForceInterval-Index (in msec):
static int fanForceTimeout[7] = { 1000, 500, 200, 100, 50, 20, 10 };

// Registrieren fr die Nachricht, da die Taskbar neu angelegt wurde
// => Neuanlegen des/der Trayicons nach Explorer-Crash oder Explorer-Kill
static UINT WM_TASKBARCREATED = ::RegisterWindowMessageW( L"TaskbarCreated" );


//*****************************************************************************
// CAboutDlg
//*****************************************************************************

class CAboutDlg : public CDialog
{
public:
  CAboutDlg();

  //{{AFX_DATA(CAboutDlg)
  enum { IDD = IDD_ABOUTBOX };
  CHyperLink m_gplLink;
  CHyperLink m_homeLink;
  CHyperLink m_faqLink;
  CHyperLink m_manualLink;
  CPictureHyperLink m_payLink;
  CStatic m_title;
  //}}AFX_DATA

  //{{AFX_VIRTUAL(CAboutDlg)
protected:
  virtual void DoDataExchange(CDataExchange* pDX);
	//}}AFX_VIRTUAL

protected:
  //////////////////
  // ber-Dialog initialisieren
  virtual BOOL OnInitDialog();

  //{{AFX_MSG(CAboutDlg)
  //}}AFX_MSG
  DECLARE_MESSAGE_MAP()
public:
};

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
  //{{AFX_MSG_MAP(CAboutDlg)
  //}}AFX_MSG_MAP
  ON_WM_QUERYENDSESSION()
END_MESSAGE_MAP()


//---------------------------------------------------------------
// Konstruktor for About dialog
//---------------------------------------------------------------
CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
  //{{AFX_DATA_INIT(CAboutDlg)
  //}}AFX_DATA_INIT
}

//---------------------------------------------------------------
// Von der MFC aufgerufen, um Daten zwischen GUI-Elementen und einfachen
// Variablen auszutauschen
//---------------------------------------------------------------
void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(CAboutDlg)
  DDX_Control(pDX, IDC_GPLLINK, m_gplLink);
  DDX_Control(pDX, IDC_URLTEXT1, m_homeLink);
  DDX_Control(pDX, IDC_URLTEXT4, m_faqLink);
  DDX_Control(pDX, IDC_URLTEXT3, m_manualLink);
  DDX_Control(pDX, IDC_PAYPAL, m_payLink);
  DDX_Control(pDX, IDC_TITLE, m_title);
  //}}AFX_DATA_MAP
}

//---------------------------------------------------------------
// Info-Dialog initialisieren
//---------------------------------------------------------------
BOOL CAboutDlg::OnInitDialog() 
{
  m_gplLink.SetURL( L"http://www.fsf.org/licenses/gpl.txt" );
  m_gplLink.SetUnderline( TRUE );

  CString s1, s2;
  s1.LoadStringW( IDS_DONATE );
  s2.LoadStringW( IDS_DONATEMSG );
  m_payLink.SetURL( s1, s2 );

  CDialog::OnInitDialog();

  CString l1;
  l1.LoadStringW( IDS_ATITLE );
  l1.Append( RELPLATFORM );
  m_title.SetWindowTextW( l1 );

  return TRUE;
}

//*****************************************************************************
// CI8kfanguiDlg
//*****************************************************************************

//---------------------------------------------------------------
// Konstruktor
//
// Indirekter Aufruf durch: 
//   - CI8kfanguiApp::InitInstance() -> Instanzierung des Hauptfensters durch Startklasse
//---------------------------------------------------------------
CI8kfanguiDlg::CI8kfanguiDlg( CWnd* pParent /*=NULL*/)
	: CDialog(CI8kfanguiDlg::IDD, pParent )
  , m_bMaintainSpeed(FALSE)
{
  //{{AFX_DATA_INIT(CI8kfanguiDlg)
  m_bOuterFan = FALSE;
  m_bInnerFan = FALSE;
  m_szSpeedString1 = _T("---");
  m_szSpeedString2 = _T("---");
  m_szCPULoad = _T("---");
  m_szCPUTemp = _T("---");
  m_szHDTemp = _T("---");
  m_szHDTemp2 = _T("---");
  m_szFanSpeed1 = _T("---");
  m_szFanSpeed2 = _T("---");
  m_szGPUTemp = _T("---");
  m_szMemoryTemp = _T("---");
  m_szChipsetTemp = _T("---");
  m_szCPUSpeed = _T("---");
  m_iCPULevel = 0;
  m_iGPULevel = 0;
  m_iMemoryLevel = 0;
  m_iChipsetLevel = 0;
  m_iSensorSpeed1 = 0;
  m_iSensorSpeed2 = 0;
  m_iSensorIndicator1 = 0;
  m_iSensorIndicator2 = 0;
  m_szCPUTempMC1 = _T("");
  //m_szCPUTempMC2 = _T("");
  //}}AFX_DATA_INIT

  initComplete = false;
  glb_ProcGetSystemTimes = (PROCGST)GetProcAddress( GetModuleHandleW(L"kernel32"), "GetSystemTimes" );

  options.fan1Alias.LoadStringW( IDS_FANALIAS1 );
  options.fan2Alias.LoadStringW( IDS_FANALIAS2 );
  //log( L"Alias1:<%s> Alias2:<%s>", options.fan1Alias, options.fan2Alias );

  // die Optionen aus der Registry einlesen bzw. anlegen:
  getRegKeys();

  if( options.debugMode )   // altes Logfile lschen
    clearLog();

  log( L"Initiating I8kfanGUI %s%s", VERSION, RELPLATFORM );
	m_hIcon = AfxGetApp()->LoadIcon( IDR_MAINFRAME );

  log( L"Opmode:%d", options.opmode );
  log( L"Creating main window..." );
  Create( CI8kfanguiDlg::IDD, pParent );

}

//---------------------------------------------------------------
// Von der MFC aufgerufen, um Daten zwischen GUI-Elementen und einfachen
// Variablen auszutauschen
//---------------------------------------------------------------
void CI8kfanguiDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Control(pDX, IDC_SLIDER2, m_scOuterSlider);
  DDX_Control(pDX, IDC_SLIDER1, m_scInnerSlider);
  DDX_Check(pDX, IDC_CHECK2, m_bOuterFan);
  DDX_Check(pDX, IDC_CHECK1, m_bInnerFan);
  DDX_Text(pDX, IDC_EDIT4, m_szSpeedString1);
  DDX_Text(pDX, IDC_EDIT5, m_szSpeedString2);
  DDX_Text(pDX, IDC_EDIT6, m_iCPULevel);
  DDX_Text(pDX, IDC_CPULOAD, m_szCPULoad);
  DDX_Text(pDX, IDC_EDIT1, m_szCPUTemp);
  DDX_Text(pDX, IDC_TEMP1, m_szCPUTempMC1);
  DDX_Text(pDX, IDC_EDIT7, m_szHDTemp);
  DDX_Text(pDX, IDC_EDIT20, m_szHDTemp2);
  DDX_Text(pDX, IDC_EDIT2, m_szFanSpeed1);
  DDX_Text(pDX, IDC_EDIT3, m_szFanSpeed2);
  DDX_Text(pDX, IDC_EDIT9, m_szGPUTemp);
  DDX_Text(pDX, IDC_EDIT8, m_szMemoryTemp);
  DDX_Text(pDX, IDC_EDIT10, m_szChipsetTemp);
  DDX_Text(pDX, IDC_EDIT11, m_szCPUSpeed);
  DDX_Text(pDX, IDC_EDIT13, m_iSensorSpeed1);
  DDX_Text(pDX, IDC_EDIT12, m_iSensorSpeed2);
  DDX_Text(pDX, IDC_EDIT14, m_iSensorIndicator1);
  DDX_Text(pDX, IDC_EDIT15, m_iSensorIndicator2);
  DDX_Control(pDX, IDC_EDIT16, m_cControlProfile);
  DDX_Text(pDX, IDC_EDIT17, m_iGPULevel);
  DDX_Text(pDX, IDC_EDIT18, m_iMemoryLevel);
  DDX_Text(pDX, IDC_EDIT19, m_iChipsetLevel);
  DDX_Check(pDX, IDC_MAINTAINSPEED, m_bMaintainSpeed);
  DDX_Control(pDX, IDC_STATIC501, m_cCPUTemp);
  DDX_Control(pDX, IDC_STATIC502, m_cGPUTemp);
  DDX_Control(pDX, IDC_STATIC503, m_cMemoryTemp);
  DDX_Control(pDX, IDC_STATIC504, m_cChipsetTemp);
  DDX_Control(pDX, IDC_STATIC505, m_cCPUSpeed);
  DDX_Control(pDX, IDC_STATIC506, m_cCPULoad);
  DDX_Control(pDX, IDC_CORE12, m_cCoreText);
}

BEGIN_MESSAGE_MAP(CI8kfanguiDlg, CDialog)
  ON_WM_SYSCOMMAND()
  ON_WM_PAINT()
  ON_WM_QUERYDRAGICON()
  ON_BN_CLICKED(IDC_APPLY, OnApply)
  ON_BN_CLICKED(IDC_CHECK1, OnCheck1)
  ON_BN_CLICKED(IDC_CHECK2, OnCheck2)
  ON_BN_CLICKED(IDC_BUTTON1, OnAbout)
  ON_BN_CLICKED(IDC_OPTBUTTON, OnOptbutton)
  ON_WM_CLOSE()
  ON_WM_TIMER()
  ON_WM_HSCROLL()
  ON_BN_CLICKED(IDC_BUTTON2, OnTempUpdate)
  ON_BN_CLICKED(IDC_SYSINFOBUTTON, OnSysinfobutton)
  ON_MESSAGE(WM_ICONNOTIFY, OnIconNotify)
  ON_MESSAGE(WM_WTSSESSION_CHANGE,OnWtsSessionChange)
  ON_BN_CLICKED(IDC_CLOSE, OnBnClickedClose)
  ON_COMMAND(IDM_SHOWMAIN, OnShowMainMenu)
  ON_COMMAND(IDM_OBSERVERMODE, OnObserverModeMenu)
  ON_COMMAND(IDM_MANUALFANS, OnManualControlMenu)
  ON_COMMAND(IDM_TEMPCONTROL, OnTempControlMenu)
  ON_COMMAND(IDM_FANFORCING, OnForceHighSpeedMenu)
  ON_COMMAND(IDM_TEMPPROFILE1, OnTempProfile1Menu)
  ON_COMMAND(IDM_TEMPPROFILE2, OnTempProfile2Menu)
  ON_COMMAND(IDM_TEMPPROFILE3, OnTempProfile3Menu)
  ON_COMMAND(IDM_TEMPPROFILE4, OnTempProfile4Menu)
  ON_COMMAND(IDM_CELSIUSUNIT,OnCelsiusMenu)
  ON_COMMAND(IDM_FAHRENHEITUNIT,OnFahrenheitMenu)
  ON_COMMAND(IDM_SENSORUPDATE,OnSensorUpdateMenu)
  ON_COMMAND(IDM_SHOWDIAGRAM,OnShowDiagram)
  ON_COMMAND(IDM_SHOWTRAYICON01,OnSelectTrayIcon01)
  ON_COMMAND(IDM_SHOWTRAYICON02,OnSelectTrayIcon02)
  ON_COMMAND(IDM_SHOWTRAYICON11,OnSelectTrayIcon11)
  ON_COMMAND(IDM_SHOWTRAYICON12,OnSelectTrayIcon12)
  ON_COMMAND(IDM_SHOWTRAYICON21,OnSelectTrayIcon21)
  ON_COMMAND(IDM_SHOWTRAYICON22,OnSelectTrayIcon22)
  ON_COMMAND(IDM_SHOWTRAYICON31,OnSelectTrayIcon31)
  ON_COMMAND(IDM_SHOWTRAYICON32,OnSelectTrayIcon32)
  ON_COMMAND(IDM_SHOWTRAYICON41,OnSelectTrayIcon41)
  ON_COMMAND(IDM_SHOWTRAYICON42,OnSelectTrayIcon42)
  ON_COMMAND(IDM_SHOWTRAYICON51,OnSelectTrayIcon51)
  ON_COMMAND(IDM_SHOWTRAYICON52,OnSelectTrayIcon52)
  ON_COMMAND(IDM_SHOWTRAYICON61,OnSelectTrayIcon61)
  ON_COMMAND(IDM_SHOWTRAYICON62,OnSelectTrayIcon62)
  ON_COMMAND(IDM_SHOWTRAYICON71,OnSelectTrayIcon71)
  ON_COMMAND(IDM_SHOWTRAYICON72,OnSelectTrayIcon72)
  ON_COMMAND(IDM_SHOWTRAYICON81,OnSelectTrayIcon81)
  ON_COMMAND(IDM_SHOWTRAYICON82,OnSelectTrayIcon82)
  ON_COMMAND(IDM_FORCESHUTDOWN, OnSelectShutdown)
  ON_COMMAND(IDM_SPEEDICON,OnSelectSpeedIcon)
  ON_CBN_SELCHANGE(IDC_EDIT16, OnCbnSelchangeProfile)
  ON_REGISTERED_MESSAGE(WM_TASKBARCREATED, OnTaskBarCreated)
  ON_WM_ENDSESSION()
  ON_BN_CLICKED(IDC_PEAKBUTTON, &CI8kfanguiDlg::OnBnClickedPeakbutton)
  ON_BN_CLICKED(IDC_MAINTAINSPEED, &CI8kfanguiDlg::OnBnClickedMaintainspeed)
  ON_BN_CLICKED(IDC_LOGBUTTON, &CI8kfanguiDlg::OnBnClickedLogbutton)
  ON_BN_CLICKED(IDC_RESETHISTORY, &CI8kfanguiDlg::OnBnClickedResethistory)
  ON_BN_CLICKED(IDC_RADIO1, &CI8kfanguiDlg::OnBnClickedRadio1)
  ON_BN_CLICKED(IDC_RADIO2, &CI8kfanguiDlg::OnBnClickedRadio2)
  ON_BN_CLICKED(IDC_RADIO3, &CI8kfanguiDlg::OnBnClickedRadio3)
  ON_BN_CLICKED(IDC_RADIO4, &CI8kfanguiDlg::OnBnClickedRadio4)
END_MESSAGE_MAP()

//---------------------------------------------------------------
// Trayicons erneut darstellen (nach Explorer-Crash)
//---------------------------------------------------------------
LRESULT CI8kfanguiDlg::OnTaskBarCreated( WPARAM wp, LPARAM lp )
{
  trayMan.displayTrayIcon( NIM_ADD, true, true, true );
  return 0;
}

//---------------------------------------------------------------
// GUI aufbauen fr erste Anzeige
//---------------------------------------------------------------
BOOL CI8kfanguiDlg::OnInitDialog()
{
  CDialog::OnInitDialog();

  ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
  ASSERT(IDM_ABOUTBOX < 0xF000);

  SetIcon( m_hIcon, TRUE );

  glb_hWnd = m_hWnd;
  glb_hIcon = m_hIcon;

  log( L"Setting initial values..." );

  // Diagrammposition berechnen:
  ((CWnd*)GetDlgItem(IDC_CPUFRAME))->GetWindowRect( &cpuFrame );
  ((CWnd*)GetDlgItem(IDC_LEGENDFRAME))->GetWindowRect( &legendFrame );
  ScreenToClient( &cpuFrame );
  ScreenToClient( &legendFrame );
  cpuFrame.DeflateRect( 1, 1 );
  legendFrame.DeflateRect( 1, 1 );

  log( L"Using %s CPU load detection algorithm", glb_ProcGetSystemTimes ? L"new (official)" : L"old (unofficial)" );

#ifndef TIMERINFO
  ((CWnd*)GetDlgItem(IDC_FORCEINFO))->ShowWindow( FALSE );
#endif

  CString s1, s2;
  s1.LoadStringW( IDS_DLG1 );

  s2.Format( s1, VERSION, RELPLATFORM );
  SetWindowTextW( s2 );

  // Konsistenz der Hauptoptionen gewhrleisten:
  if( options.opmode == TEMPMONITOR )
  {
    options.maintainSpeeds = FALSE;
    glb_iEffectiveTimerBase = fanForceTimeout[options.fanForceInterval];
  }
  else if( options.opmode == OBSERVERMODE )
  {
    options.maintainSpeeds = FALSE;
    glb_iEffectiveTimerBase = 1000;
  }
  else if( options.maintainSpeeds )
  {
    glb_iEffectiveTimerBase = fanForceTimeout[options.fanForceInterval];
  }
  else if( options.opmode == FANFORCING )
  {
    options.maintainSpeeds = FALSE;
    glb_iEffectiveTimerBase = fanForceTimeout[options.fanForceInterval];
  }
  else
    glb_iEffectiveTimerBase = 1000;

  glb_iCPUTemp = 0;
  glb_iCPUTempC1 = 0;
  glb_iCPUTempC2 = 0;
  glb_iGPUTemp = 0;
  glb_iMemoryTemp = 0;
  glb_iChipsetTemp = 0;
  glb_iHDTemp = 0;
  glb_iHDTemp2 = 0;
  glb_bShellVer5 = false;
  glb_bWinXP = false;
  glb_bPerfCnt = false;
  glb_pPowerApiDLL = NULL;
  glb_pCallPI = NULL;
  glb_iCPULevel = glb_iGPULevel = glb_iMemoryLevel = glb_iChipsetLevel = 0;
  glb_iDiagramIndex = 0;
  glb_iSensorSpeed1 = 0;
  glb_iSensorSpeed2 = 0;
  glb_bTrayIcon1Active = false;
  glb_bTrayIcon2Active = false;
  glb_bSpeedIconActive = false;
  glb_bDirectSpeedApplied = false;
  glb_iDirectSpeedMethod = 0;      // 0=ber SMBIOS, 1=ber HW-Sensor
  glb_iDirectSpeed1 = 0;           // aus
  glb_iDirectSpeed2 = 0;           // aus
  glb_iFanStatus1 = -1;
  glb_iFanStatus2 = -1;
  glb_bActiveFanForcing = false;
  glb_bSensorFanForcing = false;
  glb_iFanForcingValue1 = -1;
  glb_iFanForcingValue2 = -1;
  glb_iCPUMax = 0;
  glb_iGPUMax = 0;
  glb_iMemoryMax = 0;
  glb_iChipsetMax = 0;
  glb_iHDMax = 0;

  //m_cCoreText.SetTextColor( RGB(0,0,255) );

  m_iHDTemp = m_iHDTemp2 = 0;

  // die Warnmeldung wegen Tempmonitor und das Tray-Minimizing ausschalten fr
  // unvorhergesehene Terminierung in OnInitDialog()...
  controlQuitWarning = false;
  forceShutdown = true;

  activeSession = TRUE;
  diagramOffset = 0;
  wtsStatus = 1;    // noch nicht bei WTS registriert
  
  int i;
  for( i=0; i<1200; i++ )
    glb_iaTempArr[i] = glb_iaGPUArr[i] = glb_iaMemArr[i] = glb_iaChipArr[i] = glb_iaLoadArr[i] = glb_iaSpeedArr[i] = 0;

  activeTimer = FALSE;
  tickCounterCPU = tickCounterFanForce = tickCounterHD = 0;

  OSVERSIONINFO ovi;

  ovi.dwOSVersionInfoSize = sizeof( ovi );

  if( GetVersionEx(&ovi) == 0 )
    return FALSE;		// dies sollte niemals passieren!

  log( L"Running on %s Version %d.%d Build %d (%s)",
       ovi.dwPlatformId==VER_PLATFORM_WIN32_NT 
           ? L"Windows NT"
           : (ovi.dwPlatformId==VER_PLATFORM_WIN32_WINDOWS
           ? L"Windows 9x" : L"Windows 3.1 (?)"),
       ovi.dwMajorVersion,
       ovi.dwMinorVersion,
       ovi.dwBuildNumber,
       ovi.szCSDVersion );

  if( ovi.dwPlatformId != VER_PLATFORM_WIN32_NT )	
  {
    log( L"/aError" );
    MessageBoxW( L"Can't run on Windows 95/98/ME", L"Error", MB_OK|MB_ICONSTOP ); 
    PostMessage( WM_CLOSE );
    return FALSE;
  }

  // wir laufen auf NT4/2000/XP/Server2003/Vista => FanIO-Treiber suchen
  log( L"/bSearching for fanio driver... " );

  glb_hHndFile = CreateFileW( L"\\\\.\\FanIODev",
                              GENERIC_WRITE,
                              FILE_SHARE_WRITE,
                              NULL,
                              OPEN_EXISTING,
                              0,
                              NULL );

  if( glb_hHndFile == INVALID_HANDLE_VALUE )
  {
    log( L"/afailed" );
    CString s2;
    s1.LoadStringW( IDS_DLG2 );
    s2.LoadStringW( IDS_SERROR );
    MessageBoxW( s1, s2, MB_OK|MB_ICONSTOP ); 
    PostMessage( WM_CLOSE );
    return FALSE;
  }

  log( L"/aOk" );
  log( L"Found fanio driver: 0x%08x", glb_hHndFile );

  // Prfung der Treiberversion
  log( L"/bChecking fanio driver version... " );
  ULONG v[2];
  if( !getDriverVersion(v,glb_hHndFile) )
  {
    log( L"/afailed" );
    CString s2;
    s1.LoadStringW( IDS_DLG3 );
    s2.LoadStringW( IDS_SERROR );
    MessageBoxW( s1, s2, MB_OK|MB_ICONSTOP ); 
    PostMessage( WM_CLOSE );
    return FALSE;
  }

  log( L"/aOk... detected version %d.%d", v[0], v[1] );

  if( v[0]!=2 || v[1]!=3 )
  {
    CString s2;
    s1.LoadStringW( IDS_DLG4 );
    s2.LoadStringW( IDS_SERROR );
    swprintf_s( text, s1, v[0], v[1] ); 
    MessageBoxW( text, s2, MB_OK|MB_ICONSTOP ); 
    PostMessage( WM_CLOSE );
    return FALSE;
  }

  // Windows XP und hher
  if( ovi.dwMajorVersion>5 || (ovi.dwMajorVersion==5 && ovi.dwMinorVersion>=1) )
    glb_bWinXP = TRUE;

  // Windows 2000 und hher
  if( ovi.dwMajorVersion >= 5 )
    glb_bPowerDLL = TRUE;

  // Shell-Version ermitteln fr Trayicon-Darstellung
  DWORD shlver = getDllVersion( _T("comctl32.dll") );
  log( L"Common Controls DLL version: %d.%d", (shlver>>16)&65535, shlver&65535 );

  if( shlver >= PACKVERSION(5,00) )
    glb_bShellVer5 = true;

  log( L"New options: RCT:%d RFS1:%d RFS2:%d SD:%d TM:%d AS:%d",
       options.readCPUTemp,
       options.readFanSpeed1,
       options.readFanSpeed2,
       options.showDiagram,
       (options.opmode==TEMPMONITOR),
       options.autoStart );

  // Info ber Basissystem suchen und ausgeben
  if( !getSystemInfo(biosInfo,glb_hHndFile) )
  {
    CString s2;
    s1.LoadStringW( IDS_DLG5 );
    s2.LoadStringW( IDS_SERROR );
    MessageBoxW( s1, s2, MB_OK|MB_ICONEXCLAMATION );
    *biosInfo = '\0';
  }

  log( L"System info:" );

  wchar_t prodName[128];

  logInfoString( biosInfo, L"Product name:", prodName );
  logInfoString( biosInfo, L"BIOS version:", biosVer );
  logInfoString( biosInfo, L"Current CPU speed:", NULL );

  log( L"Detected: <%s>", prodName );

  // Prfung auf Modelltyp und setzen der Systemumgebung
  if( !options.modelChecked )
  {
    mcheck.checkModel( prodName, biosVer );
    options.modelChecked = true;
  }

  glb_intelDTSTempSupport = hasIntelDTSSupport();

  if( glb_intelDTSTempSupport )
  {
    log( L"Current core temp: %d", intelDTSReadTemp() );
    ((CWnd*)GetDlgItem(IDC_EDIT1))->ShowWindow( FALSE );
    ((CWnd*)GetDlgItem(IDC_CORE12))->ShowWindow( TRUE );
    ((CWnd*)GetDlgItem(IDC_TEMP1))->ShowWindow( TRUE );
    m_szCPUTempMC1 = L"--- / --- C";
  }

  log( L"System settings: NumFans:<%d> Fan1#:<%d> Fan2#:<%d> Alias1:<%s> Alias2:<%s> Divider:<%d> "
       L"CPU#:%d GPU#:%d Mem#:%d CS#:%d",
       options.numFans,
       options.fan1Index,
       options.fan2Index,
       options.fan1Alias,
       options.fan2Alias,
       options.fanSpeedDivider,
       options.cpuSensorNo,
       options.gpuSensorNo,
       options.memorySensorNo,
       options.chipsetSensorNo );

  glb_iLogCPUNum = (int)getLogicalCPUCount();
  log( L"Number of logical processors: %d", glb_iLogCPUNum );
  log( L"Process affinity mask: 0x%08x", glb_dProcessMask );
  log( L"System affinity mask: 0x%08x", glb_dSystemMask );

  cpuData.checkCPUType();
  log( L"CPU info:\n-----------------------------\n%s\n---------------------------\n", cpuData.flagText );

  // Test auf Hyperthreading, Multicore & SMP-Systeme und entsprechende Manahmen...
  testMCHT();

  if( glb_iLogCPUNum > 1 )
  {
    log( L"/bSwitching process threads to CPU affinity mask %d... ", glb_dTargetAffinity );
    if( !SetProcessAffinityMask(GetCurrentProcess(),glb_dTargetAffinity) ) 
    {
      DWORD err = GetLastError();
      log( L"/afailed; Errorcode:%d", err );
      log( L"Can't switch to first logical CPU; GetLastError:%d", err );
    }
    else
    {
      Sleep( 20 );
      log( L"/aOk" );
    }

    log( L"/bSwitching this thread to CPU affinity mask %d... ", glb_dTargetAffinity );
    if( !SetThreadAffinityMask(GetCurrentThread(),glb_dTargetAffinity) ) 
    {
      DWORD err = GetLastError();
      log( L"/afailed; Errorcode:%d", err );
      log( L"Can't switch to first logical CPU; GetLastError:%d", err );
    }
    else
    {
      Sleep( 20 );
      log( L"/aOk" );
    }

    log( L"/bChecking new process affinity... " );

    DWORD_PTR processAffMask;
    DWORD_PTR systemAffMask;
    if( !GetProcessAffinityMask(OpenProcess(PROCESS_ALL_ACCESS,false,GetCurrentProcessId()),&processAffMask,&systemAffMask) )
    {
      int err = GetLastError();
      log( L"/afailed; Errorcode:%d", err );
    }
    else if( processAffMask != glb_dTargetAffinity )
      log( L"/afailed; new ProcessAffinityMask:%08x", processAffMask );
    else 
      log( L"/aOk" );

    // AffinityMasks fr die einzelnen CPUs vorberechnen fr Ermittlung der Core-Temps
    if( glb_intelDTSTempSupport )
    {
      glb_iCPUAffinityMask[0] = glb_dTargetAffinity;

      for( int i=1; i<glb_iLogCPUNum; i++ )
        glb_iCPUAffinityMask[i] = (((DWORD_PTR)1)<<i);
    }
  }

  log( L"Opmode:%d", options.opmode );
  // Hauptfensteranzeigen entsprechend initialisieren:
  createUpperFanAlias();
  setFanAlias();
  maintainFanNumLimits();
  setDiagramScale();

  m_cGPUTemp.SetTextColor( RGB(255,0,0) );
  m_cMemoryTemp.SetTextColor( RGB(255,0,255) );
  m_cChipsetTemp.SetTextColor( RGB(0,255,255) );
  m_cCPUSpeed.SetTextColor( RGB(0,255,0) );
  setLegendColors();

  // BIOS-Version gendert => Auslesen einiger Sensoren ausschalten
  log( L"Current BIOS Ver: <%s>   Last BIOS Ver: <%s>", biosVer, options.biosVer );
  if( wcscmp(biosVer,options.biosVer) && *(options.biosVer) )
  {
    if( options.readGPUTemp || options.readMemoryTemp || options.readChipsetTemp )
    {
      log( L"Sensor reading options reset due to BIOS version change" );
      CString s2;
      s1.LoadStringW( IDS_DLG6 );
      s2.LoadStringW( IDS_IINFO );
      MessageBoxW( s1, s2, MB_ICONINFORMATION|MB_OK );
      options.readGPUTemp = FALSE;
      options.readMemoryTemp = FALSE;
      options.readChipsetTemp = FALSE;
    }
  }

  if( options.autoStart )
    setAutoStartRegistry();
  else
    deleteAutoStartRegistry();

  wcscpy_s( options.biosVer, biosVer );

  if( options.opmode==TEMPMONITOR && options.intervalCPU>=4 )
  {
    options.intervalCPU = 2;
    CString s2;
    s1.LoadStringW( IDS_DLG7 );
    s2.LoadStringW( IDS_IWARN );
    MessageBoxW( s1, s2, MB_ICONINFORMATION|MB_OK );
  }

  log( L"Opmode:%d", options.opmode );
  setFanSliders( options.sensorSupport );
  hideFanControl( options.opmode != MANUALCONTROL );    //options.observerMode || options.tempMonitor || options.fanForce );
  showTempMonElements( options.opmode == TEMPMONITOR );

  //showTempControl();
  log( L"Opmode:%d", options.opmode );


  detectHDs( TRUE );      // alle HDs testen und SMART aktivieren

  // suchen der ersten HD mit Temperatursupport
  if( options.readHDTemp )
  {
    if( options.hdNo>=MAXDRIVES || options.hds[options.hdNo]!=0 )
    {
      int i=0;
      for( ; i<MAXDRIVES; i++ )
      {
        if( options.hds[i] == 0 )
        {
          options.hdNo = i;
          break;
        }
      }

      if( i == MAXDRIVES )
        options.readHDTemp = FALSE;   // keine HD mit Temperatursupport gefunden
    }
  }

  if( glb_bPowerDLL )
    importPowerDLL();

  if( glb_bWinXP )
    importUxThemeDLL();

  readDataFromSensors( true );
  updateHDTemp();
  trayMan.displayTrayIcon( NIM_ADD, true, true, true );

  ((CScrollBar*)GetDlgItem(IDC_SCROLLBAR1))->SetScrollRange( 0, 90, FALSE );
  ((CScrollBar*)GetDlgItem(IDC_SCROLLBAR1))->SetScrollPos( 90, TRUE );

  log( L"/bCreating timer with interval %d msec... ", glb_iEffectiveTimerBase );

  if( SetTimer(timerID,glb_iEffectiveTimerBase,NULL) == timerID )
  {
    activeTimer = TRUE;
    log( L"/aOk" );
  }
  else
  {
    log( L"/bfailed" );
    CString s2;
    s1.LoadStringW( IDS_DLG8 );
    s2.LoadStringW( IDS_SERROR);
    MessageBoxW( s1, s2, MB_ICONSTOP|MB_OK );
    return FALSE;
  }

  // Warnmeldung und Tray-Minimizing wieder aktivieren fr Interativ-Betrieb
  controlQuitWarning = true;
  forceShutdown = false;

  m_cControlProfile.SetCurSel( options.preDefSet );
  log( L"Control mode index: %d", options.preDefSet );

  optionsDlgActive = FALSE;
  fillProfileList();

  s1.LoadStringW( IDS_DLG9 );
  addToolTip( 0, IDC_EDIT16,  s1 );
  s1.LoadStringW( IDS_DLG10 );
  addToolTip( 1, IDC_EDIT6,   s1 );
  s1.LoadStringW( IDS_DLG11 );
  addToolTip( 2, IDC_SLIDER2, s1 );
  s1.LoadStringW( IDS_DLG12 );
  addToolTip( 3, IDC_SLIDER1, s1 );
  s1.LoadStringW( IDS_DLG13 );
  addToolTip( 4, IDC_EDIT11,  s1 );
  s1.LoadStringW( IDS_DLG14 );
  addToolTip( 5, IDC_BUTTON2, s1 );

  m_bMaintainSpeed = options.maintainSpeeds;

  if( options.opmode == FANFORCING )
  {
    glb_bActiveFanForcing = true;
    glb_bSensorFanForcing = false;
    glb_iFanForcingValue1 = (options.numFans>0 && options.fan1Force) ? 2 : -1;
    glb_iFanForcingValue2 = (options.numFans>1 && options.fan2Force) ? 2 : -1;
  }

  showTempMonElements( options.opmode==TEMPMONITOR );

  UpdateData( FALSE );

  if( glb_bWinXP )
    wtsStatus = wtsRegistration();

  log( L"Opmode:%d", options.opmode );

  ((CButton*)GetDlgItem(IDC_RADIO1))->SetCheck( options.opmode==OBSERVERMODE );
  ((CButton*)GetDlgItem(IDC_RADIO2))->SetCheck( options.opmode==MANUALCONTROL ); //!options.observerMode && !options.fanForce && !options.tempMonitor );
  ((CButton*)GetDlgItem(IDC_RADIO4))->SetCheck( options.opmode==TEMPMONITOR );
  ((CButton*)GetDlgItem(IDC_RADIO3))->SetCheck( options.opmode==FANFORCING );

  initComplete = true;

  log( L"Main window successfully created" );

/*
  //------------------------------------------
  int code=0, driverCode=0;
  PUCHAR biosVA=NULL;
  ULONG biosSize=0;

  if( !getBIOSImage(glb_hHndFile,code,driverCode,&biosVA,&biosSize) )
  {
    // Image nur teilweise erhalten, also Block wieder freigeben, dann Fehlermeldung
    if( code == 2 )
      delete[] biosVA;

    log( L"ERROR retrieving video bios info: %d / %d", code, driverCode );
  }
  else    // Image komplett erhalten !
  {
    // Image verarbeiten, durchsuchen etc.
    // ...

    delete[] biosVA;
  }
  //------------------------------------------
*/

	return TRUE;
}

//---------------------------------------------------------------
// CLI-Optionen auswerten und diverse Startaufgaben erledigen
// (wird nach OnInitDialog() ausgefhrt)
//---------------------------------------------------------------
void CI8kfanguiDlg::setCmdLine( wchar_t* cl )
{
  log( L"Checking command line args..." );

  if( !_wcsicmp(cl,L"/startup") )
  {
    if( options.autoStart )
    {
      if( options.acOnly )
      {
        SYSTEM_POWER_STATUS ps;
      
        if( GetSystemPowerStatus(&ps)!=0 && ps.ACLineStatus==0 )
        {
          controlQuitWarning = false;
          forceShutdown = true;
          PostMessage( WM_CLOSE, 0, 0 );
          return;
        }
      }

      if( options.opmode==MANUALCONTROL && options.autoFanSetting ) //!options.tempMonitor && !options.observerMode && !options.fanForce )
      {
        glb_iDirectSpeedMethod = (options.autoStartHWFans ? 1 : 0);

        if( options.autoStartHWFans )
        {
          glb_iDirectSpeed1 = options.autoSpeedDirect1;
          glb_iDirectSpeed2 = options.autoSpeedDirect2;
          setFanSpeedSensor( options.fan1SensorIndex, glb_iDirectSpeed1, glb_hHndFile );
          setFanSpeedSensor( options.fan2SensorIndex, glb_iDirectSpeed2, glb_hHndFile );
        }
        else
        {
          setSpeed( options.autoOuterSlider, options.autoInnerSlider );

          if( options.autoOuterSlider > 0 )
            glb_iDirectSpeed1 = options.autoOuterSlider-1;
          else
            options.autoOuterSlider = 0;

          if( options.autoInnerSlider > 0 )
            glb_iDirectSpeed2 = options.autoInnerSlider-1;
          else
            options.autoInnerSlider = 0;
        }

        glb_bDirectSpeedApplied = TRUE;

        if( options.terminate )
        {
          forceShutdown = true;
          controlQuitWarning = false;
          PostMessage( WM_CLOSE, 0, 0 );
          return;
        }

        if( !options.maintainSpeeds )
        {
          options.maintainSpeeds = TRUE;
          m_bMaintainSpeed = TRUE;
          UpdateData( FALSE );
        }

        glb_bActiveFanForcing = (glb_bDirectSpeedApplied ? true : false);

        if( glb_bDirectSpeedApplied )
        {
          glb_bSensorFanForcing = (glb_iDirectSpeedMethod!=0);
          glb_iFanForcingValue1 = options.autoOuterSlider-1;
          glb_iFanForcingValue2 = options.autoInnerSlider-1;
          checkTimerInterval();
        }
      }

      minimizeToTray();   // gleich ins Tray minimieren beim Start
    }
  }
  else
    ShowWindow( SW_SHOW );

  log( L"End of CL option evaluation" );
}

//---------------------------------------------------------------
// Vorprfung neuer Nachrichten (notwendig fr Tooltips)
//---------------------------------------------------------------
BOOL CI8kfanguiDlg::PreTranslateMessage( MSG* pMsg ) 
{
  if( initComplete )
    for( int i=0; i<6; i++ )
      m_tToolTip[i].RelayEvent( pMsg );

  return CDialog::PreTranslateMessage( pMsg );
}

//---------------------------------------------------------------
// nur Windows XP+: als Empfnger fr WTS-Nachrichten registrieren
//---------------------------------------------------------------
int CI8kfanguiDlg::wtsRegistration()
{
  static int cnt=0;

  if( ++cnt > 5 )
    return 1;     // max. 5 Versuche

  log( L"Importing WTSAPI32.DLL for XP (try #%d)", cnt );
  wtsApi32DLL = LoadLibraryW( L"wtsapi32.dll" );
  if( wtsApi32DLL == NULL )
  {
    CString s1, s2;
    s1.LoadStringW( IDS_DLG15 );
    s2.LoadStringW( IDS_SERROR );
    MessageBoxW( s1, s2, MB_ICONEXCLAMATION ); 
    return 1;
  } 

  log( L"Importing WTS reg/unreg functions" );
  sr = (BOOL (WINAPI *)(HWND,DWORD))GetProcAddress( wtsApi32DLL, "WTSRegisterSessionNotification" );
  su = (BOOL (WINAPI *)(HWND))GetProcAddress( wtsApi32DLL, "WTSUnRegisterSessionNotification" );

  if( !sr || !su )
  {
    CString s1, s2;
    s1.LoadStringW( IDS_DLG16 );
    s2.LoadStringW( IDS_SERROR );
    MessageBoxW( s1, s2, MB_ICONEXCLAMATION|MB_OK );
    FreeLibrary( wtsApi32DLL );
    return 2;
  }

  log( L"Registering for session change notifications" );
  if( !sr(m_hWnd,NOTIFY_FOR_THIS_SESSION) )
  {
    log( L"Registration failed" );
    FreeLibrary( wtsApi32DLL );
    return 3;     // spter nochmal versuchen
  }

  log( L"Registration succeeded" );

  return 0;       // Erfolg!
}

//---------------------------------------------------------------
// Optionen aus Registry lesen
//---------------------------------------------------------------
void CI8kfanguiDlg::getRegKeys()
{
  if( !options.getRegKeys() )
  {
    CString s1, s2;
    s1.LoadStringW( IDS_DLG17 );
    s2.LoadStringW( IDS_SERROR );
    MessageBoxW( s1, s2, MB_ICONEXCLAMATION|MB_OK );
  }
}

//---------------------------------------------------------------
// aktuelle Optionen in Registry schreiben
//---------------------------------------------------------------
void CI8kfanguiDlg::setRegKeys()
{
  if( !options.setRegKeys() )
  {
    CString s1, s2;
    s1.LoadStringW( IDS_DLG18 );
    s2.LoadStringW( IDS_SERROR );
    MessageBoxW( s1, s2, MB_ICONEXCLAMATION|MB_OK );
  }
}

//---------------------------------------------------------------
// Ins Tray minimieren
//---------------------------------------------------------------
void CI8kfanguiDlg::minimizeToTray() 
{
  trayMan.displayTrayIcon( NIM_ADD );
  ShowWindow( SW_HIDE );
}

//---------------------------------------------------------------
// Systemmennachrichten verarbeiten
//---------------------------------------------------------------
void CI8kfanguiDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if( (nID & 0xFFF0) == IDM_ABOUTBOX )
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
		CDialog::OnSysCommand( nID, lParam );
}

//---------------------------------------------------------------
// WM_PAINT-Nachricht verarbeiten frs Hauptfenster
//---------------------------------------------------------------
void CI8kfanguiDlg::OnPaint() 
{
	if( IsIconic() )
	{
		CPaintDC dc( this );

		SendMessage( WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0 );

		int cxIcon = GetSystemMetrics( SM_CXICON );
		int cyIcon = GetSystemMetrics( SM_CYICON );
		CRect rect;
		GetClientRect( &rect );
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		dc.DrawIcon( x, y, m_hIcon );
	}
	else
  {
  	drawDiagram();
		CDialog::OnPaint();
  }
}

//---------------------------------------------------------------
// Icon fr minimiertes Fenster zurckgeben
//---------------------------------------------------------------
HCURSOR CI8kfanguiDlg::OnQueryDragIcon()
{
	return (HCURSOR)m_hIcon;
}

//---------------------------------------------------------------
// GUI-Elemente fr Lfter 1(!) ein-/ausblenden
//---------------------------------------------------------------
void CI8kfanguiDlg::OnCheck2()
{
	UpdateData();

	((CSliderCtrl*)GetDlgItem(IDC_SLIDER2))->EnableWindow( m_bOuterFan );
	((CStatic*)GetDlgItem(IDC_STATIC4))->EnableWindow( m_bOuterFan );
	((CStatic*)GetDlgItem(IDC_STATIC5))->EnableWindow( m_bOuterFan );
	((CStatic*)GetDlgItem(IDC_STATIC6))->EnableWindow( m_bOuterFan );
	((CButton*)GetDlgItem(IDC_APPLY))->EnableWindow( m_bInnerFan||m_bOuterFan );
  ((CWnd*)GetDlgItem(IDC_EDIT14))->EnableWindow( m_bOuterFan );
}

//---------------------------------------------------------------
// GUI-Elemente fr Lfter 2(!) ein-/ausblenden
//---------------------------------------------------------------
void CI8kfanguiDlg::OnCheck1() 
{
	UpdateData();

  ((CSliderCtrl*)GetDlgItem(IDC_SLIDER1))->EnableWindow( m_bInnerFan );
	((CStatic*)GetDlgItem(IDC_STATIC1))->EnableWindow( m_bInnerFan );
	((CStatic*)GetDlgItem(IDC_STATIC2))->EnableWindow( m_bInnerFan );
	((CStatic*)GetDlgItem(IDC_STATIC3))->EnableWindow( m_bInnerFan );
	((CButton*)GetDlgItem(IDC_APPLY))->EnableWindow( m_bInnerFan||m_bOuterFan );
  ((CWnd*)GetDlgItem(IDC_EDIT15))->EnableWindow( m_bInnerFan );
}

//---------------------------------------------------------------
// Handler fr 'Apply'-Button der direkten Lfterkontrolle
//---------------------------------------------------------------
void CI8kfanguiDlg::OnApply() 
{
  CString s1, s2, s3, w1, w2, w3, w4, w5, w6, warn;
  s1.LoadStringW( IDS_DLG19 );
  s2.LoadStringW( IDS_DLG20 );
  s3.LoadStringW( IDS_DLG21 );
  warn.LoadStringW( IDS_WARN );

  w1.Format( s1, options.fan1Alias );
  w2.Format( s2, options.fan1Alias );
  w3.Format( s3, options.fan1Alias );
  w4.Format( s1, options.fan2Alias );
  w5.Format( s2, options.fan2Alias );
  w6.Format( s3, options.fan2Alias );

#define WARNMSG1 (MessageBox(w1,warn,MB_ICONEXCLAMATION|MB_YESNO)==IDYES)

#define WARNMSG2 (MessageBox(w2,warn,MB_ICONEXCLAMATION|MB_YESNO)==IDYES)

#define WARNMSG3 (MessageBox(w3,warn,MB_ICONEXCLAMATION|MB_OK)==125)

#define WARNMSG4 (MessageBox(w4,warn,MB_ICONEXCLAMATION|MB_YESNO)==IDYES)

#define WARNMSG5 (MessageBox(w5,warn,MB_ICONEXCLAMATION|MB_YESNO)==IDYES)

#define WARNMSG6 (MessageBox(w6,warn,MB_ICONEXCLAMATION|MB_OK)==125)

	UpdateData();

  int outerSpeed = m_scOuterSlider.GetPos();
  int innerSpeed = m_scInnerSlider.GetPos();

  if( options.opmode == TEMPMONITOR )
  {
    CString r1, r2;
    r1.LoadStringW( IDS_DLG22 );
    r2.LoadStringW( IDS_IINFO );
    MessageBox( r1, r2, MB_ICONEXCLAMATION|MB_OK );
    return;
  }

  if( !options.sensorSupport )
  {
    static int speedvalue[] = { 1, 2, 0 };    // 1=slow, 2=high, 0=off

    // kein effizienter Programmierstil, aber dafr lesbarer, anstatt die Bedingungen
    // tief zu verschachteln:
    if( (m_bOuterFan && outerSpeed<2)
    ||  (m_bOuterFan && outerSpeed==2 && glb_iCPUTemp==0 && WARNMSG1)
    ||  (m_bOuterFan && outerSpeed==2 && (glb_iCPUTemp>0 && glb_iCPUTemp<65))
    ||  (m_bOuterFan && outerSpeed==2 && (glb_iCPUTemp>=65 && glb_iCPUTemp<75) && WARNMSG2)
    ||  (m_bOuterFan && outerSpeed==2 && glb_iCPUTemp>=75 && WARNMSG3) )
  	{
      log( L"Manual switch: Fan #1 to %d", speedvalue[outerSpeed] );
      setSpeed( speedvalue[outerSpeed]+1, 0 );
      glb_bDirectSpeedApplied = TRUE;
      glb_iDirectSpeedMethod = 0;
      glb_iDirectSpeed1 = speedvalue[outerSpeed];

      if( options.maintainSpeeds )
      {
        glb_bActiveFanForcing = true;
        glb_bSensorFanForcing = false;
        glb_iFanForcingValue1 = glb_iDirectSpeed1;
      }
	  }
    else if( glb_bActiveFanForcing )
      glb_iFanForcingValue1 = -1;

    if( (m_bInnerFan && innerSpeed<2)
    ||  (m_bInnerFan && innerSpeed==2 && glb_iCPUTemp==0 && WARNMSG4)
    ||  (m_bInnerFan && innerSpeed==2 && (glb_iCPUTemp>0 && glb_iCPUTemp<65))
    ||  (m_bInnerFan && innerSpeed==2 && (glb_iCPUTemp>=65 && glb_iCPUTemp<75) && WARNMSG5)
    ||  (m_bInnerFan && innerSpeed==2 && glb_iCPUTemp>=75 && WARNMSG6) )
	  {
      log( L"Manual switch: Fan #2 to %d", speedvalue[innerSpeed] );
      setSpeed( 0, speedvalue[innerSpeed]+1 );
      glb_bDirectSpeedApplied = TRUE;
      glb_iDirectSpeedMethod = 0;
      glb_iDirectSpeed2 = speedvalue[innerSpeed];

      if( options.maintainSpeeds )
      {
        glb_bActiveFanForcing = true;
        glb_bSensorFanForcing = false;
        glb_iFanForcingValue2 = glb_iDirectSpeed2;
      }
    }
    else if( glb_bActiveFanForcing )
      glb_iFanForcingValue2 = -1;
  }
  else
  {
    // kein effizienter Programmierstil, aber dafr lesbarer, anstatt die Bedingungen
    // tief zu verschachteln:
    if( (m_bOuterFan && outerSpeed>0)
    ||  (m_bOuterFan && outerSpeed==0 && glb_iCPUTemp==0 && WARNMSG1)
    ||  (m_bOuterFan && outerSpeed==0 && (glb_iCPUTemp>0 && glb_iCPUTemp<65))
    ||  (m_bOuterFan && outerSpeed==0 && (glb_iCPUTemp>=65 && glb_iCPUTemp<75) && WARNMSG2)
    ||  (m_bOuterFan && outerSpeed==0 && glb_iCPUTemp>=75 && WARNMSG3) )
  	{
      log( L"Manual switch: Fan #1 to %d", outerSpeed );
      setFanSpeedSensor( options.fan1SensorIndex, outerSpeed, glb_hHndFile );
      glb_bDirectSpeedApplied = TRUE;
      glb_iDirectSpeedMethod = 1;
      glb_iDirectSpeed1 = outerSpeed;

      if( options.maintainSpeeds )
      {
        glb_bActiveFanForcing = true;
        glb_bSensorFanForcing = true;
        glb_iFanForcingValue1 = glb_iDirectSpeed1;
      }
	  }

    if( (m_bInnerFan && innerSpeed>0)
    ||  (m_bInnerFan && innerSpeed==0 && glb_iCPUTemp==0 && WARNMSG4)
    ||  (m_bInnerFan && innerSpeed==0 && (glb_iCPUTemp>0 && glb_iCPUTemp<65))
    ||  (m_bInnerFan && innerSpeed==0 && (glb_iCPUTemp>=65 && glb_iCPUTemp<75) && WARNMSG5)
    ||  (m_bInnerFan && innerSpeed==0 && glb_iCPUTemp>=75 && WARNMSG6) )
	  {
      log( L"Manual switch: Fan #2 to %d", innerSpeed );
      setFanSpeedSensor( options.fan2SensorIndex, innerSpeed, glb_hHndFile );
      glb_bDirectSpeedApplied = TRUE;
      glb_iDirectSpeedMethod = 1;
      glb_iDirectSpeed2 = innerSpeed;

      if( options.maintainSpeeds )
      {
        glb_bActiveFanForcing = true;
        glb_bSensorFanForcing = true;
        glb_iFanForcingValue2 = glb_iDirectSpeed2;
      }
	  }
  }

#undef WARNMSG6
#undef WARNMSG5
#undef WARNMSG4
#undef WARNMSG3
#undef WARNMSG2
#undef WARNMSG1
}

//---------------------------------------------------------------
// Info-Dialog anzeigen
//---------------------------------------------------------------
void CI8kfanguiDlg::OnAbout() 
{
	CAboutDlg dlgAbout;
	dlgAbout.DoModal();	
}

//---------------------------------------------------------------
// Handler fr Nachrichten vom Trayicon
//---------------------------------------------------------------
LRESULT CI8kfanguiDlg::OnIconNotify( WPARAM w, LPARAM l )
{
  switch( l )
  {
    case WM_LBUTTONUP:
          inPopupCreation = true;
          ShowWindow( SW_SHOW );
          SetForegroundWindow();
          inPopupCreation = false;
          break;

    case WM_RBUTTONUP:
          if( optionsDlgActive )
          {
            inPopupCreation = true;
            ShowWindow( SW_SHOW );
            SetForegroundWindow();
            inPopupCreation = false;
          }
          else
            createPopupMenu();
          break;
  }

  return 0;
}

//---------------------------------------------------------------
// Kontextmen frs Trayicon aufbauen und anzeigen
//---------------------------------------------------------------
void CI8kfanguiDlg::createPopupMenu()
{
  log( L"Start von createPopupMenu; Mode:%d", options.opmode );

  CMenu menu;
  CMenu submenu2;
  CMenu submenu;
  CString s1;

  inPopupCreation = true;

  menu.CreatePopupMenu();

  submenu2.CreatePopupMenu();
  s1.LoadStringW( IDS_DLG23 );
  submenu2.AppendMenu( MF_STRING | (options.opmode==OBSERVERMODE ? MF_CHECKED : 0), 
                       IDM_OBSERVERMODE, 
                       s1 );
  s1.LoadStringW( IDS_DLG24 );
  submenu2.AppendMenu( MF_STRING | (options.opmode==MANUALCONTROL ? MF_CHECKED : 0),
                       IDM_MANUALFANS, 
                       s1 );
  s1.LoadStringW( IDS_DLG25 );
  submenu2.AppendMenu( MF_STRING | /*(options.readCPUTemp ? 0 : MF_GRAYED) |*/ (options.opmode==TEMPMONITOR ? MF_CHECKED : 0),
                       IDM_TEMPCONTROL,
                       s1 );
  s1.LoadStringW( IDS_DLG26 );
  submenu2.AppendMenu( MF_STRING | (options.opmode==FANFORCING ? MF_CHECKED : 0),
                       IDM_FANFORCING,
                       s1 );

  s1.LoadStringW( IDS_DLG27 );
  menu.AppendMenu( MF_POPUP,
                   (UINT_PTR)submenu2.m_hMenu,
                   s1 );

  submenu.CreatePopupMenu();

  for( int i=0; i<4; i++ )
    submenu.AppendMenu( MF_STRING | (options.preDefSet==i ? MF_CHECKED : 0),
                        IDM_TEMPPROFILE1+i, 
                        options.ms[i].alias );

  menu.AppendMenu( MF_SEPARATOR );

  s1.LoadStringW( IDS_DLG28 );
  menu.AppendMenu( MF_POPUP | (options.opmode==TEMPMONITOR ? 0 : MF_GRAYED), 
                   (UINT_PTR)submenu.m_hMenu, 
                   s1 );

  menu.AppendMenu( MF_SEPARATOR );

  s1.LoadStringW( IDS_DLG29 );
  menu.AppendMenu( MF_STRING | (options.showDiagram ? MF_CHECKED : 0),
                   IDM_SHOWDIAGRAM,
                   s1 );

  menu.AppendMenu( MF_SEPARATOR );
  CMenu iconMenu1;
  CMenu iconMenu2;
  iconMenu1.CreatePopupMenu();
  iconMenu2.CreatePopupMenu();
  s1.LoadStringW( IDS_DLG30 );
  iconMenu1.AppendMenu( MF_STRING | (options.trayIcon1==0 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON01,
                        s1 );
  iconMenu2.AppendMenu( MF_STRING | (options.trayIcon2==0 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON02,
                        s1 );
  s1.LoadStringW( IDS_DLG31 );
  iconMenu1.AppendMenu( MF_STRING | (options.trayIcon1==1 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON11,
                        s1 );
  iconMenu2.AppendMenu( MF_STRING | (options.trayIcon2==1 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON12,
                        s1 );
  s1.LoadStringW( IDS_DLG32 );
  iconMenu1.AppendMenu( MF_STRING | (options.trayIcon1==2 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON21,
                        s1 );
  iconMenu2.AppendMenu( MF_STRING | (options.trayIcon2==2 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON22,
                        s1 );
  s1.LoadStringW( IDS_DLG33 );
  iconMenu1.AppendMenu( MF_STRING | (options.trayIcon1==3 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON31,
                        s1 );
  iconMenu2.AppendMenu( MF_STRING | (options.trayIcon2==3 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON32,
                        s1 );
  s1.LoadStringW( IDS_DLG34 );
  iconMenu1.AppendMenu( MF_STRING | (options.trayIcon1==4 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON41,
                        s1 );
  iconMenu2.AppendMenu( MF_STRING | (options.trayIcon2==4 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON42,
                        s1 );
  s1.LoadStringW( IDS_DLG35 );
  iconMenu1.AppendMenu( MF_STRING | (options.trayIcon1==5 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON51,
                        s1 );
  iconMenu2.AppendMenu( MF_STRING | (options.trayIcon2==5 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON52,
                        s1 );
  s1.LoadStringW( IDS_DLG36 );
  iconMenu1.AppendMenu( MF_STRING | (options.trayIcon1==6 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON61,
                        s1 );
  iconMenu2.AppendMenu( MF_STRING | (options.trayIcon2==6 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON62,
                        s1 );
  s1.LoadStringW( IDS_DLG37 );
  iconMenu1.AppendMenu( MF_STRING | (options.trayIcon1==7 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON71,
                        s1 );
  iconMenu2.AppendMenu( MF_STRING | (options.trayIcon2==7 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON72,
                        s1 );
  s1.LoadStringW( IDS_DLG38 );
  iconMenu1.AppendMenu( MF_STRING | (options.trayIcon1==8 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON81,
                        s1 );
  iconMenu2.AppendMenu( MF_STRING | (options.trayIcon2==8 ? MF_CHECKED : 0),
                        IDM_SHOWTRAYICON82,
                        s1 );

  s1.LoadStringW( IDS_DLG39 );
  menu.AppendMenu( MF_POPUP, (UINT_PTR)iconMenu1.m_hMenu, s1 );
  s1.LoadStringW( IDS_DLG40 );
  menu.AppendMenu( MF_POPUP, (UINT_PTR)iconMenu2.m_hMenu, s1 );
  s1.LoadStringW( IDS_DLG41 );
  menu.AppendMenu( MF_STRING | (options.speedIcon ? MF_CHECKED : 0), 
                   IDM_SPEEDICON, 
                   s1 );

  s1.LoadStringW( IDS_DLG42 );
  menu.AppendMenu( MF_SEPARATOR );
  menu.AppendMenu( MF_STRING, IDM_SENSORUPDATE, s1 );

  menu.AppendMenu( MF_SEPARATOR );

  CMenu submenu3;
  submenu3.CreatePopupMenu();
  s1.LoadStringW( IDS_DLG43 );
  submenu3.AppendMenuW( MF_STRING | (options.fahrenHeit ? 0 : MF_CHECKED),
                       IDM_CELSIUSUNIT,
                       s1 );
  s1.LoadStringW( IDS_DLG44 );
  submenu3.AppendMenu( MF_STRING | (options.fahrenHeit ? MF_CHECKED : 0),
                       IDM_FAHRENHEITUNIT,
                       s1 );

  s1.LoadStringW( IDS_DLG45 );
  menu.AppendMenu( MF_POPUP, (UINT_PTR)submenu3.m_hMenu, s1 );

  menu.AppendMenu( MF_SEPARATOR );
  s1.LoadStringW( IDS_DLG46 );
  menu.AppendMenu( MF_STRING, IDM_SHOWMAIN, s1 );
  SetMenuDefaultItem( menu, IDM_SHOWMAIN, FALSE );

  menu.AppendMenu( MF_SEPARATOR );
  s1.LoadStringW( IDS_DLG47 );
  menu.AppendMenu( MF_STRING, IDM_FORCESHUTDOWN, s1 );

  POINT Point;
  GetCursorPos( &Point );
  //log( L"Vor SetForegroundWindow" );
  SetForegroundWindow();

  menu.TrackPopupMenu( TPM_LEFTALIGN|TPM_BOTTOMALIGN|TPM_NONOTIFY,
                       Point.x,
                       Point.y,
                       this );
  //log( L"Nach TrackPopupMenu" );
  menu.Detach();
  inPopupCreation = false;
  log( L"Ende von createPopupMenu; Mode:%d", options.opmode );
}

//---------------------------------------------------------------
// die Handler fr Nachrichten vom Trayicon-Kontextmen...
//---------------------------------------------------------------
void CI8kfanguiDlg::OnShowMainMenu( void )
{ ShowWindow(SW_SHOW); }

//----------------------------------------------------
// Ein-/Ausschalten des Observer Mode vom Kontextmen
//----------------------------------------------------
void CI8kfanguiDlg::OnObserverModeMenu( void )
{
  log( L"Start von Icon->OnObserverModeMenu" );
  OnBnClickedRadio1();

  ((CButton*)GetDlgItem(IDC_RADIO1))->SetCheck( options.opmode==OBSERVERMODE );
  ((CButton*)GetDlgItem(IDC_RADIO2))->SetCheck( options.opmode==MANUALCONTROL );
  ((CButton*)GetDlgItem(IDC_RADIO4))->SetCheck( options.opmode==TEMPMONITOR );
  ((CButton*)GetDlgItem(IDC_RADIO3))->SetCheck( options.opmode==FANFORCING );
  log( L"Ende von Icon->OnObserverModeMenu" );
}

void CI8kfanguiDlg::OnManualControlMenu( void )
{
  log( L"Start von Icon->OnManualControlMenu" );
  OnBnClickedRadio2();

  ((CButton*)GetDlgItem(IDC_RADIO1))->SetCheck( options.opmode==OBSERVERMODE );
  ((CButton*)GetDlgItem(IDC_RADIO2))->SetCheck( options.opmode==MANUALCONTROL );
  ((CButton*)GetDlgItem(IDC_RADIO4))->SetCheck( options.opmode==TEMPMONITOR );
  ((CButton*)GetDlgItem(IDC_RADIO3))->SetCheck( options.opmode==FANFORCING );
  log( L"Ende von Icon->OnManualControlMenu" );
}

void CI8kfanguiDlg::OnTempControlMenu( void )
{
  log( L"Start von Icon->OnTempControlMenu" );
  OnBnClickedRadio4();

  ((CButton*)GetDlgItem(IDC_RADIO1))->SetCheck( options.opmode==OBSERVERMODE );
  ((CButton*)GetDlgItem(IDC_RADIO2))->SetCheck( options.opmode==MANUALCONTROL );
  ((CButton*)GetDlgItem(IDC_RADIO4))->SetCheck( options.opmode==TEMPMONITOR );
  ((CButton*)GetDlgItem(IDC_RADIO3))->SetCheck( options.opmode==FANFORCING );
  log( L"Ende von Icon->OnTempControlMenu" );
}

void CI8kfanguiDlg::OnForceHighSpeedMenu( void )
{
  log( L"Start von Icon->OnForceHighSpeedMenu" );
  OnBnClickedRadio3();

  ((CButton*)GetDlgItem(IDC_RADIO1))->SetCheck( options.opmode==OBSERVERMODE );
  ((CButton*)GetDlgItem(IDC_RADIO2))->SetCheck( options.opmode==MANUALCONTROL );
  ((CButton*)GetDlgItem(IDC_RADIO4))->SetCheck( options.opmode==TEMPMONITOR );
  ((CButton*)GetDlgItem(IDC_RADIO3))->SetCheck( options.opmode==FANFORCING );
  log( L"Ende von Icon->OnForceHighSpeedMenu" );
}

void CI8kfanguiDlg::OnTempProfile1Menu( void )
{
  options.preDefSet = 0;
  RESETLEVELS;
  m_cControlProfile.SetCurSel( options.preDefSet );
  UpdateData( FALSE );
  setRegKeys();
}

void CI8kfanguiDlg::OnTempProfile2Menu( void )
{
  options.preDefSet = 1;
  RESETLEVELS;
  m_cControlProfile.SetCurSel( options.preDefSet );
  UpdateData( FALSE );
  setRegKeys();
}

void CI8kfanguiDlg::OnTempProfile3Menu( void )
{
  options.preDefSet = 2;
  RESETLEVELS;
  m_cControlProfile.SetCurSel( options.preDefSet );
  UpdateData( FALSE );
  setRegKeys();
}

void CI8kfanguiDlg::OnTempProfile4Menu( void )
{
  options.preDefSet = 3;
  RESETLEVELS;
  m_cControlProfile.SetCurSel( options.preDefSet );
  UpdateData( FALSE );
  setRegKeys();
}

void CI8kfanguiDlg::OnCelsiusMenu( void )
{
  BOOL fa = options.fahrenHeit;
  options.fahrenHeit = FALSE;
  setRegKeys();

  if( fa != options.fahrenHeit )
  {
    setDiagramScale();
    trayMan.displayTrayIcon( NIM_MODIFY, true, true, false );
    diagramRedraw( TRUE );
  }

  OnBnClickedPeakbutton();
}

void CI8kfanguiDlg::OnFahrenheitMenu( void )
{
  BOOL fa = options.fahrenHeit;
  options.fahrenHeit = TRUE;
  setRegKeys();

  if( fa != options.fahrenHeit )
  {
    setDiagramScale();
    trayMan.displayTrayIcon( NIM_MODIFY, true, true, false );
    diagramRedraw( TRUE );
  }

  OnBnClickedPeakbutton();
}

void CI8kfanguiDlg::OnSensorUpdateMenu( void )
{
  readDataFromSensors();
  updateHDTemp();
}

void CI8kfanguiDlg::OnShowDiagram( void )
{
  options.showDiagram = !options.showDiagram;
  if( !options.showDiagram ) { diagramRedraw(TRUE); }
}

void CI8kfanguiDlg::OnSelectTrayIcon01( void )
{
  options.trayIcon1 = 0;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectTrayIcon02( void )
{
  options.trayIcon2 = 0;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectTrayIcon11( void )
{
  options.trayIcon1 = 1;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectTrayIcon12( void )
{
  options.trayIcon2 = 1;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectTrayIcon21( void )
{
  options.trayIcon1 = 2;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectTrayIcon22( void )
{
  options.trayIcon2 = 2;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectTrayIcon31( void )
{
  options.trayIcon1 = 3;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectTrayIcon32( void )
{
  options.trayIcon2 = 3;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectTrayIcon41( void )
{
  options.trayIcon1 = 4;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectTrayIcon42( void )
{
  options.trayIcon2 = 4;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectTrayIcon51( void )
{
  options.trayIcon1 = 5;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectTrayIcon52( void )
{
  options.trayIcon2 = 5;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectTrayIcon61( void )
{
  options.trayIcon1 = 6;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectTrayIcon62( void )
{
  options.trayIcon2 = 6;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectTrayIcon71( void )
{
  options.trayIcon1 = 7;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectTrayIcon72( void )
{
  options.trayIcon2 = 7;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectTrayIcon81( void )
{
  options.trayIcon1 = 8;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectTrayIcon82( void )
{
  options.trayIcon2 = 8;
  trayMan.displayTrayIcon( NIM_MODIFY );
}

void CI8kfanguiDlg::OnSelectSpeedIcon( void )
{
  options.speedIcon = !options.speedIcon;
  trayMan.displayTrayIcon( NIM_MODIFY, true, true, true );
}

void CI8kfanguiDlg::OnSelectShutdown( void )
{
  forceShutdown = true;
  PostMessage( WM_CLOSE );
}


//---------------------------------------------------------------
// Handler fr 'Options'-Button
//
// Aufruf durch:
//  - Klick auf den Optionsbutton
//---------------------------------------------------------------
void CI8kfanguiDlg::OnOptbutton() 
{
  log( L"OptionDlg starting" );

  // Dialog anlegen:
  CI8kfanguiOptionDlg dlg;

  // einige Einstellungen vorher merken:
  BOOL readhd = options.readHDTemp;
  int hdno = options.hdNo;
  BOOL readhd2 = options.readHDTemp2;
  int hdno2 = options.hdNo2;
  BOOL fa = options.fahrenHeit;

  optionsDlgActive = TRUE;    // Trayicon-Kontextmen nicht anzeigen, solange der Optionsdialog angezeigt wird
  INT_PTR x = dlg.DoModal();
  optionsDlgActive = FALSE;

  if( x != IDOK )
    return;

  // neue Einstellungen vom Optionsdialog zurcklesen
  dlg.getVars();

  m_cControlProfile.SetCurSel( options.preDefSet );
  showTempMonElements( options.opmode == TEMPMONITOR );
  fillProfileList();
  createUpperFanAlias();
  setFanAlias();
  maintainFanNumLimits();
  setDiagramScale();
  setLegendColors();

  if( !options.readCPUTemp )     { glb_iCPUTemp = 0; m_szCPUTemp = _T("---"); m_szCPUTempMC1=L"--- / --- C"; }
  if( !options.readGPUTemp )     { glb_iGPUTemp = 0; m_szGPUTemp = _T("---"); }
  if( !options.readMemoryTemp )  { glb_iMemoryTemp = 0; m_szMemoryTemp = _T("---"); }
  if( !options.readChipsetTemp ) { glb_iChipsetTemp = 0; m_szChipsetTemp = _T("---"); }
  if( !options.readHDTemp )      { m_iHDTemp=glb_iHDTemp=0; m_szHDTemp = _T("---"); }
  if( !options.readHDTemp2 )     { m_iHDTemp2=glb_iHDTemp2=0; m_szHDTemp2 = _T("---"); }
  if( !options.readFanSpeed1 )   { glb_iFanSpeed1 = 0; m_szFanSpeed1 = _T("---"); }
  if( !options.readFanSpeed2 )   { glb_iFanSpeed2 = 0; m_szFanSpeed2 = _T("---"); }
  if( !options.readFanStatus1 )  { m_szSpeedString1 = _T("---"); }
  if( !options.readFanStatus2 )  { m_szSpeedString2 = _T("---"); }
  if( !options.showDiagram )     { Invalidate(); }

  if( options.opmode == TEMPMONITOR )    // Tempmonitor ist aktiv, aber Temps gendert?
  {
    if( dlg.tempMonChanged() )
      RESETLEVELS;   // ja, Tempmonitor-Variablen zurcksetzen
  }

  setRegKeys();

  checkTimerInterval();

  if( options.opmode == FANFORCING )
  {
    glb_bActiveFanForcing = true;
    glb_bSensorFanForcing = false;
    glb_iFanForcingValue1 = (options.numFans>0 && options.fan1Force) ? 2 : -1;
    glb_iFanForcingValue2 = (options.numFans>1 && options.fan2Force) ? 2 : -1;
  }

  diagramRedraw( TRUE );

  setFanSliders( options.sensorSupport );
  //hideFanControl( options.opmode != MANUALCONTROL );
  //showTempControl();

  if( options.autoStart )
    setAutoStartRegistry();
  else
    deleteAutoStartRegistry();

  UpdateData( FALSE );

  readDataFromSensors();

  if( options.readHDTemp!=readhd || options.hdNo!=hdno || fa!=options.fahrenHeit || options.readHDTemp2!=readhd2 || options.hdNo2!=hdno2 )
    updateHDTemp();

  trayMan.displayTrayIcon( NIM_MODIFY, true, true, true );
}

//---------------------------------------------------------------
// Registryeintrag fr Autostart setzen
//
// Aufruf durch:
//   - OnInitDialog() -> beim Start der Anwendung Setzen des Eintrags, wenn 
//                       Autostart-Option aktiviert ist
//   - OnOptbutton() -> Setzen des Eintrags nach dem Schlieen des Optionsdialogs, 
//                      wenn Autostart-Option aktiviert ist
//---------------------------------------------------------------
void CI8kfanguiDlg::setAutoStartRegistry()
{
  LONG lReturn = 0;
  wchar_t path[264];
  CRegKey key;

  if( GetModuleFileNameW(NULL,path,255) == 0 )
  {
    CString s1, err;
    s1.LoadStringW( IDS_DLG48 );
    err.LoadStringW( IDS_SERROR );
    MessageBox( s1, err, MB_OK|MB_ICONEXCLAMATION );
    return;
  }

  wcscat_s( path, L" /startup" );

  lReturn = key.Create( HKEY_CURRENT_USER,
                        L"Software\\Microsoft\\Windows\\CurrentVersion\\Run\\" );

  if( lReturn == ERROR_SUCCESS )
  {
    if( key.SetValue(L"i8kfangui",REG_SZ,path,(ULONG)((_tcslen(path)+1)*2)) != ERROR_SUCCESS )
    {
      CString s1, err;
      s1.LoadStringW( IDS_DLG49 );
      err.LoadStringW( IDS_SERROR );
      MessageBox( s1, err, MB_ICONEXCLAMATION|MB_OK );
      key.Close();
      return;
    }
  }
  else
  {
    CString s1, err;
    s1.LoadStringW( IDS_DLG49 );
    err.LoadStringW( IDS_SERROR );
    MessageBox( s1, err, MB_ICONEXCLAMATION|MB_OK );
  }

  key.Close();
}

//---------------------------------------------------------------
// Autostart-Registryeintrag lschen
//
// Aufruf durch:
//   - OnInitDialog() -> beim Start der Anwendung Lschen des Eintrags, wenn 
//                       Autostart-Option deaktiviert ist
//   - OnOptbutton() -> Lschen des Eintrags nach dem Schlieen des Optionsdialogs, 
//                      wenn Autostart-Option deaktiviert ist
//---------------------------------------------------------------
void CI8kfanguiDlg::deleteAutoStartRegistry()
{
  CRegKey key;
  LONG result = key.Open( HKEY_CURRENT_USER,
                          L"Software\\Microsoft\\Windows\\CurrentVersion\\Run\\" );

  if( result == ERROR_SUCCESS )
    key.DeleteValue( L"i8kfangui");
  else
  {
    CString s1, err;
    s1.LoadStringW( IDS_DLG50 );
    err.LoadStringW( IDS_SERROR );
    MessageBox( s1, err, MB_ICONERROR );
  }
  
  key.Close();
}

//---------------------------------------------------------------
// Aufrumen nach Beenden (= Destruktor der Fensterinstanz)

// letzte Aktion dieser Klasse nach dem Schlieen des Fensters;
// wird automatisch vom Framework aufgerufen
//---------------------------------------------------------------
void CI8kfanguiDlg::PostNcDestroy()
{
  delete this;
}

//---------------------------------------------------------------
// WM_CLOSE verarbeiten
//
// Aufruf durch:
//   - Nachrichtenhandler der Anwendung, wenn die WM_CLOSE-Nachricht eingeht
//   - OnBnClickedClose() -> nach Klick auf den Close-Button, wenn nicht minimiert werden soll
//---------------------------------------------------------------
void CI8kfanguiDlg::OnClose()
{
  if( options.minimizeOnClose && !forceShutdown )
  {
    minimizeToTray();
    return;
  }

  forceShutdown = false;

  log( L"--------------------------------" );
  log( L"Begin termination" );

  if( closeQuery() )
  {
    NOTIFYICONDATA nid;
    nid.cbSize = sizeof( NOTIFYICONDATA );
    nid.hWnd = m_hWnd;
    nid.uID = 700;
    nid.uCallbackMessage = WM_ICONNOTIFY;

    if( options.tempIcon )
      nid.hIcon = glb_hTempIcon; 
    else
      nid.hIcon = m_hIcon;

    Shell_NotifyIcon( NIM_DELETE, &nid );

    if( glb_bTrayIcon2Active )
    {
      nid.cbSize = sizeof( NOTIFYICONDATA );
      nid.hWnd = m_hWnd;
      nid.uID = 701;
      nid.uCallbackMessage = WM_ICONNOTIFY;
      nid.hIcon = glb_hTempIcon2; 
      Shell_NotifyIcon( NIM_DELETE, &nid );
    }

    if( glb_bSpeedIconActive )
    {
      nid.cbSize = sizeof( NOTIFYICONDATA );
      nid.hWnd = m_hWnd;
      nid.uID = 702;
      nid.uCallbackMessage = WM_ICONNOTIFY;
      nid.hIcon = glb_hSpeedIcon; 
      Shell_NotifyIcon( NIM_DELETE, &nid );
    }

    setRegKeys();

    if( wtsStatus == 0 )
    {
      log( L"/bTerminating session notifications... " );
      su( m_hWnd );
      FreeLibrary( wtsApi32DLL );
      log( L"/aOk" );
    }

    if( activeTimer )
    {
      log( L"/bKilling timer (%d)... ", timerID );
      if( KillTimer(timerID) == 0 )
        log( L"/afailed" );
      else
        log( L"/aOk" );
      tickCounterCPU = 0;
      tickCounterFanForce = 0;
      tickCounterHD = 0;
    }

    if( glb_bPerfCnt )
    {
      log( L"/bStopping performance counter... " );
      stopPerfCnt();
      log( L"/aOk" );
    }

    if( glb_pPowerApiDLL )
    {
      log( L"/bUnloading POWRPROF.DLL... " );
      FreeLibrary( glb_pPowerApiDLL );
      glb_pCallPI = NULL;
      log( L"/aOk" );
    }

    if( glb_pUxThemeDLL )
    {
      log( L"/bUnloading UXTHEME.DLL... " );
      FreeLibrary( glb_pUxThemeDLL );
      glb_pCallThemeEnable = NULL;
      log( L"/aOk" );
    }

    log( L"/bReleasing fanio driver (0x%08x)... ", glb_hHndFile );
    if( CloseHandle(glb_hHndFile) )
      log( L"/aOk" );
    else
      log( L"/afailed" );

    log( L"/bClosing main window... " );
    DestroyIcon( glb_hTempIcon );
    DestroyIcon( glb_hSpeedIcon );
    DestroyWindow();
    log( L"/aOk" );
  }
}

//---------------------------------------------------------------
// Besttigung frs Beenden einholen, wenn Tempmonitor noch luft
//---------------------------------------------------------------
bool CI8kfanguiDlg::closeQuery()
{
  if( options.opmode==TEMPMONITOR && controlQuitWarning )
  {
    CString s1, info;
    s1.LoadStringW( IDS_DLG51 );
    info.LoadStringW( IDS_IINFO );
    if( MessageBox(s1,info,MB_ICONEXCLAMATION|MB_YESNO) != IDYES )
      return false;

    exitTempMonitor();
  }

  return true;
}

//---------------------------------------------------------------
// Highlevel-Schnittstelle zu Lfterdrehzahl-Funktionen
//---------------------------------------------------------------
int CI8kfanguiDlg::setSpeed( int outsp, int insp )
{
  int status=0;

  if( outsp>0 && !setFanSpeed(options.fan1Index,outsp-1,glb_hHndFile) )
  {
    CString s1, s2, err;
    s1.LoadStringW( IDS_DLG52 );
    s2.Format( s1, options.fan1Alias );
    err.LoadStringW( IDS_ERROR );
    MessageBox( s2, err, MB_ICONEXCLAMATION|MB_OK );
    status |= 1;
  }

  if( insp>0 && !setFanSpeed(options.fan2Index,insp-1,glb_hHndFile) )
  {
    CString s1, s2, err;
    s1.LoadStringW( IDS_DLG53 );
    s2.Format( s1, options.fan2Alias );
    err.LoadStringW( IDS_ERROR );
    MessageBox( s2, err, MB_ICONEXCLAMATION|MB_OK );
    status |= 2;
  }

  // Workaround fr I8100 BIOS A08: sofort Drehzahl nochmal setzen, damit kurzzeitiges
  // "Aufheulen" der Lfter beim Einschalten vermieden wird. Obwohl dies nur frs
  // I8100 BIOS A08 notwendig ist, sollte dies keine negativen Effekte bei anderen
  // Modellen oder BIOS-Versionen hervorrufen. Keine Ahnung, ob dies ein Bug oder
  // ein Feature vom BIOS A08 ist ;-)
  if( options.i8100Fix )
  {
    if( outsp>0 && !(status&1) )
      setFanSpeed( options.fan1Index, outsp-1, glb_hHndFile );

    if( insp>0 && !(status&2) )
      setFanSpeed( options.fan2Index, insp-1, glb_hHndFile );
  }

  log( L"Setting fan speeds to %d %d (%d)", outsp-1, insp-1, status );
  return status;
}

//---------------------------------------------------------------
// Handler fr WM_TIMER-Nachrichten
//---------------------------------------------------------------
void CI8kfanguiDlg::OnTimer( UINT_PTR nIDEvent ) 
{
  if( nIDEvent == timerID )
  {
    // Timer#1 abgelaufen ?
    if( (int)tickCounterCPU >= (options.intervalCPU*1000) )
    {  
      readDataFromSensors();    // interne Sensoren lesen
      tickCounterCPU = 0;
    }

    if( glb_bActiveFanForcing 
    && (options.opmode==FANFORCING || options.opmode==TEMPMONITOR || options.maintainSpeeds) 
    && (int)tickCounterFanForce>=fanForceTimeout[options.fanForceInterval] ) 
    {
      fanForcing();
      tickCounterFanForce = 0;
    }

    // Timer#2 abgelaufen ?
    if( (int)tickCounterHD >= options.intervalHD ) 
    {
      if( options.intervalHD != 0 )        // HD-Temp auslesen
        updateHDTemp();

      // WTS-Registrierung wiederholen falls noch nicht geklappt (synchron zu HD-Temp. Updates)
      if( glb_bWinXP && wtsStatus==3 && tickCounterHD>=60 ) 
        wtsStatus = wtsRegistration();

      if( options.intervalHD!=0 || (options.intervalHD==0 && tickCounterHD>=60) )
        tickCounterHD = 0;
    }

    tickCounterCPU += glb_iEffectiveTimerBase;
    tickCounterFanForce += glb_iEffectiveTimerBase;
    tickCounterHD++;
  }

  CDialog::OnTimer( nIDEvent );
}

//---------------------------------------------------------------
// Handler for 'Update' button
//---------------------------------------------------------------
void CI8kfanguiDlg::OnTempUpdate()
{
  readDataFromSensors();
  updateHDTemp();
}

//---------------------------------------------------------------
// Timer#1 abgelaufen => interne Sensordaten einlesen
//---------------------------------------------------------------
void CI8kfanguiDlg::readDataFromSensors( BOOL init )
{
  // nichts machen, wenn nicht aktive Session (Fast User Switching von XP)
  if( !activeSession )    
    return;

  // interne Temperaturen einlesen inklusive CPU-Speed und CPU-Last
  tempControl.readTempData( init );

  // gelesene Daten von globalen in GUI-Variablen packen
  m_szCPULoad = (CString)glb_szCPULoad + (CString)" %";
  m_szCPUSpeed = (CString)glb_szCPUSpeed + (CString)" MHz";
  m_szCPUTemp = (CString)glb_szCPUTemp + (CString) (options.fahrenHeit ? L" F" : L" C");

  if( glb_intelDTSTempSupport )
  {
    m_szCPUTempMC1 = (CString)glb_szCPUTempC1 + L" / "
                   + (CString)glb_szCPUTempC2 + (CString)(options.fahrenHeit ? L" F" : L" C");
  }
  
  m_szGPUTemp = (CString)glb_szGPUTemp + (CString) (options.fahrenHeit ? L" F" : L" C");
  m_szMemoryTemp = (CString)glb_szMemoryTemp + (CString) (options.fahrenHeit ? L" F" : L" C");;
  m_szChipsetTemp = (CString)glb_szChipsetTemp + (CString) (options.fahrenHeit ? L" F" : L" C");;

  // Peak-Temps setzen:
  if( glb_iCPUTemp > glb_iCPUMax )
  {
    glb_iCPUMax = glb_iCPUTemp;
    ((CStatic*)GetDlgItem(IDC_STATICCPUMAX))->SetWindowTextW( (CString)m_szCPUTemp );
  }

  if( glb_iGPUTemp > glb_iGPUMax )
  {
    glb_iGPUMax = glb_iGPUTemp;
    ((CWnd*)GetDlgItem(IDC_STATICGPUMAX))->SetWindowTextW( (CString)m_szGPUTemp );
  }

  if( glb_iMemoryTemp > glb_iMemoryMax )
  {
    glb_iMemoryMax = glb_iMemoryTemp;
    ((CWnd*)GetDlgItem(IDC_STATICMEMMAX))->SetWindowTextW( (CString)m_szMemoryTemp );
  }

  if( glb_iChipsetTemp > glb_iChipsetMax )
  {
    glb_iChipsetMax = glb_iChipsetTemp;
    ((CWnd*)GetDlgItem(IDC_STATICCHIPMAX))->SetWindowTextW( (CString)m_szChipsetTemp );
  }

  // jetzt die Stati und RPMs der Lfter einlesen
  tempControl.readFanStates();

  CString s1;
  s1.LoadStringW( IDS_DLG54 );

  // Lfterdaten in GUI-Variablen packen
  m_szFanSpeed1 = (CString)glb_szFanSpeed1 + s1;
  m_szFanSpeed2 = (CString)glb_szFanSpeed2 + s1;
  m_szSpeedString1 = glb_szSpeedString1;
  m_szSpeedString2 = glb_szSpeedString2;
  m_iSensorSpeed1 = glb_iSensorSpeed1;
  m_iSensorSpeed2 = glb_iSensorSpeed2;

  glb_iDiagramIndex++;
  if( glb_iDiagramIndex >= 1200 )
    glb_iDiagramIndex = 0;

  // Statusdiagramm aktualisieren
  if( options.showDiagram )   
  {
    InvalidateRect( &cpuFrame, FALSE );
    InvalidateRect( &legendFrame, FALSE );
  }

  UpdateData( FALSE );

  // Trayicon(s) anzeigen/aktualisieren
  trayMan.displayTrayIcon( NIM_MODIFY );

  log( L"%d %d %d %d / RPM1:%d (%d %d)/ RPM2:%d (%d %d)/ Sp:%d(Var:%d) / L:%d", 
       glb_iCPUTemp, 
       glb_iGPUTemp, 
       glb_iMemoryTemp, 
       glb_iChipsetTemp, 
       glb_iFanSpeed1, 
       options.readFanSpeed1,
       options.readFanStatus1,
       glb_iFanSpeed2,
       options.readFanSpeed2,
       options.readFanStatus2,
       glb_iCPUSpeed,
       options.readCPUVar,
       glb_iCPULoad );

  // Tempmonitor ausfhren, wenn er aktiviert ist
  if( options.opmode == TEMPMONITOR )
  {
    tempControl.control();

    // aktuelle Leveldaten im Hauptfenster anzeigen
    m_iCPULevel = glb_iCPULevel;
    m_iGPULevel = glb_iGPULevel;
    m_iMemoryLevel = glb_iMemoryLevel;
    m_iChipsetLevel = glb_iChipsetLevel;

    UpdateData( FALSE );
  }
}

//---------------------------------------------------------------
// Diagrammrahmen und Legende zeichnen und (sofern aktiv) den Inhalt
//---------------------------------------------------------------
void CI8kfanguiDlg::drawDiagram()
{
  CPaintDC dc( this );
  CBrush bBlackBrush( RGB(0,0,0) );                     // Schwarz als Pinsel
  CBrush bWhiteBrush( RGB(255,255,255) );               // Wei als Pinsel
  CPen bRed( PS_SOLID, 1, RGB(255,0,0) );               // Rot
  CPen bYellow( PS_SOLID, 1, RGB(255,255,0) );          // Gelb
  CPen bWhite( PS_SOLID, 1, RGB(255,255,255) );         // Wei
  CPen bGreen( PS_SOLID,1,RGB(0,255,0) );               // Grn
  CPen bMagenta( PS_SOLID, 1, RGB(255,0,255) );         // Magenta
  CPen bBlack( PS_SOLID, 1, RGB(0,0,0) );               // Schwarz
  CPen bBlue( PS_SOLID, 1, RGB(0,0,255) );              // Blau
  CPen bCyan( PS_SOLID, 1, RGB(0,255,255) );            // Cyan
  CPen bWhiteDotted( PS_DOT, 1, RGB(255,255,255) );     // Wei gepunktet fr Skala
  CPen bBlackDotted( PS_DOT, 1, RGB(0,0,0) );           // Schwarz gepunktet fr Skala

#define XBASE (cpuFrame.left)
#define YBASE (cpuFrame.top)

  // Bereich anschwrzen ;-)
  dc.SetBkMode( TRANSPARENT );
  dc.FillRect( &cpuFrame, (options.colorTheme==0) ? &bBlackBrush : &bWhiteBrush );

  // gestrichelte Linien im Diagramm bei 25, 50, 75, 100 und 125-Skala zeichnen
  CPen* old = dc.SelectObject( (options.colorTheme==0) ? &bWhiteDotted : &bBlackDotted );
  dc.MoveTo( XBASE+1, YBASE+25 );
  dc.LineTo( XBASE+351, YBASE+25 );
  dc.MoveTo( XBASE+1, YBASE+75 );
  dc.LineTo( XBASE+351, YBASE+75 );
  dc.MoveTo( XBASE+1, YBASE+100 );
  dc.LineTo( XBASE+351, YBASE+100 );
  dc.MoveTo( XBASE+1, YBASE+125 );
  dc.LineTo( XBASE+351, YBASE+125 );

//  old = dc.SelectObject( &bWhiteDashed );
  dc.MoveTo( XBASE+1, YBASE+50 );
  dc.LineTo( XBASE+351, YBASE+50 );

  if( options.showDiagram )
  {
    // Anfangswert fr Array-Zugriff bestimmen:
    int x = glb_iDiagramIndex-117-diagramOffset;
    
    // Pixelzhler zurcksetzen: zhlt die X-Ausbreitung "nach rechts" 
    // bis zum Ende des Zeichenbereichs:
    int run = 0;

    if( options.readCPUTemp && options.drawCPUTemp )
    {
      dc.SelectObject( (options.colorTheme==0) ? &bYellow : &bBlue );      // Zeichenfarbe fr CPU-Temperatur

      // der Zeichenbereich ist 350 Pixel in X-Richtung lang:
      while( run <= 348 )
      {
        // falls der Arrayzhler mal ins Negative rutscht, einfach einen Wraparound durchfhren:
        while( x < 0 )
          x = 1200+x;

        // Arraywert auslesen
        int a=glb_iaTempArr[x];
        
        // in Fahrenheit konvertieren, falls ntig:
        if( a && options.fahrenHeit )
          a = options.convertTemp( a, FAHRENHEIT ) / 2;

        if( a > 150 )
          a = 150;

        // beim ersten Schleifendurchlauf nur einen Pixel zeichnen:
        if( run == 0 )
          dc.MoveTo( XBASE+1+run+1, YBASE+150-a );

        dc.LineTo( XBASE+1+run+1, YBASE+150-a );

        x++;
        run += 3;   // um 3 Pixel vorrcken fr nchsten Samplewert

        // den Arrayzhler nicht bers Array hinauslaufen lassen:
        if( x >= 1200 )
          x = 0;
      }
    }

    if( options.readCPULoad && options.drawCPULoad )
    {
      x = glb_iDiagramIndex-117-diagramOffset;
      run = 0;

      dc.SelectObject( (options.colorTheme==0) ? &bWhite : &bBlack );
   
      while( run <= 348 )
      {
        while( x < 0 )
          x = 1200+x;

        int a=glb_iaLoadArr[x];
        if( a && options.fahrenHeit )
          a /= 2;

        if( a > 150 )
          a = 150;

        if( run == 0 )
          dc.MoveTo( XBASE+1+run+1, YBASE+150-a );

        dc.LineTo( XBASE+1+run+1, YBASE+150-a );

        x++;
        run += 3;

        if( x >= 1200 )
          x = 0;
      }
    }

    if( options.readCPUSpeed && options.drawCPUSpeed )
    {
      x = glb_iDiagramIndex-117-diagramOffset;
      run = 0;

      dc.SelectObject( &bGreen );
   
      while( run <= 348 )
      {
        while( x < 0 )
          x = 1200+x;

        int a=glb_iaSpeedArr[x] * options.speedScaling;
        if( a && options.fahrenHeit )
          a /= 2;

        if( a > 150 )
          a = 150;

        if( run == 0 )
          dc.MoveTo( XBASE+1+run+1, YBASE+150-a );

        dc.LineTo( XBASE+1+run+1, YBASE+150-a );

        x++;
        run += 3;

        if( x >= 1200 )
          x = 0;
      }
    }

    if( options.readGPUTemp && options.drawGPUTemp )
    {
      x = glb_iDiagramIndex-117-diagramOffset;
      run = 0;

      dc.SelectObject( &bRed );
   
      while( run <= 348 )
      {
        while( x < 0 )
          x = 1200+x;

        int a=glb_iaGPUArr[x];
        if( a && options.fahrenHeit )
          a = options.convertTemp( a, FAHRENHEIT ) / 2;

        if( a > 150 )
          a = 150;

        if( run == 0 )
          dc.MoveTo( XBASE+1+run+1, YBASE+150-a );

        dc.LineTo( XBASE+1+run+1, YBASE+150-a );

        x++;
        run += 3;

        if( x >= 1200 )
          x = 0;
      }
    }

    if( options.readMemoryTemp && options.drawMemoryTemp )
    {
      x = glb_iDiagramIndex-117-diagramOffset;
      run = 0;

      dc.SelectObject( &bMagenta );
   
      while( run <= 348 )
      {
        while( x < 0 )
          x = 1200+x;

        int a=glb_iaMemArr[x];
        if( a && options.fahrenHeit )
          a = options.convertTemp( a, FAHRENHEIT ) / 2;

        if( a > 150 )
          a = 150;

        if( run == 0 )
          dc.MoveTo( XBASE+1+run+1, YBASE+150-a );

        dc.LineTo( XBASE+1+run+1, YBASE+150-a );

        x++;
        run += 3;

        if( x >= 1200 )
          x = 0;
      }
    }

    if( options.readChipsetTemp && options.drawChipsetTemp )
    {
      x = glb_iDiagramIndex-117-diagramOffset;
      run = 0;

      dc.SelectObject( &bCyan );
   
      while( run <= 348 )
      {
        while( x < 0 )
          x = 1200+x;

        int a=glb_iaChipArr[x];
        if( a && options.fahrenHeit )
          a = options.convertTemp( a, FAHRENHEIT ) / 2;

        if( a > 150 )
          a = 150;

        if( run == 0 )
          dc.MoveTo( XBASE+1+run+1, YBASE+150-a );

        dc.LineTo( XBASE+1+run+1, YBASE+150-a );

        x++;
        run += 3;

        if( x >= 1200 )
          x = 0;
      }
    }
  }

#undef XBASE
#undef YBASE
}

//---------------------------------------------------------------
// Handler fr SessionChange-Nachrichten in Windows XP (FUS)
//---------------------------------------------------------------
LRESULT CI8kfanguiDlg::OnWtsSessionChange( WPARAM wp, LPARAM lp )
{
  switch( wp )
  {
    case WTS_CONSOLE_CONNECT:
          log( L"WTS_CONSOLE_CONNECT received => Session reactivated" );
          activeSession = TRUE;
          break;

    case WTS_CONSOLE_DISCONNECT:
          log( L"WTS_CONSOLE_DISCONNECT received => Session deactivated" );
          activeSession = FALSE;
          break;

    case WTS_REMOTE_CONNECT:
          log( L"WTS_REMOTE_CONNECT received => Session reactivated" );
          activeSession = TRUE;
          break;

    case WTS_REMOTE_DISCONNECT:
          log( L"WTS_REMOTE_DISCONNECT received => Session deactivated" );
          activeSession = FALSE;
          break;
  }

  return 0;
}

//---------------------------------------------------------------
// Timer#2 abgelaufen => HD-Temp auslesen
//---------------------------------------------------------------
void CI8kfanguiDlg::updateHDTemp()
{
  // nichts machen, wenn nicht aktive Session (Fast User Switching von XP)
  if( !activeSession )    
    return;

  BOOL ret;
  int t;

  if( options.readHDTemp )
  {
    log( L"Reading temp from HD1 #%d", options.hdNo );
    if( (ret=readHDTemp(options.hdNo,t,NULL,0,FALSE)) == 0 )
    { 
      log( L"Updating HD1 temperature: %d", t );
      m_iHDTemp = glb_iHDTemp = t;
      if( options.fahrenHeit )
        t = options.convertTemp( m_iHDTemp, FAHRENHEIT );
      m_szHDTemp.Format( L"%d %s", t, (options.fahrenHeit ? L" F" : L" C") );
    }
    else
    {
      log( L"HD1 temp reading failed => turned off" );
      options.readHDTemp = FALSE;
    }
  }

  if( !options.readHDTemp )
  {
    m_iHDTemp = glb_iHDTemp = 0;
    m_szHDTemp = _T("---");
  }

  if( glb_iHDTemp > glb_iHDMax )
  {
    glb_iHDMax = glb_iHDTemp;
    ((CWnd*)GetDlgItem(IDC_STATICHDMAX))->SetWindowTextW( (CString)m_szHDTemp );
  }

  if( options.readHDTemp2 )
  {
    log( L"Reading temp from HD2 #%d", options.hdNo2 );
    if( (ret=readHDTemp(options.hdNo2,t,NULL,0,FALSE)) == 0 )
    {
      log( L"Updating HD2 temperature: %d", t );
      m_iHDTemp2 = glb_iHDTemp2 = t;
      if( options.fahrenHeit )
        t = options.convertTemp( m_iHDTemp2, FAHRENHEIT );
      m_szHDTemp2.Format( _T("%d %s"), t, (options.fahrenHeit ? L" F" : L" C") );
    }
    else
    {
      log( L"HD temp #2 reading failed => turned off" );
      options.readHDTemp2 = FALSE;
    }

    if( glb_iHDTemp2 > glb_iHDMax2 )
    {
      glb_iHDMax2 = glb_iHDTemp2;
      ((CWnd*)GetDlgItem(IDC_STATICHDMAX2))->SetWindowTextW( (CString)m_szHDTemp2 );
    }
  }

  if( !options.readHDTemp2 )
  {
    m_iHDTemp2 = glb_iHDTemp2 = 0;
    m_szHDTemp2 = L"---";
  }

  trayMan.displayTrayIcon( NIM_MODIFY );

  UpdateData( FALSE );
}

//---------------------------------------------------------------
// Sysinfo-Dialog anzeigen
//---------------------------------------------------------------
void CI8kfanguiDlg::OnSysinfobutton() 
{
  log( L"vor detectHDs" );
  detectHDs( FALSE );
  log( L"vor CSystemInfo.constructor" );
  CSystemInfo dlgSysInfo;
  log( L"vor CSystemInfo.DoModal()" );
  dlgSysInfo.DoModal();	
}

//---------------------------------------------------------------
// Wechseln der GUI-Lftersteuerung zwischen HW-Sensoren und SMBIOS
//---------------------------------------------------------------
void CI8kfanguiDlg::setFanSliders( BOOL sensorSupport )
{
  CString s1, s2, s3, s4, s5;
  s1.LoadStringW( IDS_OFF );
  s2.LoadStringW( IDS_SLOW );
  s3.LoadStringW( IDS_MED );
  s4.LoadStringW( IDS_HIGH );
  s5.LoadStringW( IDS_MAX );

  ((CWnd*)GetDlgItem(IDC_STATIC1))->SetWindowTextW( sensorSupport ? s1 : s2 );
  ((CWnd*)GetDlgItem(IDC_STATIC2))->SetWindowTextW( sensorSupport ? s3 : s4 );
  ((CWnd*)GetDlgItem(IDC_STATIC3))->SetWindowTextW( sensorSupport ? s5 : s1 );
  ((CWnd*)GetDlgItem(IDC_STATIC4))->SetWindowTextW( sensorSupport ? s1 : s2 );
  ((CWnd*)GetDlgItem(IDC_STATIC5))->SetWindowTextW( sensorSupport ? s3 : s4 );
  ((CWnd*)GetDlgItem(IDC_STATIC6))->SetWindowTextW( sensorSupport ? s5 : s1 );
  ((CSliderCtrl*)GetDlgItem(IDC_SLIDER1))->SetRange( 0, sensorSupport ? 63 : 2, true );
  ((CSliderCtrl*)GetDlgItem(IDC_SLIDER2))->SetRange( 0, sensorSupport ? 63 : 2, true );

  if( m_iSensorIndicator1 > m_scOuterSlider.GetPos() )
    m_iSensorIndicator1 = m_scOuterSlider.GetPos();

  if( m_iSensorIndicator2 > m_scInnerSlider.GetPos() )
    m_iSensorIndicator2 = m_scInnerSlider.GetPos();

  ((CWnd*)GetDlgItem(IDC_EDIT14))->ShowWindow( sensorSupport ? SW_SHOW : SW_HIDE );
  ((CWnd*)GetDlgItem(IDC_EDIT15))->ShowWindow( sensorSupport ? SW_SHOW : SW_HIDE );

  UpdateData( FALSE );
}

//---------------------------------------------------------------
// Ein-/ausblenden der direkten Lfterkontrolle (wegen Observermode/Tempmonitor)
//---------------------------------------------------------------
void CI8kfanguiDlg::hideFanControl( BOOL hide )
{
  log( L"Start von hideFanControl:%d", hide );
  if( hide || options.numFans<=0 )
    m_bOuterFan = FALSE;

  if( hide || options.numFans<=1 )
    m_bInnerFan = FALSE;

  ((CWnd*)GetDlgItem(IDC_CHECK2))->EnableWindow( !hide && options.numFans>0 );
  ((CWnd*)GetDlgItem(IDC_CHECK1))->EnableWindow( !hide && options.numFans>1 );
  ((CWnd*)GetDlgItem(IDC_STATIC29))->EnableWindow( !hide );
  ((CButton*)GetDlgItem(IDC_APPLY))->EnableWindow( m_bOuterFan || m_bInnerFan );

  if( hide || options.numFans==0 )
    m_bMaintainSpeed = FALSE;

  ((CWnd*)GetDlgItem(IDC_MAINTAINSPEED))->EnableWindow( !hide && options.numFans>0 );

  ((CSliderCtrl*)GetDlgItem(IDC_SLIDER2))->EnableWindow( !hide && options.numFans>0 && m_bOuterFan );
  ((CStatic*)GetDlgItem(IDC_STATIC4))->EnableWindow( !hide && options.numFans>0 && m_bOuterFan );
  ((CStatic*)GetDlgItem(IDC_STATIC5))->EnableWindow( !hide && options.numFans>0 && m_bOuterFan );
  ((CStatic*)GetDlgItem(IDC_STATIC6))->EnableWindow( !hide && options.numFans>0 && m_bOuterFan );
  ((CWnd*)GetDlgItem(IDC_EDIT14))->EnableWindow( !hide && options.numFans>0 && m_bOuterFan );

  ((CSliderCtrl*)GetDlgItem(IDC_SLIDER1))->EnableWindow( !hide && options.numFans>1 && m_bInnerFan );
  ((CStatic*)GetDlgItem(IDC_STATIC1))->EnableWindow( !hide && options.numFans>1 && m_bInnerFan );
  ((CStatic*)GetDlgItem(IDC_STATIC2))->EnableWindow( !hide && options.numFans>1 && m_bInnerFan );
  ((CStatic*)GetDlgItem(IDC_STATIC3))->EnableWindow( !hide && options.numFans>1 && m_bInnerFan );
  ((CWnd*)GetDlgItem(IDC_EDIT15))->EnableWindow( !hide && options.numFans>1 && m_bInnerFan );

  UpdateData( FALSE );
  log( L"Ende von hideFanControl" );
}

//---------------------------------------------------------------
// Handler fr 'Close'-Button
//---------------------------------------------------------------
void CI8kfanguiDlg::OnBnClickedClose()
{
  if( options.minimizeOnClose )
  {
    minimizeToTray();
    return;
  }

  OnClose();
}

//---------------------------------------------------------------
// Lokale Harddrives suchen (nur IDE untersttzt)
//---------------------------------------------------------------
void CI8kfanguiDlg::detectHDs( BOOL enSMART )
{
  for( int i=0; i<MAXDRIVES; i++ )      // get SMART status of first 4 drives
  {  
    int temp;
    options.hds[i] = readHDTemp( i, temp, NULL, 0, enSMART );
  }
}

//---------------------------------------------------------------
// Horizontales Scrollen in diversen Controls verarbeiten
//---------------------------------------------------------------
void CI8kfanguiDlg::OnHScroll( UINT nSBCode, UINT nPos, CScrollBar* pScrollBar )
{
  int curpos;
  int newpos;
  int minpos = 0;
  int maxpos;
  int barNo;

  if( pScrollBar == ((CScrollBar*)GetDlgItem(IDC_SLIDER2)) )
  {
    curpos = m_scOuterSlider.GetPos();
    m_iSensorIndicator1 = curpos;
    UpdateData( FALSE );
    CDialog::OnHScroll( nSBCode, nPos, pScrollBar );
    return;
  }
  else if( pScrollBar == ((CScrollBar*)GetDlgItem(IDC_SLIDER1)) )
  {
    curpos = m_scInnerSlider.GetPos();
    m_iSensorIndicator2 = curpos;
    UpdateData( FALSE );
    CDialog::OnHScroll( nSBCode, nPos, pScrollBar );
    return;
  }
  else if( pScrollBar == ((CScrollBar*)GetDlgItem(IDC_SCROLLBAR1)) )
  {
    curpos = pScrollBar->GetScrollPos();

    barNo = 2;
    minpos = 0;
    maxpos = 90;
  }
  else
  {
    CDialog::OnHScroll( nSBCode, nPos, pScrollBar );
    return;
  }

  newpos = curpos;

  switch( nSBCode )
  {
    case SB_THUMBPOSITION:
    case SB_THUMBTRACK:
           newpos = nPos;
           break;

    case SB_LEFT:
           newpos = minpos;
           break;

    case SB_RIGHT:
           newpos = maxpos;
           break;

    case SB_LINELEFT:
    case SB_PAGELEFT:
           if( curpos > minpos )
             newpos = curpos-1;
           break;

    case SB_LINERIGHT:
    case SB_PAGERIGHT:
           if( curpos < maxpos )
             newpos = curpos+1;
           break;

    case SB_ENDSCROLL:
           break;
  }

  if( newpos != curpos )
  {
    pScrollBar->SetScrollPos( newpos, TRUE );
    diagramOffset = (90-newpos) * 12;
    InvalidateRect( &cpuFrame, FALSE );
  }

  CDialog::OnHScroll( nSBCode, nPos, pScrollBar );
}

//---------------------------------------------------------------
// Ein-/ausblenden der Tempmonitor-Elemente
//---------------------------------------------------------------
void CI8kfanguiDlg::showTempMonElements( BOOL x )
{
  //log( L"Start von activeTempmonitorElements: %d", x );
  ((CWnd*)GetDlgItem(IDC_STATIC200))->EnableWindow( x );
  ((CWnd*)GetDlgItem(IDC_STATIC201))->EnableWindow( x );
  ((CWnd*)GetDlgItem(IDC_STATIC801))->EnableWindow( x );
  ((CWnd*)GetDlgItem(IDC_STATIC802))->EnableWindow( x );
  ((CWnd*)GetDlgItem(IDC_STATIC803))->EnableWindow( x );
  ((CWnd*)GetDlgItem(IDC_STATIC804))->EnableWindow( x );
  ((CWnd*)GetDlgItem(IDC_EDIT6))->EnableWindow( x );
  ((CWnd*)GetDlgItem(IDC_EDIT16))->EnableWindow( x );
  ((CWnd*)GetDlgItem(IDC_EDIT17))->EnableWindow( x );
  ((CWnd*)GetDlgItem(IDC_EDIT18))->EnableWindow( x );
  ((CWnd*)GetDlgItem(IDC_EDIT19))->EnableWindow( x );
  //log( L"Ende von activeTempmonitorElements" );
}

//---------------------------------------------------------------
// Tempmonitor-Combobox mit aktuellen Profilnamen fllen
//---------------------------------------------------------------
void CI8kfanguiDlg::fillProfileList()
{
  m_cControlProfile.ResetContent();
  for( int i=0; i<4; i++ )
    m_cControlProfile.AddString( options.ms[i].alias );

  m_cControlProfile.SetCurSel( options.preDefSet );
}

//---------------------------------------------------------------
// Handler fr Profilwechsel ber Combobox auf Hauptfenster
//---------------------------------------------------------------
void CI8kfanguiDlg::OnCbnSelchangeProfile()
{
  int x = m_cControlProfile.GetCurSel();

  log( L"new policy:%d previous:%d", x, options.preDefSet );

  if( x != options.preDefSet )
  {
    RESETLEVELS;
    options.preDefSet = x;
    setRegKeys();
  }

  UpdateData( FALSE );
}

//---------------------------------------------------------------
// Neuzeichnen des Diagramms auslsen (mit vorheriger Lschung)
//---------------------------------------------------------------
void CI8kfanguiDlg::diagramRedraw( BOOL erase )
{
  InvalidateRect( &cpuFrame, erase );
  InvalidateRect( &legendFrame, erase );
}

//---------------------------------------------------------------
// Tooltip zu einem GUI-Element hinzufgen
//---------------------------------------------------------------
void CI8kfanguiDlg::addToolTip( int no, int id, CString str )
{
  CWnd* wnd = GetDlgItem( id );
  CRect rect; 
  wnd->GetClientRect( rect );
  m_tToolTip[no].Create( wnd );
  m_tToolTip[no].AddTool( wnd, str, rect, TOOLTIP_ID );
}

//---------------------------------------------------------------
// Ende der Session
//---------------------------------------------------------------
void CI8kfanguiDlg::OnEndSession( BOOL bEnding )
{
  CDialog::OnEndSession( bEnding );

  log( L"Pending termination" );
  if( bEnding )
    setRegKeys();
}

//---------------------------------------------------------------
// Alias fr Lfter in der GUI setzen
//---------------------------------------------------------------
void CI8kfanguiDlg::setFanAlias()
{
  CString s1, s2, s3;
  s1.LoadStringW( IDS_DLG61 );
  s2.LoadStringW( IDS_DLG62 );

  s3.Format( s1, options.fan1Alias );
  ((CWnd*)GetDlgItem(IDC_CHECK2))->SetWindowText( s3 );
  s3.Format( s1, options.fan2Alias );
  ((CWnd*)GetDlgItem(IDC_CHECK1))->SetWindowText( s3 );

  s3.Format( s2, options.fan1Alias );
  ((CWnd*)GetDlgItem(IDC_FANGROUP1))->SetWindowText( s3 );
  s3.Format( s2, options.fan2Alias );
  ((CWnd*)GetDlgItem(IDC_FANGROUP2))->SetWindowText( s3 );
}

//---------------------------------------------------------------
// Alias fr Lfter mit ersten Zeichen in Grobuchstaben generieren (z.B. 'first' -> 'First')
//---------------------------------------------------------------
void CI8kfanguiDlg::createUpperFanAlias()
{
  glb_szFan1AliasUpper = options.fan1Alias;
  glb_szFan1AliasUpper.SetAt( 0, towupper(options.fan1Alias.GetAt(0)) );

  glb_szFan2AliasUpper = options.fan2Alias;
  glb_szFan2AliasUpper.SetAt( 0, towupper(options.fan2Alias.GetAt(0)) );
}

//---------------------------------------------------------------
// GUI-Steuerung der Optionen fr die Lfter (ein-/ausblenden)
//---------------------------------------------------------------
void CI8kfanguiDlg::maintainFanNumLimits()
{
  if( options.numFans < 2 )
  {
    // Deaktivierung aller Einstellungen fr Lfter #2
    
    // 1. Tempmonitor:
    for( int i=0; i<4; i++ )
    {
      options.ms[i].cpuFan[1] = FALSE;
      options.ms[i].gpuFan[1] = FALSE;
      options.ms[i].memoryFan[1] = FALSE;
      options.ms[i].chipsetFan[1] = FALSE;
    }

    // 2. Statuslesen deaktivieren:
    options.readFanStatus2 = FALSE;
    options.readFanSpeed2 = FALSE;

    // 3. Autostart fr Lfter 2 deaktivieren:
    options.autoInnerSlider = 0;
    options.autoSpeedDirect2 = 0;
  }

  if( options.numFans < 1 )
  {
    // Deaktivierung aller Einstellungen fr Lfter #1
    
    // 1. Tempmonitor:
    for( int i=0; i<4; i++ )
    {
      options.ms[i].cpuFan[0] = FALSE;
      options.ms[i].gpuFan[0] = FALSE;
      options.ms[i].memoryFan[0] = FALSE;
      options.ms[i].chipsetFan[0] = FALSE;
    }

    // 2. Statuslesen deaktivieren:
    options.readFanStatus1 = FALSE;
    options.readFanSpeed1 = FALSE;

    // 3. Autostart fr Lfter 1 deaktivieren:
    options.autoOuterSlider = 0;
    options.autoSpeedDirect1 = 0;
  }
}

//---------------------------------------------------------------
// Anpassung der Diagrammskala
//---------------------------------------------------------------
void CI8kfanguiDlg::setDiagramScale()
{
  ((CWnd*)GetDlgItem(IDC_SCALE150))->SetWindowTextW( options.fahrenHeit ? L"300" : L"150" );
  ((CWnd*)GetDlgItem(IDC_SCALE100))->SetWindowTextW( options.fahrenHeit ? L"200" : L"100" );
  ((CWnd*)GetDlgItem(IDC_SCALE50))->SetWindowTextW( options.fahrenHeit ? L"100" : L"50" );
}

//---------------------------------------------------------------
// Handler fr Reset der Peaks
//---------------------------------------------------------------
void CI8kfanguiDlg::OnBnClickedPeakbutton()
{
  glb_iCPUMax = 0;
  glb_iGPUMax = 0;
  glb_iMemoryMax = 0;
  glb_iChipsetMax = 0;
  glb_iHDMax = 0;
  glb_iHDMax2 = 0;

  ((CWnd*)GetDlgItem(IDC_STATICCPUMAX))->SetWindowTextW( (CString)L"---" );
  ((CWnd*)GetDlgItem(IDC_STATICGPUMAX))->SetWindowTextW( (CString)L"---" );
  ((CWnd*)GetDlgItem(IDC_STATICMEMMAX))->SetWindowTextW( (CString)L"---" );
  ((CWnd*)GetDlgItem(IDC_STATICCHIPMAX))->SetWindowTextW( (CString)L"---" );
  ((CWnd*)GetDlgItem(IDC_STATICHDMAX))->SetWindowTextW( (CString)L"---" );
  ((CWnd*)GetDlgItem(IDC_STATICHDMAX2))->SetWindowTextW( (CString)L"---" );

  readDataFromSensors();
  updateHDTemp();
}

//---------------------------------------------------------------
// Ausfhrung des Highspeed-Fanforcings durch 'OnTimer'
//---------------------------------------------------------------
void CI8kfanguiDlg::fanForcing()
{
#ifdef TIMERINFO
  static int nr=0;
  CString text;

  text.Format( L"#%d MFS:%d SENS:%d 1:%d 2:%d", 
               nr++, 
               options.maintainSpeeds,
               glb_bSensorFanForcing, 
               glb_iFanForcingValue1, 
               glb_iFanForcingValue2 );

  ((CWnd*)GetDlgItem(IDC_FORCEINFO))->SetWindowTextW( text );
#endif

  if( options.maintainSpeeds )
    tempControl.maintainSpeeds();
  else if( glb_bSensorFanForcing )
  {
    if( glb_iFanForcingValue1 != -1 )
      setFanSpeedSensor( options.fan1SensorIndex, glb_iFanForcingValue1, glb_hHndFile );

    if( glb_iFanForcingValue2 != -1 )
      setFanSpeedSensor( options.fan2SensorIndex, glb_iFanForcingValue2, glb_hHndFile );
  }
  else
  {
    if( glb_iFanForcingValue1 != -1 )
      setFanSpeed( options.fan1Index, glb_iFanForcingValue1, glb_hHndFile );

    if( glb_iFanForcingValue2 != -1 )
      setFanSpeed( options.fan2Index, glb_iFanForcingValue2, glb_hHndFile );
  }
}

//---------------------------------------------------------------
// Handler fr 'Maintain minimum fan speed'-Button
//---------------------------------------------------------------
void CI8kfanguiDlg::OnBnClickedMaintainspeed()
{
  UpdateData();

  if( m_bMaintainSpeed )
  {
    glb_bActiveFanForcing = (glb_bDirectSpeedApplied ? true : false);

    if( glb_bDirectSpeedApplied )
    {
      glb_bSensorFanForcing = (glb_iDirectSpeedMethod!=0);
      glb_iFanForcingValue1 = glb_iDirectSpeed1;
      glb_iFanForcingValue2 = glb_iDirectSpeed2;
    }
  }
  else
    glb_bActiveFanForcing = false;

  options.maintainSpeeds = m_bMaintainSpeed;

  checkTimerInterval();

  UpdateData( FALSE );
}

//---------------------------------------------------------------
// Timerintervall prfen ung ggf. anpassen
//---------------------------------------------------------------
void CI8kfanguiDlg::checkTimerInterval()
{
  // evtl. Timerintervall ndern:
  int timerNew = glb_iEffectiveTimerBase;

  // FanForce oder Tempmonitor aktiv:
  if( options.opmode==FANFORCING || options.opmode==TEMPMONITOR || options.maintainSpeeds )
    timerNew = fanForceTimeout[options.fanForceInterval];
  else    // nicht aktiv => auf Default 1000 zurcksetzen
    timerNew = 1000;

  if( timerNew != glb_iEffectiveTimerBase )
  {
    glb_iEffectiveTimerBase = timerNew;

    log( L"/bSwitching to new timer interval %d... ", timerNew );
    if( KillTimer(timerID) == 0 )
      log( L"/afailed; Timer %d not found !?", timerID );

    if( SetTimer(timerID,glb_iEffectiveTimerBase,NULL) != timerID )
    {
      log( L"/afailed" );
      CString s1, s2;
      s1.LoadStringW( IDS_DLG63 );
      s2.LoadStringW( IDS_ERROR );
      MessageBoxW( s1, s2, MB_ICONSTOP|MB_OK );
    }
    else
      log( L"/aOk" );
  }
}

//---------------------------------------------------------------
// Historylog schreiben
//---------------------------------------------------------------
void CI8kfanguiDlg::OnBnClickedLogbutton()
{
  CString s1;
  s1.LoadStringW( IDS_DLG64 );
  CFileDialog* fd = new CFileDialog( FALSE, 
                                     NULL, 
                                     NULL,
                                     OFN_HIDEREADONLY, 
                                     s1,
                                     this );
  if( fd->DoModal() != IDOK )
  {
    delete fd;
    return;
  }

  FILE* f;

  if( _wfopen_s(&f,fd->GetPathName(),L"wt") )
  {
    delete fd;
    CString s2;
    s1.LoadStringW( IDS_DLG65 );
    s2.LoadStringW( IDS_SERROR );
    MessageBoxW( s1, s2, MB_ICONEXCLAMATION|MB_OK );
    return;
  }

  delete fd;

  int x = glb_iDiagramIndex;
  int run = 0;

  time_t zeit = time( NULL );
  struct tm ts;
  int err = localtime_s( &ts, &zeit );

  s1.LoadStringW( IDS_DLG66 );
  fwprintf( f, s1, "---" );
  s1.LoadStringW( IDS_DLG67 );
  fwprintf( f, s1, options.fahrenHeit ? "F" : "C" );
  s1.LoadStringW( IDS_DLG68 );
  fwprintf( f, s1 );

  int value[8];

  // 1200 Werte sind maximal vorhanden (0-1199)
  while( run < 1200 )
  {
    // falls der Arrayzhler mal bers Ziel hinausschiet, einfach zurcksetzen:
    if( x >= 1200 ) x = 0;

    value[0]=glb_iaTempArr[x];  // CPU-Temperatur
    if( value[0] && options.fahrenHeit ) value[0] = options.convertTemp( value[0], FAHRENHEIT );

    value[1]=glb_iaGPUArr[x];  // GPU-Temperatur
    if( value[1] && options.fahrenHeit ) value[1] = options.convertTemp( value[1], FAHRENHEIT );

    value[2]=glb_iaMemArr[x];  // Memory-Temperatur
    if( value[2] && options.fahrenHeit ) value[2] = options.convertTemp( value[2], FAHRENHEIT );

    value[3]=glb_iaChipArr[x];  // Chipset-Temperatur
    if( value[3] && options.fahrenHeit ) value[3] = options.convertTemp( value[3], FAHRENHEIT );

    value[4]=glb_iaSpeedArr[x]*100;  // CPU-Speed
    value[5]=glb_iaLoadArr[x];  // CPU-Last
    value[6]=glb_iaFan1Speed[x];
    value[7]=glb_iaFan2Speed[x];

    fwprintf( f, L"%3d %3d %3d %3d %4d %3d%% %5d %5d\n", 
              value[0], value[1], value[2], value[3], 
              value[4], value[5], value[6], value[7] );
    
    x++;
    run++;
  }

  fclose( f );
}

//---------------------------------------------------------------
// History lschen
//---------------------------------------------------------------
void CI8kfanguiDlg::OnBnClickedResethistory()
{
  for( int i=0; i<1200; i++ )
  {
    glb_iaTempArr[i] = glb_iaGPUArr[i] = glb_iaMemArr[i] = glb_iaChipArr[i] = 
    glb_iaSpeedArr[i] = glb_iaLoadArr[i] = glb_iaFan1Speed[i] = glb_iaFan2Speed[i] = 0;
  }

  diagramRedraw( TRUE );
}

//---------------------------------------------------------------
// Observer Mode einschalten
//---------------------------------------------------------------
void CI8kfanguiDlg::OnBnClickedRadio1()
{
  if( inPopupCreation )
    return;

  log( L"Start von ClickedRadio1 (Observer)" );
  if( options.opmode == TEMPMONITOR )
    exitTempMonitor();

  options.opmode = OBSERVERMODE;

  showTempMonElements( FALSE );
  RESETLEVELS;

  m_bMaintainSpeed = FALSE;
  options.maintainSpeeds = FALSE;

  glb_bActiveFanForcing = false;

  //showTempControl();
  hideFanControl( TRUE );
  checkTimerInterval();
  setRegKeys();
  UpdateData( FALSE );
  log( L"Ende von ClickedRadio1 (Observer)" );
}

//---------------------------------------------------------------
// 'Manual fan control' einschalten
//---------------------------------------------------------------
void CI8kfanguiDlg::OnBnClickedRadio2()
{
  if( inPopupCreation )
    return;

  log( L"Start von ClickedRadio2 (Manual)" );
  if( options.opmode == TEMPMONITOR )
    exitTempMonitor();

  options.opmode = MANUALCONTROL;
  RESETLEVELS;

  showTempMonElements( FALSE );

  glb_bActiveFanForcing = false;

  hideFanControl( FALSE );
  checkTimerInterval();  
  setRegKeys();
  log( L"Ende von ClickedRadio2 (Manual)" );
}

//---------------------------------------------------------------
// 'Automatic temperature control' einschalten
//---------------------------------------------------------------
void CI8kfanguiDlg::OnBnClickedRadio4()
{
  if( inPopupCreation )
    return;

  log( L"Start von ClickedRadio4 (Tempmonitor)" );
  if( options.intervalCPU >= 4 )
  {
    options.intervalCPU = 2;
    CString s1, s2;
    s1.LoadStringW( IDS_DLG69 );
    s2.LoadStringW( IDS_IWARN );
    MessageBox( s1, s2, MB_ICONINFORMATION|MB_OK );
  }

  options.opmode = TEMPMONITOR;
  RESETLEVELS;

  showTempMonElements( TRUE );

  glb_bActiveFanForcing = false;

  m_bMaintainSpeed = FALSE;
  options.maintainSpeeds = FALSE;

  hideFanControl( TRUE );
  checkTimerInterval();  
  setRegKeys();
  UpdateData( FALSE );
  log( L"Ende von ClickedRadio4 (Tempmonitor)" );
}

//---------------------------------------------------------------
// 'Fan forcing to high speed' einschalten
//---------------------------------------------------------------
void CI8kfanguiDlg::OnBnClickedRadio3()
{
  if( inPopupCreation )
    return;

  log( L"Start von ClickedRadio3 (Fan forcing)" );
  if( options.opmode == TEMPMONITOR )
    exitTempMonitor();

  options.opmode = FANFORCING;
  RESETLEVELS;

  showTempMonElements( FALSE );

  m_bMaintainSpeed = FALSE;
  options.maintainSpeeds = FALSE;

  glb_bActiveFanForcing = true;
  glb_bSensorFanForcing = false;
  glb_iFanForcingValue1 = (options.numFans>0 && options.fan1Force) ? 2 : -1;
  glb_iFanForcingValue2 = (options.numFans>1 && options.fan2Force) ? 2 : -1;
  tickCounterFanForce = 1000000;    // sehr hoch setzen, damit gleich ein Timeout ausgelst wird

  hideFanControl( TRUE );
  checkTimerInterval();
  setRegKeys();
  UpdateData( FALSE );
  log( L"Ende von ClickedRadio3 (Fan forcing)" );
}

//---------------------------------------------------------------
// 'Automatic temp control' beenden wenn aktiviert
//---------------------------------------------------------------
void CI8kfanguiDlg::exitTempMonitor()
{
  if( glb_iCPUTemp<=60 && (!options.readGPUTemp || glb_iGPUTemp<55) )
  {
    log( L"Disabling temp monitor; CPU Temp:%d, GPU Temp:%d", glb_iCPUTemp, glb_iGPUTemp );
    setFanSpeed( options.fan1Index, 0, glb_hHndFile );
    setFanSpeed( options.fan2Index, 0, glb_hHndFile );
  }
}

//---------------------------------------------------------------
// Setzt die Textfarben der Verlauflegende
//---------------------------------------------------------------
void CI8kfanguiDlg::setLegendColors()
{
  COLORREF back;

  back = (options.colorTheme==0) ? BLACK : WHITE;

  m_cCPUTemp.SetBkColor( back );
  m_cGPUTemp.SetBkColor( back );
  m_cMemoryTemp.SetBkColor( back );
  m_cChipsetTemp.SetBkColor( back );
  m_cCPUSpeed.SetBkColor( back );
  m_cCPULoad.SetBkColor( back );

  m_cCPUTemp.SetTextColor( (options.colorTheme==0) ? RGB(255,255,0) : RGB(0,0,255) );
  m_cCPULoad.SetTextColor( (options.colorTheme==0) ? RGB(255,255,255) : RGB(0,0,0) );
}
